export var CallReportingStatus;
(function (CallReportingStatus) {
    CallReportingStatus["Abandoned"] = "abandoned";
    CallReportingStatus["Cancel"] = "cancel";
    CallReportingStatus["Success"] = "success";
})(CallReportingStatus || (CallReportingStatus = {}));
export var CallActions;
(function (CallActions) {
    CallActions["Ringing"] = "ringing";
    CallActions["Active"] = "active";
    CallActions["Bridge"] = "bridge";
    CallActions["Hold"] = "hold";
    CallActions["DTMF"] = "dtmf";
    CallActions["Voice"] = "voice";
    CallActions["Silence"] = "silence";
    CallActions["Execute"] = "execute";
    CallActions["Update"] = "update";
    CallActions["Hangup"] = "hangup";
    CallActions["Reporting"] = "reporting";
    CallActions["PeerStream"] = "peerStream";
    CallActions["LocalStream"] = "localStream";
    CallActions["Destroy"] = "destroy";
})(CallActions || (CallActions = {}));
export var CallDirection;
(function (CallDirection) {
    CallDirection["Inbound"] = "inbound";
    CallDirection["Outbound"] = "outbound";
})(CallDirection || (CallDirection = {}));
export class Call {
    constructor(client, e) {
        this.client = client;
        // FIXME check _muted from channel
        const callInfo = e.data;
        this._muted = false;
        this.voice = true;
        this.createdAt = +e.timestamp;
        this.task = null;
        this.data = null;
        this.answeredAt = 0;
        this.hangupAt = 0;
        this.bridgedAt = 0;
        this.reportingAt = 0;
        this.autoAnswered = false;
        this.peerStreams = [];
        this.localStreams = [];
        // fixme
        if (client.phone) {
            if (callInfo.sip_id) {
                this.setSip(client.phone.sipSessionBySipId(callInfo.sip_id));
            }
            else {
                this.setSip(client.phone.sipSessionByCallId(e.id));
            }
        }
        this.params = {};
        this.id = e.id;
        this.digits = [];
        this.applications = [];
        this.appId = e.app_id;
        this.setState(e);
        this.setInfo(callInfo);
        if (this.queue && this.client.agent) {
            this.task = this.client.agent.task.get(+this.queue.attempt_id) || null;
        }
    }
    // set private
    setState(s) {
        this.state = s.event;
    }
    setSip(sip) {
        if (sip && !this.sip) {
            this.sip = sip;
            this.localStreams = sip.getLocalMedia();
            this.peerStreams = sip.getPeerMedia();
        }
    }
    get duration() {
        if (this.hangupAt === 0) {
            return Math.round((Date.now() - this.createdAt) / 1000); // tslint:disable-line
        }
        else {
            return Math.round((this.hangupAt - this.createdAt) / 1000); // tslint:disable-line
        }
    }
    get allowDtmf() {
        return this.answeredAt > 0 && this.allowHangup;
    }
    get allowHangup() {
        return this.hangupAt === 0;
    }
    get allowHold() {
        return (this.hangupAt === 0 &&
            (this.state === 'active' || this.state === 'bridge'));
    }
    get allowAnswer() {
        return (this.client.phone &&
            this.hangupAt === 0 &&
            this.answeredAt === 0 &&
            (this.queue ||
                this.direction === CallDirection.Inbound ||
                (this.params && !this.params.autoAnswer)));
    }
    get allowUnHold() {
        return this.hangupAt === 0 && this.state === 'hold';
    }
    get memberCommunication() {
        if (!this.task) {
            return null;
        }
        else {
            return this.task.communication;
        }
    }
    // todo
    async getMember(req) {
        if (!this.isMember) {
            throw new Error(`call is not from outbound queue`);
        }
        return this.client.request(`cc_member_page`, {
            queue_id: this.task.queueId,
            member_id: this.task.memberId,
        });
    }
    get isMember() {
        return !!(this.task && this.task.memberId);
    }
    setActive(e) {
        if (!this.answeredAt) {
            if (this.direction === CallDirection.Inbound) {
                this.bridgedAt = +e.timestamp;
                if (this.parentId) {
                    this.bridgedId = this.parentId;
                }
            }
            this.answeredAt = +e.timestamp;
        }
        this.setState(e);
    }
    setBridged(s) {
        const bridged = s.data;
        if (!this.bridgedAt) {
            this.bridgedAt = +s.timestamp;
        }
        this.bridgedId = bridged.bridged_id;
        if (bridged.to) {
            if (this.direction === CallDirection.Inbound) {
                this.from = bridged.to;
            }
            else {
                this.to = bridged.to;
            }
        }
        this.bridgedId = bridged.bridged_id;
        if (bridged.payload) {
            this.variables = bridged.payload;
        }
        if (bridged.queue) {
            this.queue = bridged.queue;
        }
    }
    setHold(e) {
        this.setState(e);
    }
    setInfo(s) {
        this.parentId = s.parent_id;
        this.destination = s.destination;
        this.direction = s.direction;
        this.from = s.from;
        this.to = s.to;
        this.variables = s.payload;
        this.queue = s.queue || null;
        this.sipId = s.sip_id || null;
        this.params = s.params;
    }
    setPeerStreams(streams) {
        if (!streams) {
            this.peerStreams = [];
        }
        else {
            this.peerStreams = streams;
        }
    }
    setLocalStreams(streams) {
        if (!streams) {
            this.localStreams = [];
        }
        else {
            this.localStreams = streams;
        }
    }
    setVoice() {
        this.voice = true;
    }
    setSilence() {
        this.voice = false;
    }
    setExecute(s) {
        this.applications.push(s.application);
    }
    addDigit(s) {
        this.digits.push(s.digit);
    }
    setHangup(s) {
        const hangup = s.data;
        this.hangupAt = +s.timestamp;
        this.hangupCause = hangup.cause;
        this.hangupSipCode = hangup.sip;
        this.voice = false;
        this.peerStreams = [];
        if (+hangup.reporting_at) {
            this.reportingAt = +hangup.reporting_at; // FIXME type number
        }
        this.setState(s);
    }
    get allowReporting() {
        return this.answeredAt > 0 && this.hasReporting;
    }
    get hasReporting() {
        if (this.task) {
            return this.task.hasReporting;
        }
        return false;
    }
    get display() {
        return `${this.displayNumber} (${this.displayName})`;
    }
    get muted() {
        return this._muted;
    }
    get isHold() {
        return this.state === CallActions.Hold;
    }
    get allowInboundVideo() {
        if (this.params) {
            return this.params.video || false;
        }
        return false;
    }
    get allowOutboundVideo() {
        if (this.params) {
            return this.params.video || false;
        }
        return false;
    }
    get active() {
        return this.hangupAt === 0;
    }
    get displayNumber() {
        if (this.direction === 'inbound') {
            return this.from.number;
        }
        else {
            if (this.to) {
                return this.to.number;
            }
            return this.destination;
        }
    }
    get displayName() {
        if (this.direction === 'inbound') {
            return this.from.name;
        }
        else {
            if (this.to) {
                return this.to.name;
            }
            return this.destination;
        }
    }
    get autoAnswer() {
        return ((this.queue && this.queue.queue_type === 'offline') ||
            (this.params && this.params.autoAnswer === true));
    }
    /* Call control */
    async answer(req) {
        if (this.sip && this.client.phone) {
            const params = await this.client.phone.callOption(req);
            await this.sip.answer(params);
            return true;
        }
        return false;
    }
    answerDelay(req, delay) {
        if (this.autoAnswered) {
            return;
        }
        this.autoAnswered = true;
        setTimeout(async () => {
            await this.answer(req);
        }, delay);
    }
    async hangup(cause) {
        let _cause = cause;
        if (this.answeredAt === 0 && !cause) {
            _cause =
                this.direction === CallDirection.Inbound
                    ? 'USER_BUSY'
                    : 'ORIGINATOR_CANCEL';
        }
        return this.client.request('call_hangup', {
            id: this.id,
            app_id: this.appId,
            cause: _cause,
        });
    }
    async eavesdrop(req) {
        req.id = this.id;
        return this.client.request('call_eavesdrop', req);
    }
    async toggleHold() {
        if (this.state === CallActions.Hold) {
            return this.unHold();
        }
        else {
            return this.hold();
        }
    }
    async hold() {
        if (this.state === CallActions.Hold) {
            throw new Error('Call is hold');
        }
        return this.client.request('call_hold', {
            id: this.id,
            app_id: this.appId,
        });
    }
    async unHold() {
        if (this.state !== CallActions.Hold) {
            throw new Error('Call is active');
        }
        return this.client.request('call_unhold', {
            id: this.id,
            app_id: this.appId,
        });
    }
    async sendDTMF(dtmf) {
        return this.client.request('call_dtmf', {
            id: this.id,
            app_id: this.appId,
            dtmf,
        });
    }
    async blindTransfer(destination) {
        return this.client.request('call_blind_transfer', {
            id: this.id,
            app_id: this.appId,
            destination,
        });
    }
    async mute(mute = false) {
        const res = await this.client.request('call_mute', {
            id: this.id,
            app_id: this.appId,
            mute,
        });
        this._muted = mute;
        return res;
    }
    async bridgeTo(call) {
        return this.client.request('call_bridge', {
            from_id: this.id,
            from_app_id: this.appId,
            to_id: call.id,
            to_app_id: call.appId,
        });
    }
    async reporting(reporting) {
        if (!this.task) {
            throw new Error(`this call not in queue`);
        }
        return this.task.reporting(reporting);
    }
    async renew(sec) {
        if (!this.task) {
            throw new Error(`this conversation not in queue`);
        }
        return this.task.renew(sec);
    }
    async callToUser(req) {
        req.nodeId = this.appId;
        req.parentCallId = this.id || null;
        req.sendToCallId = this.parentId || null;
        return this.client.inviteToUser(req);
    }
}
//# sourceMappingURL=call.js.map