import { parse, format } from 'url';
import globalAxios from 'axios';
import { EventEmitter } from 'ee-ts';
import { debug, WebSocketInterface, UA } from 'jssip';

// tslint:disable
const BASE_PATH = 'https://dev.webitel.com/api'.replace(/\/+$/, '');
/**
 *
 * @export
 * @class BaseAPI
 */
class BaseAPI {
    constructor(configuration, basePath = BASE_PATH, axios = globalAxios) {
        this.basePath = basePath;
        this.axios = axios;
        if (configuration) {
            this.configuration = configuration;
            this.basePath = configuration.basePath || this.basePath;
        }
    }
}
/**
 *
 * @export
 * @class RequiredError
 * @extends {Error}
 */
class RequiredError extends Error {
    constructor(field, msg) {
        super(msg);
        this.field = field;
        this.name = 'RequiredError';
    }
}

// tslint:disable
/**
 * AgentPauseCauseServiceApi - axios parameter creator
 * @export
 */
const AgentPauseCauseServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @param {EngineCreateAgentPauseCauseRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createAgentPauseCause: async (body, options = {}) => {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createAgentPauseCause.');
            }
            const localVarPath = `/call_center/pause_causes`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteAgentPauseCause: async (id, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteAgentPauseCause.');
            }
            const localVarPath = `/call_center/pause_causes/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} id
         * @param {EnginePatchAgentPauseCauseRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchAgentPauseCause: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling patchAgentPauseCause.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling patchAgentPauseCause.');
            }
            const localVarPath = `/call_center/pause_causes/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readAgentPauseCause: async (id, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readAgentPauseCause.');
            }
            const localVarPath = `/call_center/pause_causes/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentPauseCause: async (page, size, q, sort, fields, id, options = {}) => {
            const localVarPath = `/call_center/pause_causes`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} id
         * @param {EngineUpdateAgentPauseCauseRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateAgentPauseCause: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateAgentPauseCause.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateAgentPauseCause.');
            }
            const localVarPath = `/call_center/pause_causes/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * AgentPauseCauseServiceApi - functional programming interface
 * @export
 */
const AgentPauseCauseServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @param {EngineCreateAgentPauseCauseRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createAgentPauseCause(body, options) {
            const localVarAxiosArgs = await AgentPauseCauseServiceApiAxiosParamCreator(configuration).createAgentPauseCause(body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteAgentPauseCause(id, options) {
            const localVarAxiosArgs = await AgentPauseCauseServiceApiAxiosParamCreator(configuration).deleteAgentPauseCause(id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} id
         * @param {EnginePatchAgentPauseCauseRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async patchAgentPauseCause(id, body, options) {
            const localVarAxiosArgs = await AgentPauseCauseServiceApiAxiosParamCreator(configuration).patchAgentPauseCause(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readAgentPauseCause(id, options) {
            const localVarAxiosArgs = await AgentPauseCauseServiceApiAxiosParamCreator(configuration).readAgentPauseCause(id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchAgentPauseCause(page, size, q, sort, fields, id, options) {
            const localVarAxiosArgs = await AgentPauseCauseServiceApiAxiosParamCreator(configuration).searchAgentPauseCause(page, size, q, sort, fields, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} id
         * @param {EngineUpdateAgentPauseCauseRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateAgentPauseCause(id, body, options) {
            const localVarAxiosArgs = await AgentPauseCauseServiceApiAxiosParamCreator(configuration).updateAgentPauseCause(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * AgentPauseCauseServiceApi - factory interface
 * @export
 */
const AgentPauseCauseServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @param {EngineCreateAgentPauseCauseRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createAgentPauseCause(body, options) {
            return AgentPauseCauseServiceApiFp(configuration)
                .createAgentPauseCause(body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteAgentPauseCause(id, options) {
            return AgentPauseCauseServiceApiFp(configuration)
                .deleteAgentPauseCause(id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} id
         * @param {EnginePatchAgentPauseCauseRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchAgentPauseCause(id, body, options) {
            return AgentPauseCauseServiceApiFp(configuration)
                .patchAgentPauseCause(id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readAgentPauseCause(id, options) {
            return AgentPauseCauseServiceApiFp(configuration)
                .readAgentPauseCause(id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentPauseCause(page, size, q, sort, fields, id, options) {
            return AgentPauseCauseServiceApiFp(configuration)
                .searchAgentPauseCause(page, size, q, sort, fields, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} id
         * @param {EngineUpdateAgentPauseCauseRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateAgentPauseCause(id, body, options) {
            return AgentPauseCauseServiceApiFp(configuration)
                .updateAgentPauseCause(id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * AgentPauseCauseServiceApi - object-oriented interface
 * @export
 * @class AgentPauseCauseServiceApi
 * @extends {BaseAPI}
 */
class AgentPauseCauseServiceApi extends BaseAPI {
    /**
     *
     * @param {EngineCreateAgentPauseCauseRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentPauseCauseServiceApi
     */
    createAgentPauseCause(body, options) {
        return AgentPauseCauseServiceApiFp(this.configuration)
            .createAgentPauseCause(body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} id
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentPauseCauseServiceApi
     */
    deleteAgentPauseCause(id, options) {
        return AgentPauseCauseServiceApiFp(this.configuration)
            .deleteAgentPauseCause(id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} id
     * @param {EnginePatchAgentPauseCauseRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentPauseCauseServiceApi
     */
    patchAgentPauseCause(id, body, options) {
        return AgentPauseCauseServiceApiFp(this.configuration)
            .patchAgentPauseCause(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} id
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentPauseCauseServiceApi
     */
    readAgentPauseCause(id, options) {
        return AgentPauseCauseServiceApiFp(this.configuration)
            .readAgentPauseCause(id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentPauseCauseServiceApi
     */
    searchAgentPauseCause(page, size, q, sort, fields, id, options) {
        return AgentPauseCauseServiceApiFp(this.configuration)
            .searchAgentPauseCause(page, size, q, sort, fields, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} id
     * @param {EngineUpdateAgentPauseCauseRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentPauseCauseServiceApi
     */
    updateAgentPauseCause(id, body, options) {
        return AgentPauseCauseServiceApiFp(this.configuration)
            .updateAgentPauseCause(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * AgentServiceApi - axios parameter creator
 * @export
 */
const AgentServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @param {number} agentId
         * @param {EngineAgentSetStateRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        agentSetState: async (agentId, body, options = {}) => {
            // verify required parameter 'agentId' is not null or undefined
            if (agentId === null || agentId === undefined) {
                throw new RequiredError('agentId', 'Required parameter agentId was null or undefined when calling agentSetState.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling agentSetState.');
            }
            const localVarPath = `/call_center/agents/{agent_id}/states/waiting`.replace(`{${'agent_id'}}`, encodeURIComponent(String(agentId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {string} agentId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [timeFrom]
         * @param {string} [timeTo]
         * @param {string} [q]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        agentStateHistory: async (agentId, page, size, timeFrom, timeTo, q, domainId, options = {}) => {
            // verify required parameter 'agentId' is not null or undefined
            if (agentId === null || agentId === undefined) {
                throw new RequiredError('agentId', 'Required parameter agentId was null or undefined when calling agentStateHistory.');
            }
            const localVarPath = `/call_center/agents/{agent_id}/states/history`.replace(`{${'agent_id'}}`, encodeURIComponent(String(agentId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (timeFrom !== undefined) {
                localVarQueryParameter['time_from'] = timeFrom;
            }
            if (timeTo !== undefined) {
                localVarQueryParameter['time_to'] = timeTo;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Create Agent
         * @param {EngineCreateAgentRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createAgent: async (body, options = {}) => {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createAgent.');
            }
            const localVarPath = `/call_center/agents`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Remove Agent
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteAgent: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteAgent.');
            }
            const localVarPath = `/call_center/agents/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update Agent
         * @param {string} id
         * @param {EnginePatchAgentRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchAgent: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling patchAgent.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling patchAgent.');
            }
            const localVarPath = `/call_center/agents/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Agent item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readAgent: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readAgent.');
            }
            const localVarPath = `/call_center/agents/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List of Agent
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<string>} [id]
         * @param {Array<string>} [allowChannels]
         * @param {Array<number>} [supervisorId]
         * @param {Array<number>} [teamId]
         * @param {Array<number>} [regionId]
         * @param {Array<number>} [auditorId]
         * @param {boolean} [isSupervisor]
         * @param {Array<number>} [skillId]
         * @param {Array<number>} [queueId]
         * @param {boolean} [notSupervisor]
         * @param {Array<string>} [userId]
         * @param {Array<string>} [extension]
         * @param {Array<number>} [notTeamId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgent: async (page, size, q, sort, fields, id, allowChannels, supervisorId, teamId, regionId, auditorId, isSupervisor, skillId, queueId, notSupervisor, userId, extension, notTeamId, options = {}) => {
            const localVarPath = `/call_center/agents`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            if (allowChannels) {
                localVarQueryParameter['allow_channels'] = allowChannels;
            }
            if (supervisorId) {
                localVarQueryParameter['supervisor_id'] = supervisorId;
            }
            if (teamId) {
                localVarQueryParameter['team_id'] = teamId;
            }
            if (regionId) {
                localVarQueryParameter['region_id'] = regionId;
            }
            if (auditorId) {
                localVarQueryParameter['auditor_id'] = auditorId;
            }
            if (isSupervisor !== undefined) {
                localVarQueryParameter['is_supervisor'] = isSupervisor;
            }
            if (skillId) {
                localVarQueryParameter['skill_id'] = skillId;
            }
            if (queueId) {
                localVarQueryParameter['queue_id'] = queueId;
            }
            if (notSupervisor !== undefined) {
                localVarQueryParameter['not_supervisor'] = notSupervisor;
            }
            if (userId) {
                localVarQueryParameter['user_id'] = userId;
            }
            if (extension) {
                localVarQueryParameter['extension'] = extension;
            }
            if (notTeamId) {
                localVarQueryParameter['not_team_id'] = notTeamId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [timeFrom]
         * @param {string} [timeTo]
         * @param {Array<number>} [agentId]
         * @param {string} [domainId]
         * @param {string} [q]
         * @param {Array<string>} [fields]
         * @param {string} [sort]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentCallStatistics: async (page, size, timeFrom, timeTo, agentId, domainId, q, fields, sort, options = {}) => {
            const localVarPath = `/call_center/agents/reports/call`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (timeFrom !== undefined) {
                localVarQueryParameter['time.from'] = timeFrom;
            }
            if (timeTo !== undefined) {
                localVarQueryParameter['time.to'] = timeTo;
            }
            if (agentId) {
                localVarQueryParameter['agent_id'] = agentId;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {string} id
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentInQueue: async (id, page, size, q, sort, fields, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling searchAgentInQueue.');
            }
            const localVarPath = `/call_center/agents/{id}/queues`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {string} agentId
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentInQueueStatistics: async (agentId, domainId, options = {}) => {
            // verify required parameter 'agentId' is not null or undefined
            if (agentId === null || agentId === undefined) {
                throw new RequiredError('agentId', 'Required parameter agentId was null or undefined when calling searchAgentInQueueStatistics.');
            }
            const localVarPath = `/call_center/agents/{agent_id}/statistics/queues`.replace(`{${'agent_id'}}`, encodeURIComponent(String(agentId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary todo deprecated ?
         * @param {string} id
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentInTeam: async (id, page, size, q, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling searchAgentInTeam.');
            }
            const localVarPath = `/call_center/agents/{id}/teams`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [joinedAtFrom]
         * @param {string} [joinedAtTo]
         * @param {Array<string>} [agentId]
         * @param {string} [sort]
         * @param {string} [domainId]
         * @param {string} [fromId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentStateHistory: async (page, size, joinedAtFrom, joinedAtTo, agentId, sort, domainId, fromId, options = {}) => {
            const localVarPath = `/call_center/agents/states/history`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (joinedAtFrom !== undefined) {
                localVarQueryParameter['joined_at.from'] = joinedAtFrom;
            }
            if (joinedAtTo !== undefined) {
                localVarQueryParameter['joined_at.to'] = joinedAtTo;
            }
            if (agentId) {
                localVarQueryParameter['agent_id'] = agentId;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            if (fromId !== undefined) {
                localVarQueryParameter['from_id'] = fromId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<string>} [agentId]
         * @param {string} [timeFrom]
         * @param {string} [timeTo]
         * @param {Array<string>} [status]
         * @param {Array<number>} [queueId]
         * @param {Array<number>} [teamId]
         * @param {string} [utilizationFrom]
         * @param {string} [utilizationTo]
         * @param {boolean} [hasCall]
         * @param {Array<number>} [skillId]
         * @param {Array<number>} [regionId]
         * @param {Array<number>} [supervisorId]
         * @param {Array<string>} [auditorId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentStatusStatistic: async (page, size, q, sort, fields, agentId, timeFrom, timeTo, status, queueId, teamId, utilizationFrom, utilizationTo, hasCall, skillId, regionId, supervisorId, auditorId, options = {}) => {
            const localVarPath = `/call_center/agents/reports/status`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (agentId) {
                localVarQueryParameter['agent_id'] = agentId;
            }
            if (timeFrom !== undefined) {
                localVarQueryParameter['time.from'] = timeFrom;
            }
            if (timeTo !== undefined) {
                localVarQueryParameter['time.to'] = timeTo;
            }
            if (status) {
                localVarQueryParameter['status'] = status;
            }
            if (queueId) {
                localVarQueryParameter['queue_id'] = queueId;
            }
            if (teamId) {
                localVarQueryParameter['team_id'] = teamId;
            }
            if (utilizationFrom !== undefined) {
                localVarQueryParameter['utilization.from'] = utilizationFrom;
            }
            if (utilizationTo !== undefined) {
                localVarQueryParameter['utilization.to'] = utilizationTo;
            }
            if (hasCall !== undefined) {
                localVarQueryParameter['has_call'] = hasCall;
            }
            if (skillId) {
                localVarQueryParameter['skill_id'] = skillId;
            }
            if (regionId) {
                localVarQueryParameter['region_id'] = regionId;
            }
            if (supervisorId) {
                localVarQueryParameter['supervisor_id'] = supervisorId;
            }
            if (auditorId) {
                localVarQueryParameter['auditor_id'] = auditorId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {string} agentId
         * @param {string} [timeFrom]
         * @param {string} [timeTo]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentStatusStatisticItem: async (agentId, timeFrom, timeTo, options = {}) => {
            // verify required parameter 'agentId' is not null or undefined
            if (agentId === null || agentId === undefined) {
                throw new RequiredError('agentId', 'Required parameter agentId was null or undefined when calling searchAgentStatusStatisticItem.');
            }
            const localVarPath = `/call_center/agents/reports/status/{agent_id}`.replace(`{${'agent_id'}}`, encodeURIComponent(String(agentId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (timeFrom !== undefined) {
                localVarQueryParameter['time.from'] = timeFrom;
            }
            if (timeTo !== undefined) {
                localVarQueryParameter['time.to'] = timeTo;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary SearchLookupAgentNotExistsUser
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchLookupUsersAgentNotExists: async (page, size, q, domainId, options = {}) => {
            const localVarPath = `/call_center/lookups/agents/users`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {string} agentId
         * @param {boolean} [allowChange]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchPauseCauseForAgent: async (agentId, allowChange, options = {}) => {
            // verify required parameter 'agentId' is not null or undefined
            if (agentId === null || agentId === undefined) {
                throw new RequiredError('agentId', 'Required parameter agentId was null or undefined when calling searchPauseCauseForAgent.');
            }
            const localVarPath = `/call_center/agents/{agent_id}/pause_causes`.replace(`{${'agent_id'}}`, encodeURIComponent(String(agentId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (allowChange !== undefined) {
                localVarQueryParameter['allow_change'] = allowChange;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update Agent
         * @param {string} id
         * @param {EngineUpdateAgentRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateAgent: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateAgent.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateAgent.');
            }
            const localVarPath = `/call_center/agents/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Change agent status
         * @param {string} id
         * @param {EngineAgentStatusRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateAgentStatus: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateAgentStatus.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateAgentStatus.');
            }
            const localVarPath = `/call_center/agents/{id}/status`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * AgentServiceApi - functional programming interface
 * @export
 */
const AgentServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @param {number} agentId
         * @param {EngineAgentSetStateRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async agentSetState(agentId, body, options) {
            const localVarAxiosArgs = await AgentServiceApiAxiosParamCreator(configuration).agentSetState(agentId, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {string} agentId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [timeFrom]
         * @param {string} [timeTo]
         * @param {string} [q]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async agentStateHistory(agentId, page, size, timeFrom, timeTo, q, domainId, options) {
            const localVarAxiosArgs = await AgentServiceApiAxiosParamCreator(configuration).agentStateHistory(agentId, page, size, timeFrom, timeTo, q, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Create Agent
         * @param {EngineCreateAgentRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createAgent(body, options) {
            const localVarAxiosArgs = await AgentServiceApiAxiosParamCreator(configuration).createAgent(body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Remove Agent
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteAgent(id, domainId, options) {
            const localVarAxiosArgs = await AgentServiceApiAxiosParamCreator(configuration).deleteAgent(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update Agent
         * @param {string} id
         * @param {EnginePatchAgentRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async patchAgent(id, body, options) {
            const localVarAxiosArgs = await AgentServiceApiAxiosParamCreator(configuration).patchAgent(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Agent item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readAgent(id, domainId, options) {
            const localVarAxiosArgs = await AgentServiceApiAxiosParamCreator(configuration).readAgent(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List of Agent
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<string>} [id]
         * @param {Array<string>} [allowChannels]
         * @param {Array<number>} [supervisorId]
         * @param {Array<number>} [teamId]
         * @param {Array<number>} [regionId]
         * @param {Array<number>} [auditorId]
         * @param {boolean} [isSupervisor]
         * @param {Array<number>} [skillId]
         * @param {Array<number>} [queueId]
         * @param {boolean} [notSupervisor]
         * @param {Array<string>} [userId]
         * @param {Array<string>} [extension]
         * @param {Array<number>} [notTeamId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchAgent(page, size, q, sort, fields, id, allowChannels, supervisorId, teamId, regionId, auditorId, isSupervisor, skillId, queueId, notSupervisor, userId, extension, notTeamId, options) {
            const localVarAxiosArgs = await AgentServiceApiAxiosParamCreator(configuration).searchAgent(page, size, q, sort, fields, id, allowChannels, supervisorId, teamId, regionId, auditorId, isSupervisor, skillId, queueId, notSupervisor, userId, extension, notTeamId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [timeFrom]
         * @param {string} [timeTo]
         * @param {Array<number>} [agentId]
         * @param {string} [domainId]
         * @param {string} [q]
         * @param {Array<string>} [fields]
         * @param {string} [sort]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchAgentCallStatistics(page, size, timeFrom, timeTo, agentId, domainId, q, fields, sort, options) {
            const localVarAxiosArgs = await AgentServiceApiAxiosParamCreator(configuration).searchAgentCallStatistics(page, size, timeFrom, timeTo, agentId, domainId, q, fields, sort, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {string} id
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchAgentInQueue(id, page, size, q, sort, fields, options) {
            const localVarAxiosArgs = await AgentServiceApiAxiosParamCreator(configuration).searchAgentInQueue(id, page, size, q, sort, fields, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {string} agentId
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchAgentInQueueStatistics(agentId, domainId, options) {
            const localVarAxiosArgs = await AgentServiceApiAxiosParamCreator(configuration).searchAgentInQueueStatistics(agentId, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary todo deprecated ?
         * @param {string} id
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchAgentInTeam(id, page, size, q, domainId, options) {
            const localVarAxiosArgs = await AgentServiceApiAxiosParamCreator(configuration).searchAgentInTeam(id, page, size, q, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [joinedAtFrom]
         * @param {string} [joinedAtTo]
         * @param {Array<string>} [agentId]
         * @param {string} [sort]
         * @param {string} [domainId]
         * @param {string} [fromId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchAgentStateHistory(page, size, joinedAtFrom, joinedAtTo, agentId, sort, domainId, fromId, options) {
            const localVarAxiosArgs = await AgentServiceApiAxiosParamCreator(configuration).searchAgentStateHistory(page, size, joinedAtFrom, joinedAtTo, agentId, sort, domainId, fromId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<string>} [agentId]
         * @param {string} [timeFrom]
         * @param {string} [timeTo]
         * @param {Array<string>} [status]
         * @param {Array<number>} [queueId]
         * @param {Array<number>} [teamId]
         * @param {string} [utilizationFrom]
         * @param {string} [utilizationTo]
         * @param {boolean} [hasCall]
         * @param {Array<number>} [skillId]
         * @param {Array<number>} [regionId]
         * @param {Array<number>} [supervisorId]
         * @param {Array<string>} [auditorId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchAgentStatusStatistic(page, size, q, sort, fields, agentId, timeFrom, timeTo, status, queueId, teamId, utilizationFrom, utilizationTo, hasCall, skillId, regionId, supervisorId, auditorId, options) {
            const localVarAxiosArgs = await AgentServiceApiAxiosParamCreator(configuration).searchAgentStatusStatistic(page, size, q, sort, fields, agentId, timeFrom, timeTo, status, queueId, teamId, utilizationFrom, utilizationTo, hasCall, skillId, regionId, supervisorId, auditorId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {string} agentId
         * @param {string} [timeFrom]
         * @param {string} [timeTo]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchAgentStatusStatisticItem(agentId, timeFrom, timeTo, options) {
            const localVarAxiosArgs = await AgentServiceApiAxiosParamCreator(configuration).searchAgentStatusStatisticItem(agentId, timeFrom, timeTo, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary SearchLookupAgentNotExistsUser
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchLookupUsersAgentNotExists(page, size, q, domainId, options) {
            const localVarAxiosArgs = await AgentServiceApiAxiosParamCreator(configuration).searchLookupUsersAgentNotExists(page, size, q, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {string} agentId
         * @param {boolean} [allowChange]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchPauseCauseForAgent(agentId, allowChange, options) {
            const localVarAxiosArgs = await AgentServiceApiAxiosParamCreator(configuration).searchPauseCauseForAgent(agentId, allowChange, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update Agent
         * @param {string} id
         * @param {EngineUpdateAgentRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateAgent(id, body, options) {
            const localVarAxiosArgs = await AgentServiceApiAxiosParamCreator(configuration).updateAgent(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Change agent status
         * @param {string} id
         * @param {EngineAgentStatusRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateAgentStatus(id, body, options) {
            const localVarAxiosArgs = await AgentServiceApiAxiosParamCreator(configuration).updateAgentStatus(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * AgentServiceApi - factory interface
 * @export
 */
const AgentServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @param {number} agentId
         * @param {EngineAgentSetStateRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        agentSetState(agentId, body, options) {
            return AgentServiceApiFp(configuration)
                .agentSetState(agentId, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {string} agentId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [timeFrom]
         * @param {string} [timeTo]
         * @param {string} [q]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        agentStateHistory(agentId, page, size, timeFrom, timeTo, q, domainId, options) {
            return AgentServiceApiFp(configuration)
                .agentStateHistory(agentId, page, size, timeFrom, timeTo, q, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Create Agent
         * @param {EngineCreateAgentRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createAgent(body, options) {
            return AgentServiceApiFp(configuration)
                .createAgent(body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Remove Agent
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteAgent(id, domainId, options) {
            return AgentServiceApiFp(configuration)
                .deleteAgent(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update Agent
         * @param {string} id
         * @param {EnginePatchAgentRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchAgent(id, body, options) {
            return AgentServiceApiFp(configuration)
                .patchAgent(id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Agent item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readAgent(id, domainId, options) {
            return AgentServiceApiFp(configuration)
                .readAgent(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List of Agent
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<string>} [id]
         * @param {Array<string>} [allowChannels]
         * @param {Array<number>} [supervisorId]
         * @param {Array<number>} [teamId]
         * @param {Array<number>} [regionId]
         * @param {Array<number>} [auditorId]
         * @param {boolean} [isSupervisor]
         * @param {Array<number>} [skillId]
         * @param {Array<number>} [queueId]
         * @param {boolean} [notSupervisor]
         * @param {Array<string>} [userId]
         * @param {Array<string>} [extension]
         * @param {Array<number>} [notTeamId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgent(page, size, q, sort, fields, id, allowChannels, supervisorId, teamId, regionId, auditorId, isSupervisor, skillId, queueId, notSupervisor, userId, extension, notTeamId, options) {
            return AgentServiceApiFp(configuration)
                .searchAgent(page, size, q, sort, fields, id, allowChannels, supervisorId, teamId, regionId, auditorId, isSupervisor, skillId, queueId, notSupervisor, userId, extension, notTeamId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [timeFrom]
         * @param {string} [timeTo]
         * @param {Array<number>} [agentId]
         * @param {string} [domainId]
         * @param {string} [q]
         * @param {Array<string>} [fields]
         * @param {string} [sort]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentCallStatistics(page, size, timeFrom, timeTo, agentId, domainId, q, fields, sort, options) {
            return AgentServiceApiFp(configuration)
                .searchAgentCallStatistics(page, size, timeFrom, timeTo, agentId, domainId, q, fields, sort, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {string} id
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentInQueue(id, page, size, q, sort, fields, options) {
            return AgentServiceApiFp(configuration)
                .searchAgentInQueue(id, page, size, q, sort, fields, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {string} agentId
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentInQueueStatistics(agentId, domainId, options) {
            return AgentServiceApiFp(configuration)
                .searchAgentInQueueStatistics(agentId, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary todo deprecated ?
         * @param {string} id
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentInTeam(id, page, size, q, domainId, options) {
            return AgentServiceApiFp(configuration)
                .searchAgentInTeam(id, page, size, q, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [joinedAtFrom]
         * @param {string} [joinedAtTo]
         * @param {Array<string>} [agentId]
         * @param {string} [sort]
         * @param {string} [domainId]
         * @param {string} [fromId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentStateHistory(page, size, joinedAtFrom, joinedAtTo, agentId, sort, domainId, fromId, options) {
            return AgentServiceApiFp(configuration)
                .searchAgentStateHistory(page, size, joinedAtFrom, joinedAtTo, agentId, sort, domainId, fromId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<string>} [agentId]
         * @param {string} [timeFrom]
         * @param {string} [timeTo]
         * @param {Array<string>} [status]
         * @param {Array<number>} [queueId]
         * @param {Array<number>} [teamId]
         * @param {string} [utilizationFrom]
         * @param {string} [utilizationTo]
         * @param {boolean} [hasCall]
         * @param {Array<number>} [skillId]
         * @param {Array<number>} [regionId]
         * @param {Array<number>} [supervisorId]
         * @param {Array<string>} [auditorId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentStatusStatistic(page, size, q, sort, fields, agentId, timeFrom, timeTo, status, queueId, teamId, utilizationFrom, utilizationTo, hasCall, skillId, regionId, supervisorId, auditorId, options) {
            return AgentServiceApiFp(configuration)
                .searchAgentStatusStatistic(page, size, q, sort, fields, agentId, timeFrom, timeTo, status, queueId, teamId, utilizationFrom, utilizationTo, hasCall, skillId, regionId, supervisorId, auditorId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {string} agentId
         * @param {string} [timeFrom]
         * @param {string} [timeTo]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentStatusStatisticItem(agentId, timeFrom, timeTo, options) {
            return AgentServiceApiFp(configuration)
                .searchAgentStatusStatisticItem(agentId, timeFrom, timeTo, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary SearchLookupAgentNotExistsUser
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchLookupUsersAgentNotExists(page, size, q, domainId, options) {
            return AgentServiceApiFp(configuration)
                .searchLookupUsersAgentNotExists(page, size, q, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {string} agentId
         * @param {boolean} [allowChange]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchPauseCauseForAgent(agentId, allowChange, options) {
            return AgentServiceApiFp(configuration)
                .searchPauseCauseForAgent(agentId, allowChange, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update Agent
         * @param {string} id
         * @param {EngineUpdateAgentRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateAgent(id, body, options) {
            return AgentServiceApiFp(configuration)
                .updateAgent(id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Change agent status
         * @param {string} id
         * @param {EngineAgentStatusRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateAgentStatus(id, body, options) {
            return AgentServiceApiFp(configuration)
                .updateAgentStatus(id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * AgentServiceApi - object-oriented interface
 * @export
 * @class AgentServiceApi
 * @extends {BaseAPI}
 */
class AgentServiceApi extends BaseAPI {
    /**
     *
     * @param {number} agentId
     * @param {EngineAgentSetStateRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentServiceApi
     */
    agentSetState(agentId, body, options) {
        return AgentServiceApiFp(this.configuration)
            .agentSetState(agentId, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {string} agentId
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [timeFrom]
     * @param {string} [timeTo]
     * @param {string} [q]
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentServiceApi
     */
    agentStateHistory(agentId, page, size, timeFrom, timeTo, q, domainId, options) {
        return AgentServiceApiFp(this.configuration)
            .agentStateHistory(agentId, page, size, timeFrom, timeTo, q, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Create Agent
     * @param {EngineCreateAgentRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentServiceApi
     */
    createAgent(body, options) {
        return AgentServiceApiFp(this.configuration)
            .createAgent(body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Remove Agent
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentServiceApi
     */
    deleteAgent(id, domainId, options) {
        return AgentServiceApiFp(this.configuration)
            .deleteAgent(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update Agent
     * @param {string} id
     * @param {EnginePatchAgentRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentServiceApi
     */
    patchAgent(id, body, options) {
        return AgentServiceApiFp(this.configuration)
            .patchAgent(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Agent item
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentServiceApi
     */
    readAgent(id, domainId, options) {
        return AgentServiceApiFp(this.configuration)
            .readAgent(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List of Agent
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<string>} [id]
     * @param {Array<string>} [allowChannels]
     * @param {Array<number>} [supervisorId]
     * @param {Array<number>} [teamId]
     * @param {Array<number>} [regionId]
     * @param {Array<number>} [auditorId]
     * @param {boolean} [isSupervisor]
     * @param {Array<number>} [skillId]
     * @param {Array<number>} [queueId]
     * @param {boolean} [notSupervisor]
     * @param {Array<string>} [userId]
     * @param {Array<string>} [extension]
     * @param {Array<number>} [notTeamId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentServiceApi
     */
    searchAgent(page, size, q, sort, fields, id, allowChannels, supervisorId, teamId, regionId, auditorId, isSupervisor, skillId, queueId, notSupervisor, userId, extension, notTeamId, options) {
        return AgentServiceApiFp(this.configuration)
            .searchAgent(page, size, q, sort, fields, id, allowChannels, supervisorId, teamId, regionId, auditorId, isSupervisor, skillId, queueId, notSupervisor, userId, extension, notTeamId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [timeFrom]
     * @param {string} [timeTo]
     * @param {Array<number>} [agentId]
     * @param {string} [domainId]
     * @param {string} [q]
     * @param {Array<string>} [fields]
     * @param {string} [sort]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentServiceApi
     */
    searchAgentCallStatistics(page, size, timeFrom, timeTo, agentId, domainId, q, fields, sort, options) {
        return AgentServiceApiFp(this.configuration)
            .searchAgentCallStatistics(page, size, timeFrom, timeTo, agentId, domainId, q, fields, sort, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {string} id
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentServiceApi
     */
    searchAgentInQueue(id, page, size, q, sort, fields, options) {
        return AgentServiceApiFp(this.configuration)
            .searchAgentInQueue(id, page, size, q, sort, fields, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {string} agentId
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentServiceApi
     */
    searchAgentInQueueStatistics(agentId, domainId, options) {
        return AgentServiceApiFp(this.configuration)
            .searchAgentInQueueStatistics(agentId, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary todo deprecated ?
     * @param {string} id
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentServiceApi
     */
    searchAgentInTeam(id, page, size, q, domainId, options) {
        return AgentServiceApiFp(this.configuration)
            .searchAgentInTeam(id, page, size, q, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [joinedAtFrom]
     * @param {string} [joinedAtTo]
     * @param {Array<string>} [agentId]
     * @param {string} [sort]
     * @param {string} [domainId]
     * @param {string} [fromId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentServiceApi
     */
    searchAgentStateHistory(page, size, joinedAtFrom, joinedAtTo, agentId, sort, domainId, fromId, options) {
        return AgentServiceApiFp(this.configuration)
            .searchAgentStateHistory(page, size, joinedAtFrom, joinedAtTo, agentId, sort, domainId, fromId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<string>} [agentId]
     * @param {string} [timeFrom]
     * @param {string} [timeTo]
     * @param {Array<string>} [status]
     * @param {Array<number>} [queueId]
     * @param {Array<number>} [teamId]
     * @param {string} [utilizationFrom]
     * @param {string} [utilizationTo]
     * @param {boolean} [hasCall]
     * @param {Array<number>} [skillId]
     * @param {Array<number>} [regionId]
     * @param {Array<number>} [supervisorId]
     * @param {Array<string>} [auditorId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentServiceApi
     */
    searchAgentStatusStatistic(page, size, q, sort, fields, agentId, timeFrom, timeTo, status, queueId, teamId, utilizationFrom, utilizationTo, hasCall, skillId, regionId, supervisorId, auditorId, options) {
        return AgentServiceApiFp(this.configuration)
            .searchAgentStatusStatistic(page, size, q, sort, fields, agentId, timeFrom, timeTo, status, queueId, teamId, utilizationFrom, utilizationTo, hasCall, skillId, regionId, supervisorId, auditorId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {string} agentId
     * @param {string} [timeFrom]
     * @param {string} [timeTo]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentServiceApi
     */
    searchAgentStatusStatisticItem(agentId, timeFrom, timeTo, options) {
        return AgentServiceApiFp(this.configuration)
            .searchAgentStatusStatisticItem(agentId, timeFrom, timeTo, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary SearchLookupAgentNotExistsUser
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentServiceApi
     */
    searchLookupUsersAgentNotExists(page, size, q, domainId, options) {
        return AgentServiceApiFp(this.configuration)
            .searchLookupUsersAgentNotExists(page, size, q, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {string} agentId
     * @param {boolean} [allowChange]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentServiceApi
     */
    searchPauseCauseForAgent(agentId, allowChange, options) {
        return AgentServiceApiFp(this.configuration)
            .searchPauseCauseForAgent(agentId, allowChange, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update Agent
     * @param {string} id
     * @param {EngineUpdateAgentRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentServiceApi
     */
    updateAgent(id, body, options) {
        return AgentServiceApiFp(this.configuration)
            .updateAgent(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Change agent status
     * @param {string} id
     * @param {EngineAgentStatusRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentServiceApi
     */
    updateAgentStatus(id, body, options) {
        return AgentServiceApiFp(this.configuration)
            .updateAgentStatus(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * AgentSkillServiceApi - axios parameter creator
 * @export
 */
const AgentSkillServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @summary Create AgentSkill
         * @param {string} agentId
         * @param {EngineCreateAgentSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createAgentSkill: async (agentId, body, options = {}) => {
            // verify required parameter 'agentId' is not null or undefined
            if (agentId === null || agentId === undefined) {
                throw new RequiredError('agentId', 'Required parameter agentId was null or undefined when calling createAgentSkill.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createAgentSkill.');
            }
            const localVarPath = `/call_center/agents/{agent_id}/skills`.replace(`{${'agent_id'}}`, encodeURIComponent(String(agentId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Remove AgentSkill
         * @param {string} agentId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteAgentSkill: async (agentId, id, domainId, options = {}) => {
            // verify required parameter 'agentId' is not null or undefined
            if (agentId === null || agentId === undefined) {
                throw new RequiredError('agentId', 'Required parameter agentId was null or undefined when calling deleteAgentSkill.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteAgentSkill.');
            }
            const localVarPath = `/call_center/agents/{agent_id}/skills/{id}`
                .replace(`{${'agent_id'}}`, encodeURIComponent(String(agentId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {string} agentId
         * @param {string} id
         * @param {EnginePatchAgentSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchAgentSkill: async (agentId, id, body, options = {}) => {
            // verify required parameter 'agentId' is not null or undefined
            if (agentId === null || agentId === undefined) {
                throw new RequiredError('agentId', 'Required parameter agentId was null or undefined when calling patchAgentSkill.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling patchAgentSkill.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling patchAgentSkill.');
            }
            const localVarPath = `/call_center/agents/{agent_id}/skills/{id}`
                .replace(`{${'agent_id'}}`, encodeURIComponent(String(agentId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary AgentSkill item
         * @param {string} agentId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readAgentSkill: async (agentId, id, domainId, options = {}) => {
            // verify required parameter 'agentId' is not null or undefined
            if (agentId === null || agentId === undefined) {
                throw new RequiredError('agentId', 'Required parameter agentId was null or undefined when calling readAgentSkill.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readAgentSkill.');
            }
            const localVarPath = `/call_center/agents/{agent_id}/skills/{id}`
                .replace(`{${'agent_id'}}`, encodeURIComponent(String(agentId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List of AgentSkill
         * @param {string} agentId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentSkill: async (agentId, page, size, q, sort, fields, id, options = {}) => {
            // verify required parameter 'agentId' is not null or undefined
            if (agentId === null || agentId === undefined) {
                throw new RequiredError('agentId', 'Required parameter agentId was null or undefined when calling searchAgentSkill.');
            }
            const localVarPath = `/call_center/agents/{agent_id}/skills`.replace(`{${'agent_id'}}`, encodeURIComponent(String(agentId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary SearchLookupAgentNotExistsSkill
         * @param {string} agentId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchLookupAgentNotExistsSkill: async (agentId, page, size, q, domainId, options = {}) => {
            // verify required parameter 'agentId' is not null or undefined
            if (agentId === null || agentId === undefined) {
                throw new RequiredError('agentId', 'Required parameter agentId was null or undefined when calling searchLookupAgentNotExistsSkill.');
            }
            const localVarPath = `/call_center/lookups/agents/skills/{agent_id}`.replace(`{${'agent_id'}}`, encodeURIComponent(String(agentId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update AgentSkill
         * @param {string} agentId
         * @param {string} id
         * @param {EngineUpdateAgentSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateAgentSkill: async (agentId, id, body, options = {}) => {
            // verify required parameter 'agentId' is not null or undefined
            if (agentId === null || agentId === undefined) {
                throw new RequiredError('agentId', 'Required parameter agentId was null or undefined when calling updateAgentSkill.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateAgentSkill.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateAgentSkill.');
            }
            const localVarPath = `/call_center/agents/{agent_id}/skills/{id}`
                .replace(`{${'agent_id'}}`, encodeURIComponent(String(agentId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * AgentSkillServiceApi - functional programming interface
 * @export
 */
const AgentSkillServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @summary Create AgentSkill
         * @param {string} agentId
         * @param {EngineCreateAgentSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createAgentSkill(agentId, body, options) {
            const localVarAxiosArgs = await AgentSkillServiceApiAxiosParamCreator(configuration).createAgentSkill(agentId, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Remove AgentSkill
         * @param {string} agentId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteAgentSkill(agentId, id, domainId, options) {
            const localVarAxiosArgs = await AgentSkillServiceApiAxiosParamCreator(configuration).deleteAgentSkill(agentId, id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {string} agentId
         * @param {string} id
         * @param {EnginePatchAgentSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async patchAgentSkill(agentId, id, body, options) {
            const localVarAxiosArgs = await AgentSkillServiceApiAxiosParamCreator(configuration).patchAgentSkill(agentId, id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary AgentSkill item
         * @param {string} agentId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readAgentSkill(agentId, id, domainId, options) {
            const localVarAxiosArgs = await AgentSkillServiceApiAxiosParamCreator(configuration).readAgentSkill(agentId, id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List of AgentSkill
         * @param {string} agentId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchAgentSkill(agentId, page, size, q, sort, fields, id, options) {
            const localVarAxiosArgs = await AgentSkillServiceApiAxiosParamCreator(configuration).searchAgentSkill(agentId, page, size, q, sort, fields, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary SearchLookupAgentNotExistsSkill
         * @param {string} agentId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchLookupAgentNotExistsSkill(agentId, page, size, q, domainId, options) {
            const localVarAxiosArgs = await AgentSkillServiceApiAxiosParamCreator(configuration).searchLookupAgentNotExistsSkill(agentId, page, size, q, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update AgentSkill
         * @param {string} agentId
         * @param {string} id
         * @param {EngineUpdateAgentSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateAgentSkill(agentId, id, body, options) {
            const localVarAxiosArgs = await AgentSkillServiceApiAxiosParamCreator(configuration).updateAgentSkill(agentId, id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * AgentSkillServiceApi - factory interface
 * @export
 */
const AgentSkillServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @summary Create AgentSkill
         * @param {string} agentId
         * @param {EngineCreateAgentSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createAgentSkill(agentId, body, options) {
            return AgentSkillServiceApiFp(configuration)
                .createAgentSkill(agentId, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Remove AgentSkill
         * @param {string} agentId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteAgentSkill(agentId, id, domainId, options) {
            return AgentSkillServiceApiFp(configuration)
                .deleteAgentSkill(agentId, id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {string} agentId
         * @param {string} id
         * @param {EnginePatchAgentSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchAgentSkill(agentId, id, body, options) {
            return AgentSkillServiceApiFp(configuration)
                .patchAgentSkill(agentId, id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary AgentSkill item
         * @param {string} agentId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readAgentSkill(agentId, id, domainId, options) {
            return AgentSkillServiceApiFp(configuration)
                .readAgentSkill(agentId, id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List of AgentSkill
         * @param {string} agentId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentSkill(agentId, page, size, q, sort, fields, id, options) {
            return AgentSkillServiceApiFp(configuration)
                .searchAgentSkill(agentId, page, size, q, sort, fields, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary SearchLookupAgentNotExistsSkill
         * @param {string} agentId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchLookupAgentNotExistsSkill(agentId, page, size, q, domainId, options) {
            return AgentSkillServiceApiFp(configuration)
                .searchLookupAgentNotExistsSkill(agentId, page, size, q, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update AgentSkill
         * @param {string} agentId
         * @param {string} id
         * @param {EngineUpdateAgentSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateAgentSkill(agentId, id, body, options) {
            return AgentSkillServiceApiFp(configuration)
                .updateAgentSkill(agentId, id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * AgentSkillServiceApi - object-oriented interface
 * @export
 * @class AgentSkillServiceApi
 * @extends {BaseAPI}
 */
class AgentSkillServiceApi extends BaseAPI {
    /**
     *
     * @summary Create AgentSkill
     * @param {string} agentId
     * @param {EngineCreateAgentSkillRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentSkillServiceApi
     */
    createAgentSkill(agentId, body, options) {
        return AgentSkillServiceApiFp(this.configuration)
            .createAgentSkill(agentId, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Remove AgentSkill
     * @param {string} agentId
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentSkillServiceApi
     */
    deleteAgentSkill(agentId, id, domainId, options) {
        return AgentSkillServiceApiFp(this.configuration)
            .deleteAgentSkill(agentId, id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {string} agentId
     * @param {string} id
     * @param {EnginePatchAgentSkillRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentSkillServiceApi
     */
    patchAgentSkill(agentId, id, body, options) {
        return AgentSkillServiceApiFp(this.configuration)
            .patchAgentSkill(agentId, id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary AgentSkill item
     * @param {string} agentId
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentSkillServiceApi
     */
    readAgentSkill(agentId, id, domainId, options) {
        return AgentSkillServiceApiFp(this.configuration)
            .readAgentSkill(agentId, id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List of AgentSkill
     * @param {string} agentId
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentSkillServiceApi
     */
    searchAgentSkill(agentId, page, size, q, sort, fields, id, options) {
        return AgentSkillServiceApiFp(this.configuration)
            .searchAgentSkill(agentId, page, size, q, sort, fields, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary SearchLookupAgentNotExistsSkill
     * @param {string} agentId
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentSkillServiceApi
     */
    searchLookupAgentNotExistsSkill(agentId, page, size, q, domainId, options) {
        return AgentSkillServiceApiFp(this.configuration)
            .searchLookupAgentNotExistsSkill(agentId, page, size, q, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update AgentSkill
     * @param {string} agentId
     * @param {string} id
     * @param {EngineUpdateAgentSkillRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentSkillServiceApi
     */
    updateAgentSkill(agentId, id, body, options) {
        return AgentSkillServiceApiFp(this.configuration)
            .updateAgentSkill(agentId, id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * AgentTeamServiceApi - axios parameter creator
 * @export
 */
const AgentTeamServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @summary Create AgentTeam
         * @param {EngineCreateAgentTeamRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createAgentTeam: async (body, options = {}) => {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createAgentTeam.');
            }
            const localVarPath = `/call_center/teams`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Remove AgentTeam
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteAgentTeam: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteAgentTeam.');
            }
            const localVarPath = `/call_center/teams/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary AgentTeam item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readAgentTeam: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readAgentTeam.');
            }
            const localVarPath = `/call_center/teams/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List of AgentTeam
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {Array<string>} [strategy]
         * @param {Array<number>} [adminId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentTeam: async (page, size, q, sort, fields, id, strategy, adminId, options = {}) => {
            const localVarPath = `/call_center/teams`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            if (strategy) {
                localVarQueryParameter['strategy'] = strategy;
            }
            if (adminId) {
                localVarQueryParameter['admin_id'] = adminId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update AgentTeam
         * @param {string} id
         * @param {EngineUpdateAgentTeamRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateAgentTeam: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateAgentTeam.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateAgentTeam.');
            }
            const localVarPath = `/call_center/teams/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * AgentTeamServiceApi - functional programming interface
 * @export
 */
const AgentTeamServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @summary Create AgentTeam
         * @param {EngineCreateAgentTeamRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createAgentTeam(body, options) {
            const localVarAxiosArgs = await AgentTeamServiceApiAxiosParamCreator(configuration).createAgentTeam(body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Remove AgentTeam
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteAgentTeam(id, domainId, options) {
            const localVarAxiosArgs = await AgentTeamServiceApiAxiosParamCreator(configuration).deleteAgentTeam(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary AgentTeam item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readAgentTeam(id, domainId, options) {
            const localVarAxiosArgs = await AgentTeamServiceApiAxiosParamCreator(configuration).readAgentTeam(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List of AgentTeam
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {Array<string>} [strategy]
         * @param {Array<number>} [adminId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchAgentTeam(page, size, q, sort, fields, id, strategy, adminId, options) {
            const localVarAxiosArgs = await AgentTeamServiceApiAxiosParamCreator(configuration).searchAgentTeam(page, size, q, sort, fields, id, strategy, adminId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update AgentTeam
         * @param {string} id
         * @param {EngineUpdateAgentTeamRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateAgentTeam(id, body, options) {
            const localVarAxiosArgs = await AgentTeamServiceApiAxiosParamCreator(configuration).updateAgentTeam(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * AgentTeamServiceApi - factory interface
 * @export
 */
const AgentTeamServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @summary Create AgentTeam
         * @param {EngineCreateAgentTeamRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createAgentTeam(body, options) {
            return AgentTeamServiceApiFp(configuration)
                .createAgentTeam(body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Remove AgentTeam
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteAgentTeam(id, domainId, options) {
            return AgentTeamServiceApiFp(configuration)
                .deleteAgentTeam(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary AgentTeam item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readAgentTeam(id, domainId, options) {
            return AgentTeamServiceApiFp(configuration)
                .readAgentTeam(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List of AgentTeam
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {Array<string>} [strategy]
         * @param {Array<number>} [adminId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAgentTeam(page, size, q, sort, fields, id, strategy, adminId, options) {
            return AgentTeamServiceApiFp(configuration)
                .searchAgentTeam(page, size, q, sort, fields, id, strategy, adminId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update AgentTeam
         * @param {string} id
         * @param {EngineUpdateAgentTeamRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateAgentTeam(id, body, options) {
            return AgentTeamServiceApiFp(configuration)
                .updateAgentTeam(id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * AgentTeamServiceApi - object-oriented interface
 * @export
 * @class AgentTeamServiceApi
 * @extends {BaseAPI}
 */
class AgentTeamServiceApi extends BaseAPI {
    /**
     *
     * @summary Create AgentTeam
     * @param {EngineCreateAgentTeamRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentTeamServiceApi
     */
    createAgentTeam(body, options) {
        return AgentTeamServiceApiFp(this.configuration)
            .createAgentTeam(body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Remove AgentTeam
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentTeamServiceApi
     */
    deleteAgentTeam(id, domainId, options) {
        return AgentTeamServiceApiFp(this.configuration)
            .deleteAgentTeam(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary AgentTeam item
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentTeamServiceApi
     */
    readAgentTeam(id, domainId, options) {
        return AgentTeamServiceApiFp(this.configuration)
            .readAgentTeam(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List of AgentTeam
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {Array<string>} [strategy]
     * @param {Array<number>} [adminId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentTeamServiceApi
     */
    searchAgentTeam(page, size, q, sort, fields, id, strategy, adminId, options) {
        return AgentTeamServiceApiFp(this.configuration)
            .searchAgentTeam(page, size, q, sort, fields, id, strategy, adminId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update AgentTeam
     * @param {string} id
     * @param {EngineUpdateAgentTeamRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof AgentTeamServiceApi
     */
    updateAgentTeam(id, body, options) {
        return AgentTeamServiceApiFp(this.configuration)
            .updateAgentTeam(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * BackendProfileServiceApi - axios parameter creator
 * @export
 */
const BackendProfileServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @summary Create BackendProfile
         * @param {StorageCreateBackendProfileRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createBackendProfile: async (body, options = {}) => {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createBackendProfile.');
            }
            const localVarPath = `/storage/backend_profiles`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Remove BackendProfile
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteBackendProfile: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteBackendProfile.');
            }
            const localVarPath = `/storage/backend_profiles/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Patch BackendProfile
         * @param {string} id
         * @param {StoragePatchBackendProfileRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchBackendProfile: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling patchBackendProfile.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling patchBackendProfile.');
            }
            const localVarPath = `/storage/backend_profiles/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary BackendProfile item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readBackendProfile: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readBackendProfile.');
            }
            const localVarPath = `/storage/backend_profiles/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List of BackendProfile
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchBackendProfile: async (page, size, q, sort, fields, id, options = {}) => {
            const localVarPath = `/storage/backend_profiles`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update BackendProfile
         * @param {string} id
         * @param {StorageUpdateBackendProfileRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateBackendProfile: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateBackendProfile.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateBackendProfile.');
            }
            const localVarPath = `/storage/backend_profiles/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * BackendProfileServiceApi - functional programming interface
 * @export
 */
const BackendProfileServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @summary Create BackendProfile
         * @param {StorageCreateBackendProfileRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createBackendProfile(body, options) {
            const localVarAxiosArgs = await BackendProfileServiceApiAxiosParamCreator(configuration).createBackendProfile(body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Remove BackendProfile
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteBackendProfile(id, domainId, options) {
            const localVarAxiosArgs = await BackendProfileServiceApiAxiosParamCreator(configuration).deleteBackendProfile(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Patch BackendProfile
         * @param {string} id
         * @param {StoragePatchBackendProfileRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async patchBackendProfile(id, body, options) {
            const localVarAxiosArgs = await BackendProfileServiceApiAxiosParamCreator(configuration).patchBackendProfile(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary BackendProfile item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readBackendProfile(id, domainId, options) {
            const localVarAxiosArgs = await BackendProfileServiceApiAxiosParamCreator(configuration).readBackendProfile(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List of BackendProfile
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchBackendProfile(page, size, q, sort, fields, id, options) {
            const localVarAxiosArgs = await BackendProfileServiceApiAxiosParamCreator(configuration).searchBackendProfile(page, size, q, sort, fields, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update BackendProfile
         * @param {string} id
         * @param {StorageUpdateBackendProfileRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateBackendProfile(id, body, options) {
            const localVarAxiosArgs = await BackendProfileServiceApiAxiosParamCreator(configuration).updateBackendProfile(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * BackendProfileServiceApi - factory interface
 * @export
 */
const BackendProfileServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @summary Create BackendProfile
         * @param {StorageCreateBackendProfileRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createBackendProfile(body, options) {
            return BackendProfileServiceApiFp(configuration)
                .createBackendProfile(body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Remove BackendProfile
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteBackendProfile(id, domainId, options) {
            return BackendProfileServiceApiFp(configuration)
                .deleteBackendProfile(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Patch BackendProfile
         * @param {string} id
         * @param {StoragePatchBackendProfileRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchBackendProfile(id, body, options) {
            return BackendProfileServiceApiFp(configuration)
                .patchBackendProfile(id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary BackendProfile item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readBackendProfile(id, domainId, options) {
            return BackendProfileServiceApiFp(configuration)
                .readBackendProfile(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List of BackendProfile
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchBackendProfile(page, size, q, sort, fields, id, options) {
            return BackendProfileServiceApiFp(configuration)
                .searchBackendProfile(page, size, q, sort, fields, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update BackendProfile
         * @param {string} id
         * @param {StorageUpdateBackendProfileRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateBackendProfile(id, body, options) {
            return BackendProfileServiceApiFp(configuration)
                .updateBackendProfile(id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * BackendProfileServiceApi - object-oriented interface
 * @export
 * @class BackendProfileServiceApi
 * @extends {BaseAPI}
 */
class BackendProfileServiceApi extends BaseAPI {
    /**
     *
     * @summary Create BackendProfile
     * @param {StorageCreateBackendProfileRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof BackendProfileServiceApi
     */
    createBackendProfile(body, options) {
        return BackendProfileServiceApiFp(this.configuration)
            .createBackendProfile(body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Remove BackendProfile
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof BackendProfileServiceApi
     */
    deleteBackendProfile(id, domainId, options) {
        return BackendProfileServiceApiFp(this.configuration)
            .deleteBackendProfile(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Patch BackendProfile
     * @param {string} id
     * @param {StoragePatchBackendProfileRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof BackendProfileServiceApi
     */
    patchBackendProfile(id, body, options) {
        return BackendProfileServiceApiFp(this.configuration)
            .patchBackendProfile(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary BackendProfile item
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof BackendProfileServiceApi
     */
    readBackendProfile(id, domainId, options) {
        return BackendProfileServiceApiFp(this.configuration)
            .readBackendProfile(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List of BackendProfile
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof BackendProfileServiceApi
     */
    searchBackendProfile(page, size, q, sort, fields, id, options) {
        return BackendProfileServiceApiFp(this.configuration)
            .searchBackendProfile(page, size, q, sort, fields, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update BackendProfile
     * @param {string} id
     * @param {StorageUpdateBackendProfileRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof BackendProfileServiceApi
     */
    updateBackendProfile(id, body, options) {
        return BackendProfileServiceApiFp(this.configuration)
            .updateBackendProfile(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * BucketServiceApi - axios parameter creator
 * @export
 */
const BucketServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @summary Create Bucket
         * @param {EngineCreateBucketRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createBucket: async (body, options = {}) => {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createBucket.');
            }
            const localVarPath = `/call_center/buckets`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Remove Bucket
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteBucket: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteBucket.');
            }
            const localVarPath = `/call_center/buckets/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Bucket item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readBucket: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readBucket.');
            }
            const localVarPath = `/call_center/buckets/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List of Bucket
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchBucket: async (page, size, q, sort, fields, id, options = {}) => {
            const localVarPath = `/call_center/buckets`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update Bucket
         * @param {string} id
         * @param {EngineUpdateBucketRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateBucket: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateBucket.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateBucket.');
            }
            const localVarPath = `/call_center/buckets/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * BucketServiceApi - functional programming interface
 * @export
 */
const BucketServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @summary Create Bucket
         * @param {EngineCreateBucketRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createBucket(body, options) {
            const localVarAxiosArgs = await BucketServiceApiAxiosParamCreator(configuration).createBucket(body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Remove Bucket
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteBucket(id, domainId, options) {
            const localVarAxiosArgs = await BucketServiceApiAxiosParamCreator(configuration).deleteBucket(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Bucket item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readBucket(id, domainId, options) {
            const localVarAxiosArgs = await BucketServiceApiAxiosParamCreator(configuration).readBucket(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List of Bucket
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchBucket(page, size, q, sort, fields, id, options) {
            const localVarAxiosArgs = await BucketServiceApiAxiosParamCreator(configuration).searchBucket(page, size, q, sort, fields, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update Bucket
         * @param {string} id
         * @param {EngineUpdateBucketRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateBucket(id, body, options) {
            const localVarAxiosArgs = await BucketServiceApiAxiosParamCreator(configuration).updateBucket(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * BucketServiceApi - factory interface
 * @export
 */
const BucketServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @summary Create Bucket
         * @param {EngineCreateBucketRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createBucket(body, options) {
            return BucketServiceApiFp(configuration)
                .createBucket(body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Remove Bucket
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteBucket(id, domainId, options) {
            return BucketServiceApiFp(configuration)
                .deleteBucket(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Bucket item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readBucket(id, domainId, options) {
            return BucketServiceApiFp(configuration)
                .readBucket(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List of Bucket
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchBucket(page, size, q, sort, fields, id, options) {
            return BucketServiceApiFp(configuration)
                .searchBucket(page, size, q, sort, fields, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update Bucket
         * @param {string} id
         * @param {EngineUpdateBucketRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateBucket(id, body, options) {
            return BucketServiceApiFp(configuration)
                .updateBucket(id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * BucketServiceApi - object-oriented interface
 * @export
 * @class BucketServiceApi
 * @extends {BaseAPI}
 */
class BucketServiceApi extends BaseAPI {
    /**
     *
     * @summary Create Bucket
     * @param {EngineCreateBucketRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof BucketServiceApi
     */
    createBucket(body, options) {
        return BucketServiceApiFp(this.configuration)
            .createBucket(body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Remove Bucket
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof BucketServiceApi
     */
    deleteBucket(id, domainId, options) {
        return BucketServiceApiFp(this.configuration)
            .deleteBucket(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Bucket item
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof BucketServiceApi
     */
    readBucket(id, domainId, options) {
        return BucketServiceApiFp(this.configuration)
            .readBucket(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List of Bucket
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof BucketServiceApi
     */
    searchBucket(page, size, q, sort, fields, id, options) {
        return BucketServiceApiFp(this.configuration)
            .searchBucket(page, size, q, sort, fields, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update Bucket
     * @param {string} id
     * @param {EngineUpdateBucketRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof BucketServiceApi
     */
    updateBucket(id, body, options) {
        return BucketServiceApiFp(this.configuration)
            .updateBucket(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * CalendarServiceApi - axios parameter creator
 * @export
 */
const CalendarServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @summary Create calendar
         * @param {EngineCreateCalendarRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createCalendar: async (body, options = {}) => {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createCalendar.');
            }
            const localVarPath = `/calendars`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Remove calendar
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteCalendar: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteCalendar.');
            }
            const localVarPath = `/calendars/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Calendar item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readCalendar: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readCalendar.');
            }
            const localVarPath = `/calendars/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List of calendar
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchCalendar: async (page, size, q, sort, fields, id, options = {}) => {
            const localVarPath = `/calendars`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List timezones
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchTimezones: async (page, size, q, options = {}) => {
            const localVarPath = `/calendars/timezones`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update calendar
         * @param {string} id
         * @param {EngineUpdateCalendarRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateCalendar: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateCalendar.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateCalendar.');
            }
            const localVarPath = `/calendars/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * CalendarServiceApi - functional programming interface
 * @export
 */
const CalendarServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @summary Create calendar
         * @param {EngineCreateCalendarRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createCalendar(body, options) {
            const localVarAxiosArgs = await CalendarServiceApiAxiosParamCreator(configuration).createCalendar(body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Remove calendar
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteCalendar(id, domainId, options) {
            const localVarAxiosArgs = await CalendarServiceApiAxiosParamCreator(configuration).deleteCalendar(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Calendar item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readCalendar(id, domainId, options) {
            const localVarAxiosArgs = await CalendarServiceApiAxiosParamCreator(configuration).readCalendar(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List of calendar
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchCalendar(page, size, q, sort, fields, id, options) {
            const localVarAxiosArgs = await CalendarServiceApiAxiosParamCreator(configuration).searchCalendar(page, size, q, sort, fields, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List timezones
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchTimezones(page, size, q, options) {
            const localVarAxiosArgs = await CalendarServiceApiAxiosParamCreator(configuration).searchTimezones(page, size, q, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update calendar
         * @param {string} id
         * @param {EngineUpdateCalendarRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateCalendar(id, body, options) {
            const localVarAxiosArgs = await CalendarServiceApiAxiosParamCreator(configuration).updateCalendar(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * CalendarServiceApi - factory interface
 * @export
 */
const CalendarServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @summary Create calendar
         * @param {EngineCreateCalendarRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createCalendar(body, options) {
            return CalendarServiceApiFp(configuration)
                .createCalendar(body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Remove calendar
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteCalendar(id, domainId, options) {
            return CalendarServiceApiFp(configuration)
                .deleteCalendar(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Calendar item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readCalendar(id, domainId, options) {
            return CalendarServiceApiFp(configuration)
                .readCalendar(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List of calendar
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchCalendar(page, size, q, sort, fields, id, options) {
            return CalendarServiceApiFp(configuration)
                .searchCalendar(page, size, q, sort, fields, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List timezones
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchTimezones(page, size, q, options) {
            return CalendarServiceApiFp(configuration)
                .searchTimezones(page, size, q, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update calendar
         * @param {string} id
         * @param {EngineUpdateCalendarRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateCalendar(id, body, options) {
            return CalendarServiceApiFp(configuration)
                .updateCalendar(id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * CalendarServiceApi - object-oriented interface
 * @export
 * @class CalendarServiceApi
 * @extends {BaseAPI}
 */
class CalendarServiceApi extends BaseAPI {
    /**
     *
     * @summary Create calendar
     * @param {EngineCreateCalendarRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CalendarServiceApi
     */
    createCalendar(body, options) {
        return CalendarServiceApiFp(this.configuration)
            .createCalendar(body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Remove calendar
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CalendarServiceApi
     */
    deleteCalendar(id, domainId, options) {
        return CalendarServiceApiFp(this.configuration)
            .deleteCalendar(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Calendar item
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CalendarServiceApi
     */
    readCalendar(id, domainId, options) {
        return CalendarServiceApiFp(this.configuration)
            .readCalendar(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List of calendar
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CalendarServiceApi
     */
    searchCalendar(page, size, q, sort, fields, id, options) {
        return CalendarServiceApiFp(this.configuration)
            .searchCalendar(page, size, q, sort, fields, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List timezones
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CalendarServiceApi
     */
    searchTimezones(page, size, q, options) {
        return CalendarServiceApiFp(this.configuration)
            .searchTimezones(page, size, q, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update calendar
     * @param {string} id
     * @param {EngineUpdateCalendarRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CalendarServiceApi
     */
    updateCalendar(id, body, options) {
        return CalendarServiceApiFp(this.configuration)
            .updateCalendar(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * CallServiceApi - axios parameter creator
 * @export
 */
const CallServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @param {EngineAggregateHistoryCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        aggregateHistoryCall: async (body, options = {}) => {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling aggregateHistoryCall.');
            }
            const localVarPath = `/calls/history/aggregate`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {string} id
         * @param {EngineBlindTransferCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        blindTransferCall: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling blindTransferCall.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling blindTransferCall.');
            }
            const localVarPath = `/calls/active/{id}/transfer`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Create e call
         * @param {EngineCreateCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createCall: async (body, options = {}) => {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createCall.');
            }
            const localVarPath = `/calls`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {string} callId
         * @param {EngineCreateCallAnnotationRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createCallAnnotation: async (callId, body, options = {}) => {
            // verify required parameter 'callId' is not null or undefined
            if (callId === null || callId === undefined) {
                throw new RequiredError('callId', 'Required parameter callId was null or undefined when calling createCallAnnotation.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createCallAnnotation.');
            }
            const localVarPath = `/calls/history/{call_id}/annotation`.replace(`{${'call_id'}}`, encodeURIComponent(String(callId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {string} callId
         * @param {string} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteCallAnnotation: async (callId, id, options = {}) => {
            // verify required parameter 'callId' is not null or undefined
            if (callId === null || callId === undefined) {
                throw new RequiredError('callId', 'Required parameter callId was null or undefined when calling deleteCallAnnotation.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteCallAnnotation.');
            }
            const localVarPath = `/calls/history/{call_id}/annotation/{id}`
                .replace(`{${'call_id'}}`, encodeURIComponent(String(callId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {string} id
         * @param {EngineDtmfCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        dtmfCall: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling dtmfCall.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling dtmfCall.');
            }
            const localVarPath = `/calls/active/{id}/dtmf`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {string} id
         * @param {EngineEavesdropCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        eavesdropCall: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling eavesdropCall.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling eavesdropCall.');
            }
            const localVarPath = `/calls/active/{id}/eavesdrop`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {string} id
         * @param {EngineHangupCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        hangupCall: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling hangupCall.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling hangupCall.');
            }
            const localVarPath = `/calls/active/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {string} id
         * @param {EngineUserCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        holdCall: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling holdCall.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling holdCall.');
            }
            const localVarPath = `/calls/active/{id}/hold`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Call item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readCall: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readCall.');
            }
            const localVarPath = `/calls/active/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {string} [createdAtFrom]
         * @param {string} [createdAtTo]
         * @param {Array<string>} [userId]
         * @param {Array<string>} [agentId]
         * @param {Array<string>} [queueId]
         * @param {Array<string>} [teamId]
         * @param {Array<string>} [memberId]
         * @param {Array<string>} [gatewayId]
         * @param {string} [durationFrom]
         * @param {string} [durationTo]
         * @param {boolean} [skipParent]
         * @param {string} [parentId]
         * @param {Array<string>} [cause]
         * @param {boolean} [existsFile]
         * @param {string} [domainId]
         * @param {string} [number]
         * @param {Array<string>} [direction]
         * @param {string} [answeredAtFrom]
         * @param {string} [answeredAtTo]
         * @param {boolean} [missed]
         * @param {string} [storedAtFrom]
         * @param {string} [storedAtTo]
         * @param {Array<string>} [supervisorId]
         * @param {Array<string>} [state]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchActiveCall: async (page, size, q, sort, fields, createdAtFrom, createdAtTo, userId, agentId, queueId, teamId, memberId, gatewayId, durationFrom, durationTo, skipParent, parentId, cause, existsFile, domainId, number, direction, answeredAtFrom, answeredAtTo, missed, storedAtFrom, storedAtTo, supervisorId, state, options = {}) => {
            const localVarPath = `/calls/active`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (createdAtFrom !== undefined) {
                localVarQueryParameter['created_at.from'] = createdAtFrom;
            }
            if (createdAtTo !== undefined) {
                localVarQueryParameter['created_at.to'] = createdAtTo;
            }
            if (userId) {
                localVarQueryParameter['user_id'] = userId;
            }
            if (agentId) {
                localVarQueryParameter['agent_id'] = agentId;
            }
            if (queueId) {
                localVarQueryParameter['queue_id'] = queueId;
            }
            if (teamId) {
                localVarQueryParameter['team_id'] = teamId;
            }
            if (memberId) {
                localVarQueryParameter['member_id'] = memberId;
            }
            if (gatewayId) {
                localVarQueryParameter['gateway_id'] = gatewayId;
            }
            if (durationFrom !== undefined) {
                localVarQueryParameter['duration.from'] = durationFrom;
            }
            if (durationTo !== undefined) {
                localVarQueryParameter['duration.to'] = durationTo;
            }
            if (skipParent !== undefined) {
                localVarQueryParameter['skip_parent'] = skipParent;
            }
            if (parentId !== undefined) {
                localVarQueryParameter['parent_id'] = parentId;
            }
            if (cause) {
                localVarQueryParameter['cause'] = cause;
            }
            if (existsFile !== undefined) {
                localVarQueryParameter['exists_file'] = existsFile;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            if (number !== undefined) {
                localVarQueryParameter['number'] = number;
            }
            if (direction) {
                localVarQueryParameter['direction'] = direction;
            }
            if (answeredAtFrom !== undefined) {
                localVarQueryParameter['answered_at.from'] = answeredAtFrom;
            }
            if (answeredAtTo !== undefined) {
                localVarQueryParameter['answered_at.to'] = answeredAtTo;
            }
            if (missed !== undefined) {
                localVarQueryParameter['missed'] = missed;
            }
            if (storedAtFrom !== undefined) {
                localVarQueryParameter['stored_at.from'] = storedAtFrom;
            }
            if (storedAtTo !== undefined) {
                localVarQueryParameter['stored_at.to'] = storedAtTo;
            }
            if (supervisorId) {
                localVarQueryParameter['supervisor_id'] = supervisorId;
            }
            if (state) {
                localVarQueryParameter['state'] = state;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List of call
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {string} [createdAtFrom]
         * @param {string} [createdAtTo]
         * @param {Array<string>} [userId]
         * @param {Array<string>} [agentId]
         * @param {Array<string>} [queueId]
         * @param {Array<string>} [teamId]
         * @param {Array<string>} [memberId]
         * @param {Array<string>} [gatewayId]
         * @param {string} [durationFrom]
         * @param {string} [durationTo]
         * @param {boolean} [skipParent]
         * @param {string} [parentId]
         * @param {Array<string>} [cause]
         * @param {boolean} [hasFile]
         * @param {string} [domainId]
         * @param {string} [number]
         * @param {string} [direction]
         * @param {string} [answeredAtFrom]
         * @param {string} [answeredAtTo]
         * @param {boolean} [missed]
         * @param {string} [storedAtFrom]
         * @param {string} [storedAtTo]
         * @param {Array<string>} [id]
         * @param {Array<string>} [transferFrom]
         * @param {Array<string>} [transferTo]
         * @param {Array<string>} [dependencyId]
         * @param {Array<string>} [tags]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchHistoryCall: async (page, size, q, sort, fields, createdAtFrom, createdAtTo, userId, agentId, queueId, teamId, memberId, gatewayId, durationFrom, durationTo, skipParent, parentId, cause, hasFile, domainId, number, direction, answeredAtFrom, answeredAtTo, missed, storedAtFrom, storedAtTo, id, transferFrom, transferTo, dependencyId, tags, options = {}) => {
            const localVarPath = `/calls/history`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (createdAtFrom !== undefined) {
                localVarQueryParameter['created_at.from'] = createdAtFrom;
            }
            if (createdAtTo !== undefined) {
                localVarQueryParameter['created_at.to'] = createdAtTo;
            }
            if (userId) {
                localVarQueryParameter['user_id'] = userId;
            }
            if (agentId) {
                localVarQueryParameter['agent_id'] = agentId;
            }
            if (queueId) {
                localVarQueryParameter['queue_id'] = queueId;
            }
            if (teamId) {
                localVarQueryParameter['team_id'] = teamId;
            }
            if (memberId) {
                localVarQueryParameter['member_id'] = memberId;
            }
            if (gatewayId) {
                localVarQueryParameter['gateway_id'] = gatewayId;
            }
            if (durationFrom !== undefined) {
                localVarQueryParameter['duration.from'] = durationFrom;
            }
            if (durationTo !== undefined) {
                localVarQueryParameter['duration.to'] = durationTo;
            }
            if (skipParent !== undefined) {
                localVarQueryParameter['skip_parent'] = skipParent;
            }
            if (parentId !== undefined) {
                localVarQueryParameter['parent_id'] = parentId;
            }
            if (cause) {
                localVarQueryParameter['cause'] = cause;
            }
            if (hasFile !== undefined) {
                localVarQueryParameter['has_file'] = hasFile;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            if (number !== undefined) {
                localVarQueryParameter['number'] = number;
            }
            if (direction !== undefined) {
                localVarQueryParameter['direction'] = direction;
            }
            if (answeredAtFrom !== undefined) {
                localVarQueryParameter['answered_at.from'] = answeredAtFrom;
            }
            if (answeredAtTo !== undefined) {
                localVarQueryParameter['answered_at.to'] = answeredAtTo;
            }
            if (missed !== undefined) {
                localVarQueryParameter['missed'] = missed;
            }
            if (storedAtFrom !== undefined) {
                localVarQueryParameter['stored_at.from'] = storedAtFrom;
            }
            if (storedAtTo !== undefined) {
                localVarQueryParameter['stored_at.to'] = storedAtTo;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            if (transferFrom) {
                localVarQueryParameter['transfer_from'] = transferFrom;
            }
            if (transferTo) {
                localVarQueryParameter['transfer_to'] = transferTo;
            }
            if (dependencyId) {
                localVarQueryParameter['dependency_id'] = dependencyId;
            }
            if (tags) {
                localVarQueryParameter['tags'] = tags;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {string} id
         * @param {EngineUserCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        unHoldCall: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling unHoldCall.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling unHoldCall.');
            }
            const localVarPath = `/calls/active/{id}/unhold`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {string} callId
         * @param {string} id
         * @param {EngineUpdateCallAnnotationRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateCallAnnotation: async (callId, id, body, options = {}) => {
            // verify required parameter 'callId' is not null or undefined
            if (callId === null || callId === undefined) {
                throw new RequiredError('callId', 'Required parameter callId was null or undefined when calling updateCallAnnotation.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateCallAnnotation.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateCallAnnotation.');
            }
            const localVarPath = `/calls/history/{call_id}/annotation/{id}`
                .replace(`{${'call_id'}}`, encodeURIComponent(String(callId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * CallServiceApi - functional programming interface
 * @export
 */
const CallServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @param {EngineAggregateHistoryCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async aggregateHistoryCall(body, options) {
            const localVarAxiosArgs = await CallServiceApiAxiosParamCreator(configuration).aggregateHistoryCall(body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {string} id
         * @param {EngineBlindTransferCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async blindTransferCall(id, body, options) {
            const localVarAxiosArgs = await CallServiceApiAxiosParamCreator(configuration).blindTransferCall(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Create e call
         * @param {EngineCreateCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createCall(body, options) {
            const localVarAxiosArgs = await CallServiceApiAxiosParamCreator(configuration).createCall(body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {string} callId
         * @param {EngineCreateCallAnnotationRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createCallAnnotation(callId, body, options) {
            const localVarAxiosArgs = await CallServiceApiAxiosParamCreator(configuration).createCallAnnotation(callId, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {string} callId
         * @param {string} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteCallAnnotation(callId, id, options) {
            const localVarAxiosArgs = await CallServiceApiAxiosParamCreator(configuration).deleteCallAnnotation(callId, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {string} id
         * @param {EngineDtmfCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async dtmfCall(id, body, options) {
            const localVarAxiosArgs = await CallServiceApiAxiosParamCreator(configuration).dtmfCall(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {string} id
         * @param {EngineEavesdropCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async eavesdropCall(id, body, options) {
            const localVarAxiosArgs = await CallServiceApiAxiosParamCreator(configuration).eavesdropCall(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {string} id
         * @param {EngineHangupCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async hangupCall(id, body, options) {
            const localVarAxiosArgs = await CallServiceApiAxiosParamCreator(configuration).hangupCall(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {string} id
         * @param {EngineUserCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async holdCall(id, body, options) {
            const localVarAxiosArgs = await CallServiceApiAxiosParamCreator(configuration).holdCall(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Call item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readCall(id, domainId, options) {
            const localVarAxiosArgs = await CallServiceApiAxiosParamCreator(configuration).readCall(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {string} [createdAtFrom]
         * @param {string} [createdAtTo]
         * @param {Array<string>} [userId]
         * @param {Array<string>} [agentId]
         * @param {Array<string>} [queueId]
         * @param {Array<string>} [teamId]
         * @param {Array<string>} [memberId]
         * @param {Array<string>} [gatewayId]
         * @param {string} [durationFrom]
         * @param {string} [durationTo]
         * @param {boolean} [skipParent]
         * @param {string} [parentId]
         * @param {Array<string>} [cause]
         * @param {boolean} [existsFile]
         * @param {string} [domainId]
         * @param {string} [number]
         * @param {Array<string>} [direction]
         * @param {string} [answeredAtFrom]
         * @param {string} [answeredAtTo]
         * @param {boolean} [missed]
         * @param {string} [storedAtFrom]
         * @param {string} [storedAtTo]
         * @param {Array<string>} [supervisorId]
         * @param {Array<string>} [state]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchActiveCall(page, size, q, sort, fields, createdAtFrom, createdAtTo, userId, agentId, queueId, teamId, memberId, gatewayId, durationFrom, durationTo, skipParent, parentId, cause, existsFile, domainId, number, direction, answeredAtFrom, answeredAtTo, missed, storedAtFrom, storedAtTo, supervisorId, state, options) {
            const localVarAxiosArgs = await CallServiceApiAxiosParamCreator(configuration).searchActiveCall(page, size, q, sort, fields, createdAtFrom, createdAtTo, userId, agentId, queueId, teamId, memberId, gatewayId, durationFrom, durationTo, skipParent, parentId, cause, existsFile, domainId, number, direction, answeredAtFrom, answeredAtTo, missed, storedAtFrom, storedAtTo, supervisorId, state, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List of call
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {string} [createdAtFrom]
         * @param {string} [createdAtTo]
         * @param {Array<string>} [userId]
         * @param {Array<string>} [agentId]
         * @param {Array<string>} [queueId]
         * @param {Array<string>} [teamId]
         * @param {Array<string>} [memberId]
         * @param {Array<string>} [gatewayId]
         * @param {string} [durationFrom]
         * @param {string} [durationTo]
         * @param {boolean} [skipParent]
         * @param {string} [parentId]
         * @param {Array<string>} [cause]
         * @param {boolean} [hasFile]
         * @param {string} [domainId]
         * @param {string} [number]
         * @param {string} [direction]
         * @param {string} [answeredAtFrom]
         * @param {string} [answeredAtTo]
         * @param {boolean} [missed]
         * @param {string} [storedAtFrom]
         * @param {string} [storedAtTo]
         * @param {Array<string>} [id]
         * @param {Array<string>} [transferFrom]
         * @param {Array<string>} [transferTo]
         * @param {Array<string>} [dependencyId]
         * @param {Array<string>} [tags]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchHistoryCall(page, size, q, sort, fields, createdAtFrom, createdAtTo, userId, agentId, queueId, teamId, memberId, gatewayId, durationFrom, durationTo, skipParent, parentId, cause, hasFile, domainId, number, direction, answeredAtFrom, answeredAtTo, missed, storedAtFrom, storedAtTo, id, transferFrom, transferTo, dependencyId, tags, options) {
            const localVarAxiosArgs = await CallServiceApiAxiosParamCreator(configuration).searchHistoryCall(page, size, q, sort, fields, createdAtFrom, createdAtTo, userId, agentId, queueId, teamId, memberId, gatewayId, durationFrom, durationTo, skipParent, parentId, cause, hasFile, domainId, number, direction, answeredAtFrom, answeredAtTo, missed, storedAtFrom, storedAtTo, id, transferFrom, transferTo, dependencyId, tags, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {string} id
         * @param {EngineUserCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async unHoldCall(id, body, options) {
            const localVarAxiosArgs = await CallServiceApiAxiosParamCreator(configuration).unHoldCall(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {string} callId
         * @param {string} id
         * @param {EngineUpdateCallAnnotationRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateCallAnnotation(callId, id, body, options) {
            const localVarAxiosArgs = await CallServiceApiAxiosParamCreator(configuration).updateCallAnnotation(callId, id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * CallServiceApi - factory interface
 * @export
 */
const CallServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @param {EngineAggregateHistoryCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        aggregateHistoryCall(body, options) {
            return CallServiceApiFp(configuration)
                .aggregateHistoryCall(body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {string} id
         * @param {EngineBlindTransferCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        blindTransferCall(id, body, options) {
            return CallServiceApiFp(configuration)
                .blindTransferCall(id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Create e call
         * @param {EngineCreateCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createCall(body, options) {
            return CallServiceApiFp(configuration)
                .createCall(body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {string} callId
         * @param {EngineCreateCallAnnotationRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createCallAnnotation(callId, body, options) {
            return CallServiceApiFp(configuration)
                .createCallAnnotation(callId, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {string} callId
         * @param {string} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteCallAnnotation(callId, id, options) {
            return CallServiceApiFp(configuration)
                .deleteCallAnnotation(callId, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {string} id
         * @param {EngineDtmfCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        dtmfCall(id, body, options) {
            return CallServiceApiFp(configuration)
                .dtmfCall(id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {string} id
         * @param {EngineEavesdropCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        eavesdropCall(id, body, options) {
            return CallServiceApiFp(configuration)
                .eavesdropCall(id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {string} id
         * @param {EngineHangupCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        hangupCall(id, body, options) {
            return CallServiceApiFp(configuration)
                .hangupCall(id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {string} id
         * @param {EngineUserCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        holdCall(id, body, options) {
            return CallServiceApiFp(configuration)
                .holdCall(id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Call item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readCall(id, domainId, options) {
            return CallServiceApiFp(configuration)
                .readCall(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {string} [createdAtFrom]
         * @param {string} [createdAtTo]
         * @param {Array<string>} [userId]
         * @param {Array<string>} [agentId]
         * @param {Array<string>} [queueId]
         * @param {Array<string>} [teamId]
         * @param {Array<string>} [memberId]
         * @param {Array<string>} [gatewayId]
         * @param {string} [durationFrom]
         * @param {string} [durationTo]
         * @param {boolean} [skipParent]
         * @param {string} [parentId]
         * @param {Array<string>} [cause]
         * @param {boolean} [existsFile]
         * @param {string} [domainId]
         * @param {string} [number]
         * @param {Array<string>} [direction]
         * @param {string} [answeredAtFrom]
         * @param {string} [answeredAtTo]
         * @param {boolean} [missed]
         * @param {string} [storedAtFrom]
         * @param {string} [storedAtTo]
         * @param {Array<string>} [supervisorId]
         * @param {Array<string>} [state]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchActiveCall(page, size, q, sort, fields, createdAtFrom, createdAtTo, userId, agentId, queueId, teamId, memberId, gatewayId, durationFrom, durationTo, skipParent, parentId, cause, existsFile, domainId, number, direction, answeredAtFrom, answeredAtTo, missed, storedAtFrom, storedAtTo, supervisorId, state, options) {
            return CallServiceApiFp(configuration)
                .searchActiveCall(page, size, q, sort, fields, createdAtFrom, createdAtTo, userId, agentId, queueId, teamId, memberId, gatewayId, durationFrom, durationTo, skipParent, parentId, cause, existsFile, domainId, number, direction, answeredAtFrom, answeredAtTo, missed, storedAtFrom, storedAtTo, supervisorId, state, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List of call
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {string} [createdAtFrom]
         * @param {string} [createdAtTo]
         * @param {Array<string>} [userId]
         * @param {Array<string>} [agentId]
         * @param {Array<string>} [queueId]
         * @param {Array<string>} [teamId]
         * @param {Array<string>} [memberId]
         * @param {Array<string>} [gatewayId]
         * @param {string} [durationFrom]
         * @param {string} [durationTo]
         * @param {boolean} [skipParent]
         * @param {string} [parentId]
         * @param {Array<string>} [cause]
         * @param {boolean} [hasFile]
         * @param {string} [domainId]
         * @param {string} [number]
         * @param {string} [direction]
         * @param {string} [answeredAtFrom]
         * @param {string} [answeredAtTo]
         * @param {boolean} [missed]
         * @param {string} [storedAtFrom]
         * @param {string} [storedAtTo]
         * @param {Array<string>} [id]
         * @param {Array<string>} [transferFrom]
         * @param {Array<string>} [transferTo]
         * @param {Array<string>} [dependencyId]
         * @param {Array<string>} [tags]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchHistoryCall(page, size, q, sort, fields, createdAtFrom, createdAtTo, userId, agentId, queueId, teamId, memberId, gatewayId, durationFrom, durationTo, skipParent, parentId, cause, hasFile, domainId, number, direction, answeredAtFrom, answeredAtTo, missed, storedAtFrom, storedAtTo, id, transferFrom, transferTo, dependencyId, tags, options) {
            return CallServiceApiFp(configuration)
                .searchHistoryCall(page, size, q, sort, fields, createdAtFrom, createdAtTo, userId, agentId, queueId, teamId, memberId, gatewayId, durationFrom, durationTo, skipParent, parentId, cause, hasFile, domainId, number, direction, answeredAtFrom, answeredAtTo, missed, storedAtFrom, storedAtTo, id, transferFrom, transferTo, dependencyId, tags, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {string} id
         * @param {EngineUserCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        unHoldCall(id, body, options) {
            return CallServiceApiFp(configuration)
                .unHoldCall(id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {string} callId
         * @param {string} id
         * @param {EngineUpdateCallAnnotationRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateCallAnnotation(callId, id, body, options) {
            return CallServiceApiFp(configuration)
                .updateCallAnnotation(callId, id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * CallServiceApi - object-oriented interface
 * @export
 * @class CallServiceApi
 * @extends {BaseAPI}
 */
class CallServiceApi extends BaseAPI {
    /**
     *
     * @param {EngineAggregateHistoryCallRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CallServiceApi
     */
    aggregateHistoryCall(body, options) {
        return CallServiceApiFp(this.configuration)
            .aggregateHistoryCall(body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {string} id
     * @param {EngineBlindTransferCallRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CallServiceApi
     */
    blindTransferCall(id, body, options) {
        return CallServiceApiFp(this.configuration)
            .blindTransferCall(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Create e call
     * @param {EngineCreateCallRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CallServiceApi
     */
    createCall(body, options) {
        return CallServiceApiFp(this.configuration)
            .createCall(body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {string} callId
     * @param {EngineCreateCallAnnotationRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CallServiceApi
     */
    createCallAnnotation(callId, body, options) {
        return CallServiceApiFp(this.configuration)
            .createCallAnnotation(callId, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {string} callId
     * @param {string} id
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CallServiceApi
     */
    deleteCallAnnotation(callId, id, options) {
        return CallServiceApiFp(this.configuration)
            .deleteCallAnnotation(callId, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {string} id
     * @param {EngineDtmfCallRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CallServiceApi
     */
    dtmfCall(id, body, options) {
        return CallServiceApiFp(this.configuration)
            .dtmfCall(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {string} id
     * @param {EngineEavesdropCallRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CallServiceApi
     */
    eavesdropCall(id, body, options) {
        return CallServiceApiFp(this.configuration)
            .eavesdropCall(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {string} id
     * @param {EngineHangupCallRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CallServiceApi
     */
    hangupCall(id, body, options) {
        return CallServiceApiFp(this.configuration)
            .hangupCall(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {string} id
     * @param {EngineUserCallRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CallServiceApi
     */
    holdCall(id, body, options) {
        return CallServiceApiFp(this.configuration)
            .holdCall(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Call item
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CallServiceApi
     */
    readCall(id, domainId, options) {
        return CallServiceApiFp(this.configuration)
            .readCall(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {string} [createdAtFrom]
     * @param {string} [createdAtTo]
     * @param {Array<string>} [userId]
     * @param {Array<string>} [agentId]
     * @param {Array<string>} [queueId]
     * @param {Array<string>} [teamId]
     * @param {Array<string>} [memberId]
     * @param {Array<string>} [gatewayId]
     * @param {string} [durationFrom]
     * @param {string} [durationTo]
     * @param {boolean} [skipParent]
     * @param {string} [parentId]
     * @param {Array<string>} [cause]
     * @param {boolean} [existsFile]
     * @param {string} [domainId]
     * @param {string} [number]
     * @param {Array<string>} [direction]
     * @param {string} [answeredAtFrom]
     * @param {string} [answeredAtTo]
     * @param {boolean} [missed]
     * @param {string} [storedAtFrom]
     * @param {string} [storedAtTo]
     * @param {Array<string>} [supervisorId]
     * @param {Array<string>} [state]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CallServiceApi
     */
    searchActiveCall(page, size, q, sort, fields, createdAtFrom, createdAtTo, userId, agentId, queueId, teamId, memberId, gatewayId, durationFrom, durationTo, skipParent, parentId, cause, existsFile, domainId, number, direction, answeredAtFrom, answeredAtTo, missed, storedAtFrom, storedAtTo, supervisorId, state, options) {
        return CallServiceApiFp(this.configuration)
            .searchActiveCall(page, size, q, sort, fields, createdAtFrom, createdAtTo, userId, agentId, queueId, teamId, memberId, gatewayId, durationFrom, durationTo, skipParent, parentId, cause, existsFile, domainId, number, direction, answeredAtFrom, answeredAtTo, missed, storedAtFrom, storedAtTo, supervisorId, state, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List of call
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {string} [createdAtFrom]
     * @param {string} [createdAtTo]
     * @param {Array<string>} [userId]
     * @param {Array<string>} [agentId]
     * @param {Array<string>} [queueId]
     * @param {Array<string>} [teamId]
     * @param {Array<string>} [memberId]
     * @param {Array<string>} [gatewayId]
     * @param {string} [durationFrom]
     * @param {string} [durationTo]
     * @param {boolean} [skipParent]
     * @param {string} [parentId]
     * @param {Array<string>} [cause]
     * @param {boolean} [hasFile]
     * @param {string} [domainId]
     * @param {string} [number]
     * @param {string} [direction]
     * @param {string} [answeredAtFrom]
     * @param {string} [answeredAtTo]
     * @param {boolean} [missed]
     * @param {string} [storedAtFrom]
     * @param {string} [storedAtTo]
     * @param {Array<string>} [id]
     * @param {Array<string>} [transferFrom]
     * @param {Array<string>} [transferTo]
     * @param {Array<string>} [dependencyId]
     * @param {Array<string>} [tags]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CallServiceApi
     */
    searchHistoryCall(page, size, q, sort, fields, createdAtFrom, createdAtTo, userId, agentId, queueId, teamId, memberId, gatewayId, durationFrom, durationTo, skipParent, parentId, cause, hasFile, domainId, number, direction, answeredAtFrom, answeredAtTo, missed, storedAtFrom, storedAtTo, id, transferFrom, transferTo, dependencyId, tags, options) {
        return CallServiceApiFp(this.configuration)
            .searchHistoryCall(page, size, q, sort, fields, createdAtFrom, createdAtTo, userId, agentId, queueId, teamId, memberId, gatewayId, durationFrom, durationTo, skipParent, parentId, cause, hasFile, domainId, number, direction, answeredAtFrom, answeredAtTo, missed, storedAtFrom, storedAtTo, id, transferFrom, transferTo, dependencyId, tags, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {string} id
     * @param {EngineUserCallRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CallServiceApi
     */
    unHoldCall(id, body, options) {
        return CallServiceApiFp(this.configuration)
            .unHoldCall(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {string} callId
     * @param {string} id
     * @param {EngineUpdateCallAnnotationRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CallServiceApi
     */
    updateCallAnnotation(callId, id, body, options) {
        return CallServiceApiFp(this.configuration)
            .updateCallAnnotation(callId, id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * CommunicationTypeServiceApi - axios parameter creator
 * @export
 */
const CommunicationTypeServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @summary Create CommunicationType
         * @param {EngineCommunicationTypeRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createCommunicationType: async (body, options = {}) => {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createCommunicationType.');
            }
            const localVarPath = `/call_center/communication_type`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Remove CommunicationType
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteCommunicationType: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteCommunicationType.');
            }
            const localVarPath = `/call_center/communication_type/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary CommunicationType item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readCommunicationType: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readCommunicationType.');
            }
            const localVarPath = `/call_center/communication_type/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List of CommunicationType
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchCommunicationType: async (page, size, q, sort, fields, id, options = {}) => {
            const localVarPath = `/call_center/communication_type`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update CommunicationType
         * @param {string} id
         * @param {EngineUpdateCommunicationTypeRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateCommunicationType: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateCommunicationType.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateCommunicationType.');
            }
            const localVarPath = `/call_center/communication_type/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * CommunicationTypeServiceApi - functional programming interface
 * @export
 */
const CommunicationTypeServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @summary Create CommunicationType
         * @param {EngineCommunicationTypeRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createCommunicationType(body, options) {
            const localVarAxiosArgs = await CommunicationTypeServiceApiAxiosParamCreator(configuration).createCommunicationType(body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Remove CommunicationType
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteCommunicationType(id, domainId, options) {
            const localVarAxiosArgs = await CommunicationTypeServiceApiAxiosParamCreator(configuration).deleteCommunicationType(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary CommunicationType item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readCommunicationType(id, domainId, options) {
            const localVarAxiosArgs = await CommunicationTypeServiceApiAxiosParamCreator(configuration).readCommunicationType(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List of CommunicationType
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchCommunicationType(page, size, q, sort, fields, id, options) {
            const localVarAxiosArgs = await CommunicationTypeServiceApiAxiosParamCreator(configuration).searchCommunicationType(page, size, q, sort, fields, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update CommunicationType
         * @param {string} id
         * @param {EngineUpdateCommunicationTypeRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateCommunicationType(id, body, options) {
            const localVarAxiosArgs = await CommunicationTypeServiceApiAxiosParamCreator(configuration).updateCommunicationType(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * CommunicationTypeServiceApi - factory interface
 * @export
 */
const CommunicationTypeServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @summary Create CommunicationType
         * @param {EngineCommunicationTypeRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createCommunicationType(body, options) {
            return CommunicationTypeServiceApiFp(configuration)
                .createCommunicationType(body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Remove CommunicationType
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteCommunicationType(id, domainId, options) {
            return CommunicationTypeServiceApiFp(configuration)
                .deleteCommunicationType(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary CommunicationType item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readCommunicationType(id, domainId, options) {
            return CommunicationTypeServiceApiFp(configuration)
                .readCommunicationType(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List of CommunicationType
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchCommunicationType(page, size, q, sort, fields, id, options) {
            return CommunicationTypeServiceApiFp(configuration)
                .searchCommunicationType(page, size, q, sort, fields, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update CommunicationType
         * @param {string} id
         * @param {EngineUpdateCommunicationTypeRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateCommunicationType(id, body, options) {
            return CommunicationTypeServiceApiFp(configuration)
                .updateCommunicationType(id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * CommunicationTypeServiceApi - object-oriented interface
 * @export
 * @class CommunicationTypeServiceApi
 * @extends {BaseAPI}
 */
class CommunicationTypeServiceApi extends BaseAPI {
    /**
     *
     * @summary Create CommunicationType
     * @param {EngineCommunicationTypeRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CommunicationTypeServiceApi
     */
    createCommunicationType(body, options) {
        return CommunicationTypeServiceApiFp(this.configuration)
            .createCommunicationType(body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Remove CommunicationType
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CommunicationTypeServiceApi
     */
    deleteCommunicationType(id, domainId, options) {
        return CommunicationTypeServiceApiFp(this.configuration)
            .deleteCommunicationType(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary CommunicationType item
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CommunicationTypeServiceApi
     */
    readCommunicationType(id, domainId, options) {
        return CommunicationTypeServiceApiFp(this.configuration)
            .readCommunicationType(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List of CommunicationType
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CommunicationTypeServiceApi
     */
    searchCommunicationType(page, size, q, sort, fields, id, options) {
        return CommunicationTypeServiceApiFp(this.configuration)
            .searchCommunicationType(page, size, q, sort, fields, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update CommunicationType
     * @param {string} id
     * @param {EngineUpdateCommunicationTypeRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof CommunicationTypeServiceApi
     */
    updateCommunicationType(id, body, options) {
        return CommunicationTypeServiceApiFp(this.configuration)
            .updateCommunicationType(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * EmailProfileServiceApi - axios parameter creator
 * @export
 */
const EmailProfileServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @summary Create EmailProfile
         * @param {EngineCreateEmailProfileRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createEmailProfile: async (body, options = {}) => {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createEmailProfile.');
            }
            const localVarPath = `/email/profile`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Remove EmailProfile
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteEmailProfile: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteEmailProfile.');
            }
            const localVarPath = `/email/profile/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary EmailProfile item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readEmailProfile: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readEmailProfile.');
            }
            const localVarPath = `/email/profile/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Search EmailProfile
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchEmailProfile: async (page, size, q, sort, fields, domainId, options = {}) => {
            const localVarPath = `/email/profile`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update EmailProfile
         * @param {string} id
         * @param {EngineUpdateEmailProfileRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateEmailProfile: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateEmailProfile.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateEmailProfile.');
            }
            const localVarPath = `/email/profile/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * EmailProfileServiceApi - functional programming interface
 * @export
 */
const EmailProfileServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @summary Create EmailProfile
         * @param {EngineCreateEmailProfileRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createEmailProfile(body, options) {
            const localVarAxiosArgs = await EmailProfileServiceApiAxiosParamCreator(configuration).createEmailProfile(body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Remove EmailProfile
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteEmailProfile(id, domainId, options) {
            const localVarAxiosArgs = await EmailProfileServiceApiAxiosParamCreator(configuration).deleteEmailProfile(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary EmailProfile item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readEmailProfile(id, domainId, options) {
            const localVarAxiosArgs = await EmailProfileServiceApiAxiosParamCreator(configuration).readEmailProfile(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Search EmailProfile
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchEmailProfile(page, size, q, sort, fields, domainId, options) {
            const localVarAxiosArgs = await EmailProfileServiceApiAxiosParamCreator(configuration).searchEmailProfile(page, size, q, sort, fields, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update EmailProfile
         * @param {string} id
         * @param {EngineUpdateEmailProfileRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateEmailProfile(id, body, options) {
            const localVarAxiosArgs = await EmailProfileServiceApiAxiosParamCreator(configuration).updateEmailProfile(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * EmailProfileServiceApi - factory interface
 * @export
 */
const EmailProfileServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @summary Create EmailProfile
         * @param {EngineCreateEmailProfileRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createEmailProfile(body, options) {
            return EmailProfileServiceApiFp(configuration)
                .createEmailProfile(body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Remove EmailProfile
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteEmailProfile(id, domainId, options) {
            return EmailProfileServiceApiFp(configuration)
                .deleteEmailProfile(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary EmailProfile item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readEmailProfile(id, domainId, options) {
            return EmailProfileServiceApiFp(configuration)
                .readEmailProfile(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Search EmailProfile
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchEmailProfile(page, size, q, sort, fields, domainId, options) {
            return EmailProfileServiceApiFp(configuration)
                .searchEmailProfile(page, size, q, sort, fields, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update EmailProfile
         * @param {string} id
         * @param {EngineUpdateEmailProfileRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateEmailProfile(id, body, options) {
            return EmailProfileServiceApiFp(configuration)
                .updateEmailProfile(id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * EmailProfileServiceApi - object-oriented interface
 * @export
 * @class EmailProfileServiceApi
 * @extends {BaseAPI}
 */
class EmailProfileServiceApi extends BaseAPI {
    /**
     *
     * @summary Create EmailProfile
     * @param {EngineCreateEmailProfileRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof EmailProfileServiceApi
     */
    createEmailProfile(body, options) {
        return EmailProfileServiceApiFp(this.configuration)
            .createEmailProfile(body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Remove EmailProfile
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof EmailProfileServiceApi
     */
    deleteEmailProfile(id, domainId, options) {
        return EmailProfileServiceApiFp(this.configuration)
            .deleteEmailProfile(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary EmailProfile item
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof EmailProfileServiceApi
     */
    readEmailProfile(id, domainId, options) {
        return EmailProfileServiceApiFp(this.configuration)
            .readEmailProfile(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Search EmailProfile
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof EmailProfileServiceApi
     */
    searchEmailProfile(page, size, q, sort, fields, domainId, options) {
        return EmailProfileServiceApiFp(this.configuration)
            .searchEmailProfile(page, size, q, sort, fields, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update EmailProfile
     * @param {string} id
     * @param {EngineUpdateEmailProfileRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof EmailProfileServiceApi
     */
    updateEmailProfile(id, body, options) {
        return EmailProfileServiceApiFp(this.configuration)
            .updateEmailProfile(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * ListServiceApi - axios parameter creator
 * @export
 */
const ListServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @summary Create List
         * @param {EngineCreateListRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createList: async (body, options = {}) => {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createList.');
            }
            const localVarPath = `/call_center/list`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Create ListCommunication
         * @param {string} listId
         * @param {EngineCreateListCommunicationRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createListCommunication: async (listId, body, options = {}) => {
            // verify required parameter 'listId' is not null or undefined
            if (listId === null || listId === undefined) {
                throw new RequiredError('listId', 'Required parameter listId was null or undefined when calling createListCommunication.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createListCommunication.');
            }
            const localVarPath = `/call_center/list/{list_id}/communication`.replace(`{${'list_id'}}`, encodeURIComponent(String(listId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Remove list
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteList: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteList.');
            }
            const localVarPath = `/call_center/list/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Remove ListCommunication
         * @param {string} listId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteListCommunication: async (listId, id, domainId, options = {}) => {
            // verify required parameter 'listId' is not null or undefined
            if (listId === null || listId === undefined) {
                throw new RequiredError('listId', 'Required parameter listId was null or undefined when calling deleteListCommunication.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteListCommunication.');
            }
            const localVarPath = `/call_center/list/{list_id}/communication/{id}`
                .replace(`{${'list_id'}}`, encodeURIComponent(String(listId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readList: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readList.');
            }
            const localVarPath = `/call_center/list/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List item
         * @param {string} listId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readListCommunication: async (listId, id, domainId, options = {}) => {
            // verify required parameter 'listId' is not null or undefined
            if (listId === null || listId === undefined) {
                throw new RequiredError('listId', 'Required parameter listId was null or undefined when calling readListCommunication.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readListCommunication.');
            }
            const localVarPath = `/call_center/list/{list_id}/communication/{id}`
                .replace(`{${'list_id'}}`, encodeURIComponent(String(listId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List of List
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchList: async (page, size, q, sort, fields, id, options = {}) => {
            const localVarPath = `/call_center/list`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List of List
         * @param {string} listId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchListCommunication: async (listId, page, size, q, sort, fields, id, options = {}) => {
            // verify required parameter 'listId' is not null or undefined
            if (listId === null || listId === undefined) {
                throw new RequiredError('listId', 'Required parameter listId was null or undefined when calling searchListCommunication.');
            }
            const localVarPath = `/call_center/list/{list_id}/communication`.replace(`{${'list_id'}}`, encodeURIComponent(String(listId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update list
         * @param {string} id
         * @param {EngineUpdateListRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateList: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateList.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateList.');
            }
            const localVarPath = `/call_center/list/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update ListCommunication
         * @param {string} listId
         * @param {string} id
         * @param {EngineUpdateListCommunicationRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateListCommunication: async (listId, id, body, options = {}) => {
            // verify required parameter 'listId' is not null or undefined
            if (listId === null || listId === undefined) {
                throw new RequiredError('listId', 'Required parameter listId was null or undefined when calling updateListCommunication.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateListCommunication.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateListCommunication.');
            }
            const localVarPath = `/call_center/list/{list_id}/communication/{id}`
                .replace(`{${'list_id'}}`, encodeURIComponent(String(listId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * ListServiceApi - functional programming interface
 * @export
 */
const ListServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @summary Create List
         * @param {EngineCreateListRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createList(body, options) {
            const localVarAxiosArgs = await ListServiceApiAxiosParamCreator(configuration).createList(body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Create ListCommunication
         * @param {string} listId
         * @param {EngineCreateListCommunicationRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createListCommunication(listId, body, options) {
            const localVarAxiosArgs = await ListServiceApiAxiosParamCreator(configuration).createListCommunication(listId, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Remove list
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteList(id, domainId, options) {
            const localVarAxiosArgs = await ListServiceApiAxiosParamCreator(configuration).deleteList(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Remove ListCommunication
         * @param {string} listId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteListCommunication(listId, id, domainId, options) {
            const localVarAxiosArgs = await ListServiceApiAxiosParamCreator(configuration).deleteListCommunication(listId, id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readList(id, domainId, options) {
            const localVarAxiosArgs = await ListServiceApiAxiosParamCreator(configuration).readList(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List item
         * @param {string} listId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readListCommunication(listId, id, domainId, options) {
            const localVarAxiosArgs = await ListServiceApiAxiosParamCreator(configuration).readListCommunication(listId, id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List of List
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchList(page, size, q, sort, fields, id, options) {
            const localVarAxiosArgs = await ListServiceApiAxiosParamCreator(configuration).searchList(page, size, q, sort, fields, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List of List
         * @param {string} listId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchListCommunication(listId, page, size, q, sort, fields, id, options) {
            const localVarAxiosArgs = await ListServiceApiAxiosParamCreator(configuration).searchListCommunication(listId, page, size, q, sort, fields, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update list
         * @param {string} id
         * @param {EngineUpdateListRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateList(id, body, options) {
            const localVarAxiosArgs = await ListServiceApiAxiosParamCreator(configuration).updateList(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update ListCommunication
         * @param {string} listId
         * @param {string} id
         * @param {EngineUpdateListCommunicationRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateListCommunication(listId, id, body, options) {
            const localVarAxiosArgs = await ListServiceApiAxiosParamCreator(configuration).updateListCommunication(listId, id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * ListServiceApi - factory interface
 * @export
 */
const ListServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @summary Create List
         * @param {EngineCreateListRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createList(body, options) {
            return ListServiceApiFp(configuration)
                .createList(body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Create ListCommunication
         * @param {string} listId
         * @param {EngineCreateListCommunicationRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createListCommunication(listId, body, options) {
            return ListServiceApiFp(configuration)
                .createListCommunication(listId, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Remove list
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteList(id, domainId, options) {
            return ListServiceApiFp(configuration)
                .deleteList(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Remove ListCommunication
         * @param {string} listId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteListCommunication(listId, id, domainId, options) {
            return ListServiceApiFp(configuration)
                .deleteListCommunication(listId, id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readList(id, domainId, options) {
            return ListServiceApiFp(configuration)
                .readList(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List item
         * @param {string} listId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readListCommunication(listId, id, domainId, options) {
            return ListServiceApiFp(configuration)
                .readListCommunication(listId, id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List of List
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchList(page, size, q, sort, fields, id, options) {
            return ListServiceApiFp(configuration)
                .searchList(page, size, q, sort, fields, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List of List
         * @param {string} listId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchListCommunication(listId, page, size, q, sort, fields, id, options) {
            return ListServiceApiFp(configuration)
                .searchListCommunication(listId, page, size, q, sort, fields, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update list
         * @param {string} id
         * @param {EngineUpdateListRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateList(id, body, options) {
            return ListServiceApiFp(configuration)
                .updateList(id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update ListCommunication
         * @param {string} listId
         * @param {string} id
         * @param {EngineUpdateListCommunicationRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateListCommunication(listId, id, body, options) {
            return ListServiceApiFp(configuration)
                .updateListCommunication(listId, id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * ListServiceApi - object-oriented interface
 * @export
 * @class ListServiceApi
 * @extends {BaseAPI}
 */
class ListServiceApi extends BaseAPI {
    /**
     *
     * @summary Create List
     * @param {EngineCreateListRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof ListServiceApi
     */
    createList(body, options) {
        return ListServiceApiFp(this.configuration)
            .createList(body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Create ListCommunication
     * @param {string} listId
     * @param {EngineCreateListCommunicationRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof ListServiceApi
     */
    createListCommunication(listId, body, options) {
        return ListServiceApiFp(this.configuration)
            .createListCommunication(listId, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Remove list
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof ListServiceApi
     */
    deleteList(id, domainId, options) {
        return ListServiceApiFp(this.configuration)
            .deleteList(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Remove ListCommunication
     * @param {string} listId
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof ListServiceApi
     */
    deleteListCommunication(listId, id, domainId, options) {
        return ListServiceApiFp(this.configuration)
            .deleteListCommunication(listId, id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List item
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof ListServiceApi
     */
    readList(id, domainId, options) {
        return ListServiceApiFp(this.configuration)
            .readList(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List item
     * @param {string} listId
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof ListServiceApi
     */
    readListCommunication(listId, id, domainId, options) {
        return ListServiceApiFp(this.configuration)
            .readListCommunication(listId, id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List of List
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof ListServiceApi
     */
    searchList(page, size, q, sort, fields, id, options) {
        return ListServiceApiFp(this.configuration)
            .searchList(page, size, q, sort, fields, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List of List
     * @param {string} listId
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof ListServiceApi
     */
    searchListCommunication(listId, page, size, q, sort, fields, id, options) {
        return ListServiceApiFp(this.configuration)
            .searchListCommunication(listId, page, size, q, sort, fields, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update list
     * @param {string} id
     * @param {EngineUpdateListRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof ListServiceApi
     */
    updateList(id, body, options) {
        return ListServiceApiFp(this.configuration)
            .updateList(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update ListCommunication
     * @param {string} listId
     * @param {string} id
     * @param {EngineUpdateListCommunicationRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof ListServiceApi
     */
    updateListCommunication(listId, id, body, options) {
        return ListServiceApiFp(this.configuration)
            .updateListCommunication(listId, id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * MediaFileServiceApi - axios parameter creator
 * @export
 */
const MediaFileServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @summary Remove MediaFile
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteMediaFile: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteMediaFile.');
            }
            const localVarPath = `/storage/media/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary MediaFile item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readMediaFile: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readMediaFile.');
            }
            const localVarPath = `/storage/media/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Search MediaFile
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchMediaFile: async (page, size, q, sort, fields, id, options = {}) => {
            const localVarPath = `/storage/media`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * MediaFileServiceApi - functional programming interface
 * @export
 */
const MediaFileServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @summary Remove MediaFile
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteMediaFile(id, domainId, options) {
            const localVarAxiosArgs = await MediaFileServiceApiAxiosParamCreator(configuration).deleteMediaFile(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary MediaFile item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readMediaFile(id, domainId, options) {
            const localVarAxiosArgs = await MediaFileServiceApiAxiosParamCreator(configuration).readMediaFile(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Search MediaFile
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchMediaFile(page, size, q, sort, fields, id, options) {
            const localVarAxiosArgs = await MediaFileServiceApiAxiosParamCreator(configuration).searchMediaFile(page, size, q, sort, fields, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * MediaFileServiceApi - factory interface
 * @export
 */
const MediaFileServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @summary Remove MediaFile
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteMediaFile(id, domainId, options) {
            return MediaFileServiceApiFp(configuration)
                .deleteMediaFile(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary MediaFile item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readMediaFile(id, domainId, options) {
            return MediaFileServiceApiFp(configuration)
                .readMediaFile(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Search MediaFile
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchMediaFile(page, size, q, sort, fields, id, options) {
            return MediaFileServiceApiFp(configuration)
                .searchMediaFile(page, size, q, sort, fields, id, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * MediaFileServiceApi - object-oriented interface
 * @export
 * @class MediaFileServiceApi
 * @extends {BaseAPI}
 */
class MediaFileServiceApi extends BaseAPI {
    /**
     *
     * @summary Remove MediaFile
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MediaFileServiceApi
     */
    deleteMediaFile(id, domainId, options) {
        return MediaFileServiceApiFp(this.configuration)
            .deleteMediaFile(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary MediaFile item
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MediaFileServiceApi
     */
    readMediaFile(id, domainId, options) {
        return MediaFileServiceApiFp(this.configuration)
            .readMediaFile(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Search MediaFile
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MediaFileServiceApi
     */
    searchMediaFile(page, size, q, sort, fields, id, options) {
        return MediaFileServiceApiFp(this.configuration)
            .searchMediaFile(page, size, q, sort, fields, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * MemberServiceApi - axios parameter creator
 * @export
 */
const MemberServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @param {string} attemptId
         * @param {EngineAttemptCallbackRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        attemptCallback: async (attemptId, body, options = {}) => {
            // verify required parameter 'attemptId' is not null or undefined
            if (attemptId === null || attemptId === undefined) {
                throw new RequiredError('attemptId', 'Required parameter attemptId was null or undefined when calling attemptCallback.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling attemptCallback.');
            }
            const localVarPath = `/call_center/attempts/{attempt_id}`.replace(`{${'attempt_id'}}`, encodeURIComponent(String(attemptId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {string} memberId
         * @param {string} attemptId
         * @param {EngineAttemptResultRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        attemptResult: async (queueId, memberId, attemptId, body, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling attemptResult.');
            }
            // verify required parameter 'memberId' is not null or undefined
            if (memberId === null || memberId === undefined) {
                throw new RequiredError('memberId', 'Required parameter memberId was null or undefined when calling attemptResult.');
            }
            // verify required parameter 'attemptId' is not null or undefined
            if (attemptId === null || attemptId === undefined) {
                throw new RequiredError('attemptId', 'Required parameter attemptId was null or undefined when calling attemptResult.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling attemptResult.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/members/{member_id}/attempts/{attempt_id}`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'member_id'}}`, encodeURIComponent(String(memberId)))
                .replace(`{${'attempt_id'}}`, encodeURIComponent(String(attemptId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {string} attemptId
         * @param {EngineAttemptRenewalResultRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        attemptsRenewalResult: async (attemptId, body, options = {}) => {
            // verify required parameter 'attemptId' is not null or undefined
            if (attemptId === null || attemptId === undefined) {
                throw new RequiredError('attemptId', 'Required parameter attemptId was null or undefined when calling attemptsRenewalResult.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling attemptsRenewalResult.');
            }
            const localVarPath = `/call_center/queues/attempts/{attempt_id}/renewal`.replace(`{${'attempt_id'}}`, encodeURIComponent(String(attemptId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Offline queue
         * @param {string} queueId
         * @param {string} memberId
         * @param {EngineCreateAttemptRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createAttempt: async (queueId, memberId, body, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling createAttempt.');
            }
            // verify required parameter 'memberId' is not null or undefined
            if (memberId === null || memberId === undefined) {
                throw new RequiredError('memberId', 'Required parameter memberId was null or undefined when calling createAttempt.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createAttempt.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/members/{member_id}/attempts`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'member_id'}}`, encodeURIComponent(String(memberId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Create Member
         * @param {string} queueId
         * @param {EngineCreateMemberRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createMember: async (queueId, body, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling createMember.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createMember.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/members`.replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Create Member
         * @param {string} queueId
         * @param {EngineCreateMemberBulkRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createMemberBulk: async (queueId, body, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling createMemberBulk.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createMemberBulk.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/members/bulk`.replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary DeleteMember
         * @param {string} queueId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteMember: async (queueId, id, domainId, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling deleteMember.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteMember.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/members/{id}`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary DeleteMembers
         * @param {string} queueId
         * @param {EngineDeleteMembersRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteMembers: async (queueId, body, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling deleteMembers.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling deleteMembers.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/members`.replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Patch Member
         * @param {string} queueId
         * @param {string} id
         * @param {EnginePatchMemberRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchMember: async (queueId, id, body, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling patchMember.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling patchMember.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling patchMember.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/members/{id}`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary ReadQueueRouting
         * @param {string} queueId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readMember: async (queueId, id, domainId, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling readMember.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readMember.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/members/{id}`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary ResetMembers
         * @param {string} queueId
         * @param {EngineResetMembersRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        resetMembers: async (queueId, body, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling resetMembers.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling resetMembers.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/members/reset`.replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary SearchAttempts
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [joinedAtFrom]
         * @param {string} [joinedAtTo]
         * @param {Array<string>} [id]
         * @param {Array<string>} [queueId]
         * @param {Array<string>} [bucketId]
         * @param {Array<string>} [memberId]
         * @param {Array<string>} [agentId]
         * @param {string} [result]
         * @param {Array<string>} [fields]
         * @param {string} [sort]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAttempts: async (page, size, joinedAtFrom, joinedAtTo, id, queueId, bucketId, memberId, agentId, result, fields, sort, domainId, options = {}) => {
            const localVarPath = `/call_center/queues/attempts/active`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (joinedAtFrom !== undefined) {
                localVarQueryParameter['joined_at.from'] = joinedAtFrom;
            }
            if (joinedAtTo !== undefined) {
                localVarQueryParameter['joined_at.to'] = joinedAtTo;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            if (queueId) {
                localVarQueryParameter['queue_id'] = queueId;
            }
            if (bucketId) {
                localVarQueryParameter['bucket_id'] = bucketId;
            }
            if (memberId) {
                localVarQueryParameter['member_id'] = memberId;
            }
            if (agentId) {
                localVarQueryParameter['agent_id'] = agentId;
            }
            if (result !== undefined) {
                localVarQueryParameter['result'] = result;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary SearchMemberAttempts
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [joinedAtFrom]
         * @param {string} [joinedAtTo]
         * @param {Array<string>} [id]
         * @param {Array<string>} [queueId]
         * @param {Array<string>} [bucketId]
         * @param {Array<string>} [memberId]
         * @param {Array<string>} [agentId]
         * @param {string} [result]
         * @param {Array<string>} [fields]
         * @param {string} [sort]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAttemptsHistory: async (page, size, joinedAtFrom, joinedAtTo, id, queueId, bucketId, memberId, agentId, result, fields, sort, domainId, options = {}) => {
            const localVarPath = `/call_center/queues/attempts/history`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (joinedAtFrom !== undefined) {
                localVarQueryParameter['joined_at.from'] = joinedAtFrom;
            }
            if (joinedAtTo !== undefined) {
                localVarQueryParameter['joined_at.to'] = joinedAtTo;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            if (queueId) {
                localVarQueryParameter['queue_id'] = queueId;
            }
            if (bucketId) {
                localVarQueryParameter['bucket_id'] = bucketId;
            }
            if (memberId) {
                localVarQueryParameter['member_id'] = memberId;
            }
            if (agentId) {
                localVarQueryParameter['agent_id'] = agentId;
            }
            if (result !== undefined) {
                localVarQueryParameter['result'] = result;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary SearchMemberAttempts
         * @param {string} queueId
         * @param {string} memberId
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchMemberAttempts: async (queueId, memberId, domainId, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling searchMemberAttempts.');
            }
            // verify required parameter 'memberId' is not null or undefined
            if (memberId === null || memberId === undefined) {
                throw new RequiredError('memberId', 'Required parameter memberId was null or undefined when calling searchMemberAttempts.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/members/{member_id}/attempts`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'member_id'}}`, encodeURIComponent(String(memberId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List of Member
         * @param {string} queueId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchMemberInQueue: async (queueId, page, size, domainId, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling searchMemberInQueue.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/members`.replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Search of Member
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [id]
         * @param {string} [queueId]
         * @param {number} [bucketId]
         * @param {string} [destination]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchMembers: async (page, size, id, queueId, bucketId, destination, domainId, options = {}) => {
            const localVarPath = `/call_center/members`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (id !== undefined) {
                localVarQueryParameter['id'] = id;
            }
            if (queueId !== undefined) {
                localVarQueryParameter['queue_id'] = queueId;
            }
            if (bucketId !== undefined) {
                localVarQueryParameter['bucket_id'] = bucketId;
            }
            if (destination !== undefined) {
                localVarQueryParameter['destination'] = destination;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary UpdateMember
         * @param {string} queueId
         * @param {string} id
         * @param {EngineUpdateMemberRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateMember: async (queueId, id, body, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling updateMember.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateMember.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateMember.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/members/{id}`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * MemberServiceApi - functional programming interface
 * @export
 */
const MemberServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @param {string} attemptId
         * @param {EngineAttemptCallbackRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async attemptCallback(attemptId, body, options) {
            const localVarAxiosArgs = await MemberServiceApiAxiosParamCreator(configuration).attemptCallback(attemptId, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {string} memberId
         * @param {string} attemptId
         * @param {EngineAttemptResultRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async attemptResult(queueId, memberId, attemptId, body, options) {
            const localVarAxiosArgs = await MemberServiceApiAxiosParamCreator(configuration).attemptResult(queueId, memberId, attemptId, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {string} attemptId
         * @param {EngineAttemptRenewalResultRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async attemptsRenewalResult(attemptId, body, options) {
            const localVarAxiosArgs = await MemberServiceApiAxiosParamCreator(configuration).attemptsRenewalResult(attemptId, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Offline queue
         * @param {string} queueId
         * @param {string} memberId
         * @param {EngineCreateAttemptRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createAttempt(queueId, memberId, body, options) {
            const localVarAxiosArgs = await MemberServiceApiAxiosParamCreator(configuration).createAttempt(queueId, memberId, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Create Member
         * @param {string} queueId
         * @param {EngineCreateMemberRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createMember(queueId, body, options) {
            const localVarAxiosArgs = await MemberServiceApiAxiosParamCreator(configuration).createMember(queueId, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Create Member
         * @param {string} queueId
         * @param {EngineCreateMemberBulkRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createMemberBulk(queueId, body, options) {
            const localVarAxiosArgs = await MemberServiceApiAxiosParamCreator(configuration).createMemberBulk(queueId, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary DeleteMember
         * @param {string} queueId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteMember(queueId, id, domainId, options) {
            const localVarAxiosArgs = await MemberServiceApiAxiosParamCreator(configuration).deleteMember(queueId, id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary DeleteMembers
         * @param {string} queueId
         * @param {EngineDeleteMembersRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteMembers(queueId, body, options) {
            const localVarAxiosArgs = await MemberServiceApiAxiosParamCreator(configuration).deleteMembers(queueId, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Patch Member
         * @param {string} queueId
         * @param {string} id
         * @param {EnginePatchMemberRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async patchMember(queueId, id, body, options) {
            const localVarAxiosArgs = await MemberServiceApiAxiosParamCreator(configuration).patchMember(queueId, id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary ReadQueueRouting
         * @param {string} queueId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readMember(queueId, id, domainId, options) {
            const localVarAxiosArgs = await MemberServiceApiAxiosParamCreator(configuration).readMember(queueId, id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary ResetMembers
         * @param {string} queueId
         * @param {EngineResetMembersRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async resetMembers(queueId, body, options) {
            const localVarAxiosArgs = await MemberServiceApiAxiosParamCreator(configuration).resetMembers(queueId, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary SearchAttempts
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [joinedAtFrom]
         * @param {string} [joinedAtTo]
         * @param {Array<string>} [id]
         * @param {Array<string>} [queueId]
         * @param {Array<string>} [bucketId]
         * @param {Array<string>} [memberId]
         * @param {Array<string>} [agentId]
         * @param {string} [result]
         * @param {Array<string>} [fields]
         * @param {string} [sort]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchAttempts(page, size, joinedAtFrom, joinedAtTo, id, queueId, bucketId, memberId, agentId, result, fields, sort, domainId, options) {
            const localVarAxiosArgs = await MemberServiceApiAxiosParamCreator(configuration).searchAttempts(page, size, joinedAtFrom, joinedAtTo, id, queueId, bucketId, memberId, agentId, result, fields, sort, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary SearchMemberAttempts
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [joinedAtFrom]
         * @param {string} [joinedAtTo]
         * @param {Array<string>} [id]
         * @param {Array<string>} [queueId]
         * @param {Array<string>} [bucketId]
         * @param {Array<string>} [memberId]
         * @param {Array<string>} [agentId]
         * @param {string} [result]
         * @param {Array<string>} [fields]
         * @param {string} [sort]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchAttemptsHistory(page, size, joinedAtFrom, joinedAtTo, id, queueId, bucketId, memberId, agentId, result, fields, sort, domainId, options) {
            const localVarAxiosArgs = await MemberServiceApiAxiosParamCreator(configuration).searchAttemptsHistory(page, size, joinedAtFrom, joinedAtTo, id, queueId, bucketId, memberId, agentId, result, fields, sort, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary SearchMemberAttempts
         * @param {string} queueId
         * @param {string} memberId
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchMemberAttempts(queueId, memberId, domainId, options) {
            const localVarAxiosArgs = await MemberServiceApiAxiosParamCreator(configuration).searchMemberAttempts(queueId, memberId, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List of Member
         * @param {string} queueId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchMemberInQueue(queueId, page, size, domainId, options) {
            const localVarAxiosArgs = await MemberServiceApiAxiosParamCreator(configuration).searchMemberInQueue(queueId, page, size, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Search of Member
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [id]
         * @param {string} [queueId]
         * @param {number} [bucketId]
         * @param {string} [destination]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchMembers(page, size, id, queueId, bucketId, destination, domainId, options) {
            const localVarAxiosArgs = await MemberServiceApiAxiosParamCreator(configuration).searchMembers(page, size, id, queueId, bucketId, destination, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary UpdateMember
         * @param {string} queueId
         * @param {string} id
         * @param {EngineUpdateMemberRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateMember(queueId, id, body, options) {
            const localVarAxiosArgs = await MemberServiceApiAxiosParamCreator(configuration).updateMember(queueId, id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * MemberServiceApi - factory interface
 * @export
 */
const MemberServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @param {string} attemptId
         * @param {EngineAttemptCallbackRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        attemptCallback(attemptId, body, options) {
            return MemberServiceApiFp(configuration)
                .attemptCallback(attemptId, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} queueId
         * @param {string} memberId
         * @param {string} attemptId
         * @param {EngineAttemptResultRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        attemptResult(queueId, memberId, attemptId, body, options) {
            return MemberServiceApiFp(configuration)
                .attemptResult(queueId, memberId, attemptId, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {string} attemptId
         * @param {EngineAttemptRenewalResultRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        attemptsRenewalResult(attemptId, body, options) {
            return MemberServiceApiFp(configuration)
                .attemptsRenewalResult(attemptId, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Offline queue
         * @param {string} queueId
         * @param {string} memberId
         * @param {EngineCreateAttemptRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createAttempt(queueId, memberId, body, options) {
            return MemberServiceApiFp(configuration)
                .createAttempt(queueId, memberId, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Create Member
         * @param {string} queueId
         * @param {EngineCreateMemberRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createMember(queueId, body, options) {
            return MemberServiceApiFp(configuration)
                .createMember(queueId, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Create Member
         * @param {string} queueId
         * @param {EngineCreateMemberBulkRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createMemberBulk(queueId, body, options) {
            return MemberServiceApiFp(configuration)
                .createMemberBulk(queueId, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary DeleteMember
         * @param {string} queueId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteMember(queueId, id, domainId, options) {
            return MemberServiceApiFp(configuration)
                .deleteMember(queueId, id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary DeleteMembers
         * @param {string} queueId
         * @param {EngineDeleteMembersRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteMembers(queueId, body, options) {
            return MemberServiceApiFp(configuration)
                .deleteMembers(queueId, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Patch Member
         * @param {string} queueId
         * @param {string} id
         * @param {EnginePatchMemberRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchMember(queueId, id, body, options) {
            return MemberServiceApiFp(configuration)
                .patchMember(queueId, id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary ReadQueueRouting
         * @param {string} queueId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readMember(queueId, id, domainId, options) {
            return MemberServiceApiFp(configuration)
                .readMember(queueId, id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary ResetMembers
         * @param {string} queueId
         * @param {EngineResetMembersRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        resetMembers(queueId, body, options) {
            return MemberServiceApiFp(configuration)
                .resetMembers(queueId, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary SearchAttempts
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [joinedAtFrom]
         * @param {string} [joinedAtTo]
         * @param {Array<string>} [id]
         * @param {Array<string>} [queueId]
         * @param {Array<string>} [bucketId]
         * @param {Array<string>} [memberId]
         * @param {Array<string>} [agentId]
         * @param {string} [result]
         * @param {Array<string>} [fields]
         * @param {string} [sort]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAttempts(page, size, joinedAtFrom, joinedAtTo, id, queueId, bucketId, memberId, agentId, result, fields, sort, domainId, options) {
            return MemberServiceApiFp(configuration)
                .searchAttempts(page, size, joinedAtFrom, joinedAtTo, id, queueId, bucketId, memberId, agentId, result, fields, sort, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary SearchMemberAttempts
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [joinedAtFrom]
         * @param {string} [joinedAtTo]
         * @param {Array<string>} [id]
         * @param {Array<string>} [queueId]
         * @param {Array<string>} [bucketId]
         * @param {Array<string>} [memberId]
         * @param {Array<string>} [agentId]
         * @param {string} [result]
         * @param {Array<string>} [fields]
         * @param {string} [sort]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchAttemptsHistory(page, size, joinedAtFrom, joinedAtTo, id, queueId, bucketId, memberId, agentId, result, fields, sort, domainId, options) {
            return MemberServiceApiFp(configuration)
                .searchAttemptsHistory(page, size, joinedAtFrom, joinedAtTo, id, queueId, bucketId, memberId, agentId, result, fields, sort, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary SearchMemberAttempts
         * @param {string} queueId
         * @param {string} memberId
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchMemberAttempts(queueId, memberId, domainId, options) {
            return MemberServiceApiFp(configuration)
                .searchMemberAttempts(queueId, memberId, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List of Member
         * @param {string} queueId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchMemberInQueue(queueId, page, size, domainId, options) {
            return MemberServiceApiFp(configuration)
                .searchMemberInQueue(queueId, page, size, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Search of Member
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [id]
         * @param {string} [queueId]
         * @param {number} [bucketId]
         * @param {string} [destination]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchMembers(page, size, id, queueId, bucketId, destination, domainId, options) {
            return MemberServiceApiFp(configuration)
                .searchMembers(page, size, id, queueId, bucketId, destination, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary UpdateMember
         * @param {string} queueId
         * @param {string} id
         * @param {EngineUpdateMemberRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateMember(queueId, id, body, options) {
            return MemberServiceApiFp(configuration)
                .updateMember(queueId, id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * MemberServiceApi - object-oriented interface
 * @export
 * @class MemberServiceApi
 * @extends {BaseAPI}
 */
class MemberServiceApi extends BaseAPI {
    /**
     *
     * @param {string} attemptId
     * @param {EngineAttemptCallbackRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MemberServiceApi
     */
    attemptCallback(attemptId, body, options) {
        return MemberServiceApiFp(this.configuration)
            .attemptCallback(attemptId, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} queueId
     * @param {string} memberId
     * @param {string} attemptId
     * @param {EngineAttemptResultRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MemberServiceApi
     */
    attemptResult(queueId, memberId, attemptId, body, options) {
        return MemberServiceApiFp(this.configuration)
            .attemptResult(queueId, memberId, attemptId, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {string} attemptId
     * @param {EngineAttemptRenewalResultRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MemberServiceApi
     */
    attemptsRenewalResult(attemptId, body, options) {
        return MemberServiceApiFp(this.configuration)
            .attemptsRenewalResult(attemptId, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Offline queue
     * @param {string} queueId
     * @param {string} memberId
     * @param {EngineCreateAttemptRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MemberServiceApi
     */
    createAttempt(queueId, memberId, body, options) {
        return MemberServiceApiFp(this.configuration)
            .createAttempt(queueId, memberId, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Create Member
     * @param {string} queueId
     * @param {EngineCreateMemberRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MemberServiceApi
     */
    createMember(queueId, body, options) {
        return MemberServiceApiFp(this.configuration)
            .createMember(queueId, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Create Member
     * @param {string} queueId
     * @param {EngineCreateMemberBulkRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MemberServiceApi
     */
    createMemberBulk(queueId, body, options) {
        return MemberServiceApiFp(this.configuration)
            .createMemberBulk(queueId, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary DeleteMember
     * @param {string} queueId
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MemberServiceApi
     */
    deleteMember(queueId, id, domainId, options) {
        return MemberServiceApiFp(this.configuration)
            .deleteMember(queueId, id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary DeleteMembers
     * @param {string} queueId
     * @param {EngineDeleteMembersRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MemberServiceApi
     */
    deleteMembers(queueId, body, options) {
        return MemberServiceApiFp(this.configuration)
            .deleteMembers(queueId, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Patch Member
     * @param {string} queueId
     * @param {string} id
     * @param {EnginePatchMemberRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MemberServiceApi
     */
    patchMember(queueId, id, body, options) {
        return MemberServiceApiFp(this.configuration)
            .patchMember(queueId, id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary ReadQueueRouting
     * @param {string} queueId
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MemberServiceApi
     */
    readMember(queueId, id, domainId, options) {
        return MemberServiceApiFp(this.configuration)
            .readMember(queueId, id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary ResetMembers
     * @param {string} queueId
     * @param {EngineResetMembersRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MemberServiceApi
     */
    resetMembers(queueId, body, options) {
        return MemberServiceApiFp(this.configuration)
            .resetMembers(queueId, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary SearchAttempts
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [joinedAtFrom]
     * @param {string} [joinedAtTo]
     * @param {Array<string>} [id]
     * @param {Array<string>} [queueId]
     * @param {Array<string>} [bucketId]
     * @param {Array<string>} [memberId]
     * @param {Array<string>} [agentId]
     * @param {string} [result]
     * @param {Array<string>} [fields]
     * @param {string} [sort]
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MemberServiceApi
     */
    searchAttempts(page, size, joinedAtFrom, joinedAtTo, id, queueId, bucketId, memberId, agentId, result, fields, sort, domainId, options) {
        return MemberServiceApiFp(this.configuration)
            .searchAttempts(page, size, joinedAtFrom, joinedAtTo, id, queueId, bucketId, memberId, agentId, result, fields, sort, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary SearchMemberAttempts
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [joinedAtFrom]
     * @param {string} [joinedAtTo]
     * @param {Array<string>} [id]
     * @param {Array<string>} [queueId]
     * @param {Array<string>} [bucketId]
     * @param {Array<string>} [memberId]
     * @param {Array<string>} [agentId]
     * @param {string} [result]
     * @param {Array<string>} [fields]
     * @param {string} [sort]
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MemberServiceApi
     */
    searchAttemptsHistory(page, size, joinedAtFrom, joinedAtTo, id, queueId, bucketId, memberId, agentId, result, fields, sort, domainId, options) {
        return MemberServiceApiFp(this.configuration)
            .searchAttemptsHistory(page, size, joinedAtFrom, joinedAtTo, id, queueId, bucketId, memberId, agentId, result, fields, sort, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary SearchMemberAttempts
     * @param {string} queueId
     * @param {string} memberId
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MemberServiceApi
     */
    searchMemberAttempts(queueId, memberId, domainId, options) {
        return MemberServiceApiFp(this.configuration)
            .searchMemberAttempts(queueId, memberId, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List of Member
     * @param {string} queueId
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MemberServiceApi
     */
    searchMemberInQueue(queueId, page, size, domainId, options) {
        return MemberServiceApiFp(this.configuration)
            .searchMemberInQueue(queueId, page, size, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Search of Member
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [id]
     * @param {string} [queueId]
     * @param {number} [bucketId]
     * @param {string} [destination]
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MemberServiceApi
     */
    searchMembers(page, size, id, queueId, bucketId, destination, domainId, options) {
        return MemberServiceApiFp(this.configuration)
            .searchMembers(page, size, id, queueId, bucketId, destination, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary UpdateMember
     * @param {string} queueId
     * @param {string} id
     * @param {EngineUpdateMemberRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof MemberServiceApi
     */
    updateMember(queueId, id, body, options) {
        return MemberServiceApiFp(this.configuration)
            .updateMember(queueId, id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * OutboundResourceGroupServiceApi - axios parameter creator
 * @export
 */
const OutboundResourceGroupServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @summary Create OutboundResourceGroup
         * @param {EngineCreateOutboundResourceGroupRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createOutboundResourceGroup: async (body, options = {}) => {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createOutboundResourceGroup.');
            }
            const localVarPath = `/call_center/resource_group`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Create OutboundResourceGroup
         * @param {string} groupId
         * @param {EngineCreateOutboundResourceInGroupRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createOutboundResourceInGroup: async (groupId, body, options = {}) => {
            // verify required parameter 'groupId' is not null or undefined
            if (groupId === null || groupId === undefined) {
                throw new RequiredError('groupId', 'Required parameter groupId was null or undefined when calling createOutboundResourceInGroup.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createOutboundResourceInGroup.');
            }
            const localVarPath = `/call_center/resource_group/{group_id}/resource`.replace(`{${'group_id'}}`, encodeURIComponent(String(groupId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Remove OutboundResourceGroup
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteOutboundResourceGroup: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteOutboundResourceGroup.');
            }
            const localVarPath = `/call_center/resource_group/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Remove OutboundResourceInGroup
         * @param {string} groupId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteOutboundResourceInGroup: async (groupId, id, domainId, options = {}) => {
            // verify required parameter 'groupId' is not null or undefined
            if (groupId === null || groupId === undefined) {
                throw new RequiredError('groupId', 'Required parameter groupId was null or undefined when calling deleteOutboundResourceInGroup.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteOutboundResourceInGroup.');
            }
            const localVarPath = `/call_center/resource_group/{group_id}/resource/{id}`
                .replace(`{${'group_id'}}`, encodeURIComponent(String(groupId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary OutboundResourceGroup item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readOutboundResourceGroup: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readOutboundResourceGroup.');
            }
            const localVarPath = `/call_center/resource_group/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary OutboundResourceInGroup item
         * @param {string} groupId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readOutboundResourceInGroup: async (groupId, id, domainId, options = {}) => {
            // verify required parameter 'groupId' is not null or undefined
            if (groupId === null || groupId === undefined) {
                throw new RequiredError('groupId', 'Required parameter groupId was null or undefined when calling readOutboundResourceInGroup.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readOutboundResourceInGroup.');
            }
            const localVarPath = `/call_center/resource_group/{group_id}/resource/{id}`
                .replace(`{${'group_id'}}`, encodeURIComponent(String(groupId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List of OutboundResourceGroup
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchOutboundResourceGroup: async (page, size, q, sort, fields, id, options = {}) => {
            const localVarPath = `/call_center/resource_group`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List of OutboundResourceInGroup
         * @param {string} groupId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchOutboundResourceInGroup: async (groupId, page, size, q, sort, fields, id, options = {}) => {
            // verify required parameter 'groupId' is not null or undefined
            if (groupId === null || groupId === undefined) {
                throw new RequiredError('groupId', 'Required parameter groupId was null or undefined when calling searchOutboundResourceInGroup.');
            }
            const localVarPath = `/call_center/resource_group/{group_id}/resource`.replace(`{${'group_id'}}`, encodeURIComponent(String(groupId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update OutboundResourceGroup
         * @param {string} id
         * @param {EngineUpdateOutboundResourceGroupRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateOutboundResourceGroup: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateOutboundResourceGroup.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateOutboundResourceGroup.');
            }
            const localVarPath = `/call_center/resource_group/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update OutboundResourceInGroup
         * @param {string} groupId
         * @param {string} id
         * @param {EngineUpdateOutboundResourceInGroupRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateOutboundResourceInGroup: async (groupId, id, body, options = {}) => {
            // verify required parameter 'groupId' is not null or undefined
            if (groupId === null || groupId === undefined) {
                throw new RequiredError('groupId', 'Required parameter groupId was null or undefined when calling updateOutboundResourceInGroup.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateOutboundResourceInGroup.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateOutboundResourceInGroup.');
            }
            const localVarPath = `/call_center/resource_group/{group_id}/resource/{id}`
                .replace(`{${'group_id'}}`, encodeURIComponent(String(groupId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * OutboundResourceGroupServiceApi - functional programming interface
 * @export
 */
const OutboundResourceGroupServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @summary Create OutboundResourceGroup
         * @param {EngineCreateOutboundResourceGroupRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createOutboundResourceGroup(body, options) {
            const localVarAxiosArgs = await OutboundResourceGroupServiceApiAxiosParamCreator(configuration).createOutboundResourceGroup(body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Create OutboundResourceGroup
         * @param {string} groupId
         * @param {EngineCreateOutboundResourceInGroupRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createOutboundResourceInGroup(groupId, body, options) {
            const localVarAxiosArgs = await OutboundResourceGroupServiceApiAxiosParamCreator(configuration).createOutboundResourceInGroup(groupId, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Remove OutboundResourceGroup
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteOutboundResourceGroup(id, domainId, options) {
            const localVarAxiosArgs = await OutboundResourceGroupServiceApiAxiosParamCreator(configuration).deleteOutboundResourceGroup(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Remove OutboundResourceInGroup
         * @param {string} groupId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteOutboundResourceInGroup(groupId, id, domainId, options) {
            const localVarAxiosArgs = await OutboundResourceGroupServiceApiAxiosParamCreator(configuration).deleteOutboundResourceInGroup(groupId, id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary OutboundResourceGroup item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readOutboundResourceGroup(id, domainId, options) {
            const localVarAxiosArgs = await OutboundResourceGroupServiceApiAxiosParamCreator(configuration).readOutboundResourceGroup(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary OutboundResourceInGroup item
         * @param {string} groupId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readOutboundResourceInGroup(groupId, id, domainId, options) {
            const localVarAxiosArgs = await OutboundResourceGroupServiceApiAxiosParamCreator(configuration).readOutboundResourceInGroup(groupId, id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List of OutboundResourceGroup
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchOutboundResourceGroup(page, size, q, sort, fields, id, options) {
            const localVarAxiosArgs = await OutboundResourceGroupServiceApiAxiosParamCreator(configuration).searchOutboundResourceGroup(page, size, q, sort, fields, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List of OutboundResourceInGroup
         * @param {string} groupId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchOutboundResourceInGroup(groupId, page, size, q, sort, fields, id, options) {
            const localVarAxiosArgs = await OutboundResourceGroupServiceApiAxiosParamCreator(configuration).searchOutboundResourceInGroup(groupId, page, size, q, sort, fields, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update OutboundResourceGroup
         * @param {string} id
         * @param {EngineUpdateOutboundResourceGroupRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateOutboundResourceGroup(id, body, options) {
            const localVarAxiosArgs = await OutboundResourceGroupServiceApiAxiosParamCreator(configuration).updateOutboundResourceGroup(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update OutboundResourceInGroup
         * @param {string} groupId
         * @param {string} id
         * @param {EngineUpdateOutboundResourceInGroupRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateOutboundResourceInGroup(groupId, id, body, options) {
            const localVarAxiosArgs = await OutboundResourceGroupServiceApiAxiosParamCreator(configuration).updateOutboundResourceInGroup(groupId, id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * OutboundResourceGroupServiceApi - factory interface
 * @export
 */
const OutboundResourceGroupServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @summary Create OutboundResourceGroup
         * @param {EngineCreateOutboundResourceGroupRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createOutboundResourceGroup(body, options) {
            return OutboundResourceGroupServiceApiFp(configuration)
                .createOutboundResourceGroup(body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Create OutboundResourceGroup
         * @param {string} groupId
         * @param {EngineCreateOutboundResourceInGroupRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createOutboundResourceInGroup(groupId, body, options) {
            return OutboundResourceGroupServiceApiFp(configuration)
                .createOutboundResourceInGroup(groupId, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Remove OutboundResourceGroup
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteOutboundResourceGroup(id, domainId, options) {
            return OutboundResourceGroupServiceApiFp(configuration)
                .deleteOutboundResourceGroup(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Remove OutboundResourceInGroup
         * @param {string} groupId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteOutboundResourceInGroup(groupId, id, domainId, options) {
            return OutboundResourceGroupServiceApiFp(configuration)
                .deleteOutboundResourceInGroup(groupId, id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary OutboundResourceGroup item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readOutboundResourceGroup(id, domainId, options) {
            return OutboundResourceGroupServiceApiFp(configuration)
                .readOutboundResourceGroup(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary OutboundResourceInGroup item
         * @param {string} groupId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readOutboundResourceInGroup(groupId, id, domainId, options) {
            return OutboundResourceGroupServiceApiFp(configuration)
                .readOutboundResourceInGroup(groupId, id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List of OutboundResourceGroup
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchOutboundResourceGroup(page, size, q, sort, fields, id, options) {
            return OutboundResourceGroupServiceApiFp(configuration)
                .searchOutboundResourceGroup(page, size, q, sort, fields, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List of OutboundResourceInGroup
         * @param {string} groupId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchOutboundResourceInGroup(groupId, page, size, q, sort, fields, id, options) {
            return OutboundResourceGroupServiceApiFp(configuration)
                .searchOutboundResourceInGroup(groupId, page, size, q, sort, fields, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update OutboundResourceGroup
         * @param {string} id
         * @param {EngineUpdateOutboundResourceGroupRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateOutboundResourceGroup(id, body, options) {
            return OutboundResourceGroupServiceApiFp(configuration)
                .updateOutboundResourceGroup(id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update OutboundResourceInGroup
         * @param {string} groupId
         * @param {string} id
         * @param {EngineUpdateOutboundResourceInGroupRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateOutboundResourceInGroup(groupId, id, body, options) {
            return OutboundResourceGroupServiceApiFp(configuration)
                .updateOutboundResourceInGroup(groupId, id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * OutboundResourceGroupServiceApi - object-oriented interface
 * @export
 * @class OutboundResourceGroupServiceApi
 * @extends {BaseAPI}
 */
class OutboundResourceGroupServiceApi extends BaseAPI {
    /**
     *
     * @summary Create OutboundResourceGroup
     * @param {EngineCreateOutboundResourceGroupRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceGroupServiceApi
     */
    createOutboundResourceGroup(body, options) {
        return OutboundResourceGroupServiceApiFp(this.configuration)
            .createOutboundResourceGroup(body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Create OutboundResourceGroup
     * @param {string} groupId
     * @param {EngineCreateOutboundResourceInGroupRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceGroupServiceApi
     */
    createOutboundResourceInGroup(groupId, body, options) {
        return OutboundResourceGroupServiceApiFp(this.configuration)
            .createOutboundResourceInGroup(groupId, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Remove OutboundResourceGroup
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceGroupServiceApi
     */
    deleteOutboundResourceGroup(id, domainId, options) {
        return OutboundResourceGroupServiceApiFp(this.configuration)
            .deleteOutboundResourceGroup(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Remove OutboundResourceInGroup
     * @param {string} groupId
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceGroupServiceApi
     */
    deleteOutboundResourceInGroup(groupId, id, domainId, options) {
        return OutboundResourceGroupServiceApiFp(this.configuration)
            .deleteOutboundResourceInGroup(groupId, id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary OutboundResourceGroup item
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceGroupServiceApi
     */
    readOutboundResourceGroup(id, domainId, options) {
        return OutboundResourceGroupServiceApiFp(this.configuration)
            .readOutboundResourceGroup(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary OutboundResourceInGroup item
     * @param {string} groupId
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceGroupServiceApi
     */
    readOutboundResourceInGroup(groupId, id, domainId, options) {
        return OutboundResourceGroupServiceApiFp(this.configuration)
            .readOutboundResourceInGroup(groupId, id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List of OutboundResourceGroup
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceGroupServiceApi
     */
    searchOutboundResourceGroup(page, size, q, sort, fields, id, options) {
        return OutboundResourceGroupServiceApiFp(this.configuration)
            .searchOutboundResourceGroup(page, size, q, sort, fields, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List of OutboundResourceInGroup
     * @param {string} groupId
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceGroupServiceApi
     */
    searchOutboundResourceInGroup(groupId, page, size, q, sort, fields, id, options) {
        return OutboundResourceGroupServiceApiFp(this.configuration)
            .searchOutboundResourceInGroup(groupId, page, size, q, sort, fields, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update OutboundResourceGroup
     * @param {string} id
     * @param {EngineUpdateOutboundResourceGroupRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceGroupServiceApi
     */
    updateOutboundResourceGroup(id, body, options) {
        return OutboundResourceGroupServiceApiFp(this.configuration)
            .updateOutboundResourceGroup(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update OutboundResourceInGroup
     * @param {string} groupId
     * @param {string} id
     * @param {EngineUpdateOutboundResourceInGroupRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceGroupServiceApi
     */
    updateOutboundResourceInGroup(groupId, id, body, options) {
        return OutboundResourceGroupServiceApiFp(this.configuration)
            .updateOutboundResourceInGroup(groupId, id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * OutboundResourceServiceApi - axios parameter creator
 * @export
 */
const OutboundResourceServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @summary Create OutboundResource
         * @param {EngineCreateOutboundResourceRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createOutboundResource: async (body, options = {}) => {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createOutboundResource.');
            }
            const localVarPath = `/call_center/resources`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Create CreateOutboundResourceDisplay
         * @param {string} resourceId
         * @param {EngineCreateOutboundResourceDisplayRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createOutboundResourceDisplay: async (resourceId, body, options = {}) => {
            // verify required parameter 'resourceId' is not null or undefined
            if (resourceId === null || resourceId === undefined) {
                throw new RequiredError('resourceId', 'Required parameter resourceId was null or undefined when calling createOutboundResourceDisplay.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createOutboundResourceDisplay.');
            }
            const localVarPath = `/call_center/resources/{resource_id}/display`.replace(`{${'resource_id'}}`, encodeURIComponent(String(resourceId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Remove OutboundResource
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteOutboundResource: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteOutboundResource.');
            }
            const localVarPath = `/call_center/resources/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Remove ResourceDisplay
         * @param {string} resourceId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteOutboundResourceDisplay: async (resourceId, id, domainId, options = {}) => {
            // verify required parameter 'resourceId' is not null or undefined
            if (resourceId === null || resourceId === undefined) {
                throw new RequiredError('resourceId', 'Required parameter resourceId was null or undefined when calling deleteOutboundResourceDisplay.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteOutboundResourceDisplay.');
            }
            const localVarPath = `/call_center/resources/{resource_id}/display/{id}`
                .replace(`{${'resource_id'}}`, encodeURIComponent(String(resourceId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Patch OutboundResource
         * @param {string} id
         * @param {EnginePatchOutboundResourceRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchOutboundResource: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling patchOutboundResource.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling patchOutboundResource.');
            }
            const localVarPath = `/call_center/resources/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary OutboundResource item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readOutboundResource: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readOutboundResource.');
            }
            const localVarPath = `/call_center/resources/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary ResourceDisplay item
         * @param {string} resourceId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readOutboundResourceDisplay: async (resourceId, id, domainId, options = {}) => {
            // verify required parameter 'resourceId' is not null or undefined
            if (resourceId === null || resourceId === undefined) {
                throw new RequiredError('resourceId', 'Required parameter resourceId was null or undefined when calling readOutboundResourceDisplay.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readOutboundResourceDisplay.');
            }
            const localVarPath = `/call_center/resources/{resource_id}/display/{id}`
                .replace(`{${'resource_id'}}`, encodeURIComponent(String(resourceId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List of OutboundResource
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchOutboundResource: async (page, size, q, sort, fields, id, options = {}) => {
            const localVarPath = `/call_center/resources`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List of ResourceDisplay
         * @param {string} resourceId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchOutboundResourceDisplay: async (resourceId, page, size, q, sort, fields, id, options = {}) => {
            // verify required parameter 'resourceId' is not null or undefined
            if (resourceId === null || resourceId === undefined) {
                throw new RequiredError('resourceId', 'Required parameter resourceId was null or undefined when calling searchOutboundResourceDisplay.');
            }
            const localVarPath = `/call_center/resources/{resource_id}/display`.replace(`{${'resource_id'}}`, encodeURIComponent(String(resourceId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update OutboundResource
         * @param {string} id
         * @param {EngineUpdateOutboundResourceRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateOutboundResource: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateOutboundResource.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateOutboundResource.');
            }
            const localVarPath = `/call_center/resources/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update ResourceDisplay
         * @param {string} resourceId
         * @param {string} id
         * @param {EngineUpdateOutboundResourceDisplayRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateOutboundResourceDisplay: async (resourceId, id, body, options = {}) => {
            // verify required parameter 'resourceId' is not null or undefined
            if (resourceId === null || resourceId === undefined) {
                throw new RequiredError('resourceId', 'Required parameter resourceId was null or undefined when calling updateOutboundResourceDisplay.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateOutboundResourceDisplay.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateOutboundResourceDisplay.');
            }
            const localVarPath = `/call_center/resources/{resource_id}/display/{id}`
                .replace(`{${'resource_id'}}`, encodeURIComponent(String(resourceId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * OutboundResourceServiceApi - functional programming interface
 * @export
 */
const OutboundResourceServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @summary Create OutboundResource
         * @param {EngineCreateOutboundResourceRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createOutboundResource(body, options) {
            const localVarAxiosArgs = await OutboundResourceServiceApiAxiosParamCreator(configuration).createOutboundResource(body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Create CreateOutboundResourceDisplay
         * @param {string} resourceId
         * @param {EngineCreateOutboundResourceDisplayRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createOutboundResourceDisplay(resourceId, body, options) {
            const localVarAxiosArgs = await OutboundResourceServiceApiAxiosParamCreator(configuration).createOutboundResourceDisplay(resourceId, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Remove OutboundResource
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteOutboundResource(id, domainId, options) {
            const localVarAxiosArgs = await OutboundResourceServiceApiAxiosParamCreator(configuration).deleteOutboundResource(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Remove ResourceDisplay
         * @param {string} resourceId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteOutboundResourceDisplay(resourceId, id, domainId, options) {
            const localVarAxiosArgs = await OutboundResourceServiceApiAxiosParamCreator(configuration).deleteOutboundResourceDisplay(resourceId, id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Patch OutboundResource
         * @param {string} id
         * @param {EnginePatchOutboundResourceRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async patchOutboundResource(id, body, options) {
            const localVarAxiosArgs = await OutboundResourceServiceApiAxiosParamCreator(configuration).patchOutboundResource(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary OutboundResource item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readOutboundResource(id, domainId, options) {
            const localVarAxiosArgs = await OutboundResourceServiceApiAxiosParamCreator(configuration).readOutboundResource(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary ResourceDisplay item
         * @param {string} resourceId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readOutboundResourceDisplay(resourceId, id, domainId, options) {
            const localVarAxiosArgs = await OutboundResourceServiceApiAxiosParamCreator(configuration).readOutboundResourceDisplay(resourceId, id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List of OutboundResource
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchOutboundResource(page, size, q, sort, fields, id, options) {
            const localVarAxiosArgs = await OutboundResourceServiceApiAxiosParamCreator(configuration).searchOutboundResource(page, size, q, sort, fields, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List of ResourceDisplay
         * @param {string} resourceId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchOutboundResourceDisplay(resourceId, page, size, q, sort, fields, id, options) {
            const localVarAxiosArgs = await OutboundResourceServiceApiAxiosParamCreator(configuration).searchOutboundResourceDisplay(resourceId, page, size, q, sort, fields, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update OutboundResource
         * @param {string} id
         * @param {EngineUpdateOutboundResourceRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateOutboundResource(id, body, options) {
            const localVarAxiosArgs = await OutboundResourceServiceApiAxiosParamCreator(configuration).updateOutboundResource(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update ResourceDisplay
         * @param {string} resourceId
         * @param {string} id
         * @param {EngineUpdateOutboundResourceDisplayRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateOutboundResourceDisplay(resourceId, id, body, options) {
            const localVarAxiosArgs = await OutboundResourceServiceApiAxiosParamCreator(configuration).updateOutboundResourceDisplay(resourceId, id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * OutboundResourceServiceApi - factory interface
 * @export
 */
const OutboundResourceServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @summary Create OutboundResource
         * @param {EngineCreateOutboundResourceRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createOutboundResource(body, options) {
            return OutboundResourceServiceApiFp(configuration)
                .createOutboundResource(body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Create CreateOutboundResourceDisplay
         * @param {string} resourceId
         * @param {EngineCreateOutboundResourceDisplayRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createOutboundResourceDisplay(resourceId, body, options) {
            return OutboundResourceServiceApiFp(configuration)
                .createOutboundResourceDisplay(resourceId, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Remove OutboundResource
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteOutboundResource(id, domainId, options) {
            return OutboundResourceServiceApiFp(configuration)
                .deleteOutboundResource(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Remove ResourceDisplay
         * @param {string} resourceId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteOutboundResourceDisplay(resourceId, id, domainId, options) {
            return OutboundResourceServiceApiFp(configuration)
                .deleteOutboundResourceDisplay(resourceId, id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Patch OutboundResource
         * @param {string} id
         * @param {EnginePatchOutboundResourceRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchOutboundResource(id, body, options) {
            return OutboundResourceServiceApiFp(configuration)
                .patchOutboundResource(id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary OutboundResource item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readOutboundResource(id, domainId, options) {
            return OutboundResourceServiceApiFp(configuration)
                .readOutboundResource(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary ResourceDisplay item
         * @param {string} resourceId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readOutboundResourceDisplay(resourceId, id, domainId, options) {
            return OutboundResourceServiceApiFp(configuration)
                .readOutboundResourceDisplay(resourceId, id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List of OutboundResource
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchOutboundResource(page, size, q, sort, fields, id, options) {
            return OutboundResourceServiceApiFp(configuration)
                .searchOutboundResource(page, size, q, sort, fields, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List of ResourceDisplay
         * @param {string} resourceId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchOutboundResourceDisplay(resourceId, page, size, q, sort, fields, id, options) {
            return OutboundResourceServiceApiFp(configuration)
                .searchOutboundResourceDisplay(resourceId, page, size, q, sort, fields, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update OutboundResource
         * @param {string} id
         * @param {EngineUpdateOutboundResourceRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateOutboundResource(id, body, options) {
            return OutboundResourceServiceApiFp(configuration)
                .updateOutboundResource(id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update ResourceDisplay
         * @param {string} resourceId
         * @param {string} id
         * @param {EngineUpdateOutboundResourceDisplayRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateOutboundResourceDisplay(resourceId, id, body, options) {
            return OutboundResourceServiceApiFp(configuration)
                .updateOutboundResourceDisplay(resourceId, id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * OutboundResourceServiceApi - object-oriented interface
 * @export
 * @class OutboundResourceServiceApi
 * @extends {BaseAPI}
 */
class OutboundResourceServiceApi extends BaseAPI {
    /**
     *
     * @summary Create OutboundResource
     * @param {EngineCreateOutboundResourceRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceServiceApi
     */
    createOutboundResource(body, options) {
        return OutboundResourceServiceApiFp(this.configuration)
            .createOutboundResource(body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Create CreateOutboundResourceDisplay
     * @param {string} resourceId
     * @param {EngineCreateOutboundResourceDisplayRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceServiceApi
     */
    createOutboundResourceDisplay(resourceId, body, options) {
        return OutboundResourceServiceApiFp(this.configuration)
            .createOutboundResourceDisplay(resourceId, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Remove OutboundResource
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceServiceApi
     */
    deleteOutboundResource(id, domainId, options) {
        return OutboundResourceServiceApiFp(this.configuration)
            .deleteOutboundResource(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Remove ResourceDisplay
     * @param {string} resourceId
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceServiceApi
     */
    deleteOutboundResourceDisplay(resourceId, id, domainId, options) {
        return OutboundResourceServiceApiFp(this.configuration)
            .deleteOutboundResourceDisplay(resourceId, id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Patch OutboundResource
     * @param {string} id
     * @param {EnginePatchOutboundResourceRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceServiceApi
     */
    patchOutboundResource(id, body, options) {
        return OutboundResourceServiceApiFp(this.configuration)
            .patchOutboundResource(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary OutboundResource item
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceServiceApi
     */
    readOutboundResource(id, domainId, options) {
        return OutboundResourceServiceApiFp(this.configuration)
            .readOutboundResource(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary ResourceDisplay item
     * @param {string} resourceId
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceServiceApi
     */
    readOutboundResourceDisplay(resourceId, id, domainId, options) {
        return OutboundResourceServiceApiFp(this.configuration)
            .readOutboundResourceDisplay(resourceId, id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List of OutboundResource
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceServiceApi
     */
    searchOutboundResource(page, size, q, sort, fields, id, options) {
        return OutboundResourceServiceApiFp(this.configuration)
            .searchOutboundResource(page, size, q, sort, fields, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List of ResourceDisplay
     * @param {string} resourceId
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceServiceApi
     */
    searchOutboundResourceDisplay(resourceId, page, size, q, sort, fields, id, options) {
        return OutboundResourceServiceApiFp(this.configuration)
            .searchOutboundResourceDisplay(resourceId, page, size, q, sort, fields, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update OutboundResource
     * @param {string} id
     * @param {EngineUpdateOutboundResourceRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceServiceApi
     */
    updateOutboundResource(id, body, options) {
        return OutboundResourceServiceApiFp(this.configuration)
            .updateOutboundResource(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update ResourceDisplay
     * @param {string} resourceId
     * @param {string} id
     * @param {EngineUpdateOutboundResourceDisplayRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof OutboundResourceServiceApi
     */
    updateOutboundResourceDisplay(resourceId, id, body, options) {
        return OutboundResourceServiceApiFp(this.configuration)
            .updateOutboundResourceDisplay(resourceId, id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * QueueBucketServiceApi - axios parameter creator
 * @export
 */
const QueueBucketServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @summary Create QueueBucket
         * @param {string} queueId
         * @param {EngineCreateQueueBucketRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createQueueBucket: async (queueId, body, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling createQueueBucket.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createQueueBucket.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/buckets`.replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary DeleteQueueRouting
         * @param {string} queueId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteQueueBucket: async (queueId, id, domainId, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling deleteQueueBucket.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteQueueBucket.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/buckets/{id}`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary ReadQueueRouting
         * @param {string} queueId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readQueueBucket: async (queueId, id, domainId, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling readQueueBucket.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readQueueBucket.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/buckets/{id}`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary SearchQueueRouting
         * @param {string} queueId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchQueueBucket: async (queueId, page, size, q, sort, fields, id, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling searchQueueBucket.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/buckets`.replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary UpdateQueueBucket
         * @param {string} queueId
         * @param {string} id
         * @param {EngineUpdateQueueBucketRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateQueueBucket: async (queueId, id, body, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling updateQueueBucket.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateQueueBucket.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateQueueBucket.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/buckets/{id}`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * QueueBucketServiceApi - functional programming interface
 * @export
 */
const QueueBucketServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @summary Create QueueBucket
         * @param {string} queueId
         * @param {EngineCreateQueueBucketRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createQueueBucket(queueId, body, options) {
            const localVarAxiosArgs = await QueueBucketServiceApiAxiosParamCreator(configuration).createQueueBucket(queueId, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary DeleteQueueRouting
         * @param {string} queueId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteQueueBucket(queueId, id, domainId, options) {
            const localVarAxiosArgs = await QueueBucketServiceApiAxiosParamCreator(configuration).deleteQueueBucket(queueId, id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary ReadQueueRouting
         * @param {string} queueId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readQueueBucket(queueId, id, domainId, options) {
            const localVarAxiosArgs = await QueueBucketServiceApiAxiosParamCreator(configuration).readQueueBucket(queueId, id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary SearchQueueRouting
         * @param {string} queueId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchQueueBucket(queueId, page, size, q, sort, fields, id, options) {
            const localVarAxiosArgs = await QueueBucketServiceApiAxiosParamCreator(configuration).searchQueueBucket(queueId, page, size, q, sort, fields, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary UpdateQueueBucket
         * @param {string} queueId
         * @param {string} id
         * @param {EngineUpdateQueueBucketRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateQueueBucket(queueId, id, body, options) {
            const localVarAxiosArgs = await QueueBucketServiceApiAxiosParamCreator(configuration).updateQueueBucket(queueId, id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * QueueBucketServiceApi - factory interface
 * @export
 */
const QueueBucketServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @summary Create QueueBucket
         * @param {string} queueId
         * @param {EngineCreateQueueBucketRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createQueueBucket(queueId, body, options) {
            return QueueBucketServiceApiFp(configuration)
                .createQueueBucket(queueId, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary DeleteQueueRouting
         * @param {string} queueId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteQueueBucket(queueId, id, domainId, options) {
            return QueueBucketServiceApiFp(configuration)
                .deleteQueueBucket(queueId, id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary ReadQueueRouting
         * @param {string} queueId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readQueueBucket(queueId, id, domainId, options) {
            return QueueBucketServiceApiFp(configuration)
                .readQueueBucket(queueId, id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary SearchQueueRouting
         * @param {string} queueId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchQueueBucket(queueId, page, size, q, sort, fields, id, options) {
            return QueueBucketServiceApiFp(configuration)
                .searchQueueBucket(queueId, page, size, q, sort, fields, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary UpdateQueueBucket
         * @param {string} queueId
         * @param {string} id
         * @param {EngineUpdateQueueBucketRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateQueueBucket(queueId, id, body, options) {
            return QueueBucketServiceApiFp(configuration)
                .updateQueueBucket(queueId, id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * QueueBucketServiceApi - object-oriented interface
 * @export
 * @class QueueBucketServiceApi
 * @extends {BaseAPI}
 */
class QueueBucketServiceApi extends BaseAPI {
    /**
     *
     * @summary Create QueueBucket
     * @param {string} queueId
     * @param {EngineCreateQueueBucketRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueBucketServiceApi
     */
    createQueueBucket(queueId, body, options) {
        return QueueBucketServiceApiFp(this.configuration)
            .createQueueBucket(queueId, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary DeleteQueueRouting
     * @param {string} queueId
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueBucketServiceApi
     */
    deleteQueueBucket(queueId, id, domainId, options) {
        return QueueBucketServiceApiFp(this.configuration)
            .deleteQueueBucket(queueId, id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary ReadQueueRouting
     * @param {string} queueId
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueBucketServiceApi
     */
    readQueueBucket(queueId, id, domainId, options) {
        return QueueBucketServiceApiFp(this.configuration)
            .readQueueBucket(queueId, id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary SearchQueueRouting
     * @param {string} queueId
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueBucketServiceApi
     */
    searchQueueBucket(queueId, page, size, q, sort, fields, id, options) {
        return QueueBucketServiceApiFp(this.configuration)
            .searchQueueBucket(queueId, page, size, q, sort, fields, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary UpdateQueueBucket
     * @param {string} queueId
     * @param {string} id
     * @param {EngineUpdateQueueBucketRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueBucketServiceApi
     */
    updateQueueBucket(queueId, id, body, options) {
        return QueueBucketServiceApiFp(this.configuration)
            .updateQueueBucket(queueId, id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * QueueHookServiceApi - axios parameter creator
 * @export
 */
const QueueHookServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @param {number} queueId
         * @param {EngineCreateQueueHookRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createQueueHook: async (queueId, body, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling createQueueHook.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createQueueHook.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/hooks`.replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteQueueHook: async (queueId, id, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling deleteQueueHook.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteQueueHook.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/hooks/{id}`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {EnginePatchQueueHookRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchQueueHook: async (queueId, id, body, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling patchQueueHook.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling patchQueueHook.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling patchQueueHook.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/hooks/{id}`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readQueueHook: async (queueId, id, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling readQueueHook.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readQueueHook.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/hooks/{id}`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {Array<number>} [schemaId]
         * @param {Array<string>} [event]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchQueueHook: async (queueId, page, size, q, sort, fields, id, schemaId, event, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling searchQueueHook.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/hooks`.replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            if (schemaId) {
                localVarQueryParameter['schema_id'] = schemaId;
            }
            if (event) {
                localVarQueryParameter['event'] = event;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {EngineUpdateQueueHookRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateQueueHook: async (queueId, id, body, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling updateQueueHook.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateQueueHook.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateQueueHook.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/hooks/{id}`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * QueueHookServiceApi - functional programming interface
 * @export
 */
const QueueHookServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @param {number} queueId
         * @param {EngineCreateQueueHookRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createQueueHook(queueId, body, options) {
            const localVarAxiosArgs = await QueueHookServiceApiAxiosParamCreator(configuration).createQueueHook(queueId, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteQueueHook(queueId, id, options) {
            const localVarAxiosArgs = await QueueHookServiceApiAxiosParamCreator(configuration).deleteQueueHook(queueId, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {EnginePatchQueueHookRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async patchQueueHook(queueId, id, body, options) {
            const localVarAxiosArgs = await QueueHookServiceApiAxiosParamCreator(configuration).patchQueueHook(queueId, id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readQueueHook(queueId, id, options) {
            const localVarAxiosArgs = await QueueHookServiceApiAxiosParamCreator(configuration).readQueueHook(queueId, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {Array<number>} [schemaId]
         * @param {Array<string>} [event]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchQueueHook(queueId, page, size, q, sort, fields, id, schemaId, event, options) {
            const localVarAxiosArgs = await QueueHookServiceApiAxiosParamCreator(configuration).searchQueueHook(queueId, page, size, q, sort, fields, id, schemaId, event, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {EngineUpdateQueueHookRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateQueueHook(queueId, id, body, options) {
            const localVarAxiosArgs = await QueueHookServiceApiAxiosParamCreator(configuration).updateQueueHook(queueId, id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * QueueHookServiceApi - factory interface
 * @export
 */
const QueueHookServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @param {number} queueId
         * @param {EngineCreateQueueHookRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createQueueHook(queueId, body, options) {
            return QueueHookServiceApiFp(configuration)
                .createQueueHook(queueId, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteQueueHook(queueId, id, options) {
            return QueueHookServiceApiFp(configuration)
                .deleteQueueHook(queueId, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {EnginePatchQueueHookRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchQueueHook(queueId, id, body, options) {
            return QueueHookServiceApiFp(configuration)
                .patchQueueHook(queueId, id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readQueueHook(queueId, id, options) {
            return QueueHookServiceApiFp(configuration)
                .readQueueHook(queueId, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} queueId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {Array<number>} [schemaId]
         * @param {Array<string>} [event]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchQueueHook(queueId, page, size, q, sort, fields, id, schemaId, event, options) {
            return QueueHookServiceApiFp(configuration)
                .searchQueueHook(queueId, page, size, q, sort, fields, id, schemaId, event, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {EngineUpdateQueueHookRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateQueueHook(queueId, id, body, options) {
            return QueueHookServiceApiFp(configuration)
                .updateQueueHook(queueId, id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * QueueHookServiceApi - object-oriented interface
 * @export
 * @class QueueHookServiceApi
 * @extends {BaseAPI}
 */
class QueueHookServiceApi extends BaseAPI {
    /**
     *
     * @param {number} queueId
     * @param {EngineCreateQueueHookRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueHookServiceApi
     */
    createQueueHook(queueId, body, options) {
        return QueueHookServiceApiFp(this.configuration)
            .createQueueHook(queueId, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} queueId
     * @param {number} id
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueHookServiceApi
     */
    deleteQueueHook(queueId, id, options) {
        return QueueHookServiceApiFp(this.configuration)
            .deleteQueueHook(queueId, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} queueId
     * @param {number} id
     * @param {EnginePatchQueueHookRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueHookServiceApi
     */
    patchQueueHook(queueId, id, body, options) {
        return QueueHookServiceApiFp(this.configuration)
            .patchQueueHook(queueId, id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} queueId
     * @param {number} id
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueHookServiceApi
     */
    readQueueHook(queueId, id, options) {
        return QueueHookServiceApiFp(this.configuration)
            .readQueueHook(queueId, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} queueId
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {Array<number>} [schemaId]
     * @param {Array<string>} [event]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueHookServiceApi
     */
    searchQueueHook(queueId, page, size, q, sort, fields, id, schemaId, event, options) {
        return QueueHookServiceApiFp(this.configuration)
            .searchQueueHook(queueId, page, size, q, sort, fields, id, schemaId, event, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} queueId
     * @param {number} id
     * @param {EngineUpdateQueueHookRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueHookServiceApi
     */
    updateQueueHook(queueId, id, body, options) {
        return QueueHookServiceApiFp(this.configuration)
            .updateQueueHook(queueId, id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * QueueResourcesServiceApi - axios parameter creator
 * @export
 */
const QueueResourcesServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @summary CreateQueueResourceGroup
         * @param {string} queueId
         * @param {EngineCreateQueueResourceGroupRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createQueueResourceGroup: async (queueId, body, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling createQueueResourceGroup.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createQueueResourceGroup.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/resource_groups`.replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary DeleteQueueResourceGroup
         * @param {string} queueId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteQueueResourceGroup: async (queueId, id, domainId, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling deleteQueueResourceGroup.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteQueueResourceGroup.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/resource_groups/{id}`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary ReadQueueResourceGroup
         * @param {string} queueId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readQueueResourceGroup: async (queueId, id, domainId, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling readQueueResourceGroup.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readQueueResourceGroup.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/resource_groups/{id}`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary SearchQueueResourceGroup
         * @param {string} queueId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchQueueResourceGroup: async (queueId, page, size, q, sort, fields, id, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling searchQueueResourceGroup.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/resource_groups`.replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary UpdateQueueResourceGroup
         * @param {string} queueId
         * @param {string} id
         * @param {EngineUpdateQueueResourceGroupRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateQueueResourceGroup: async (queueId, id, body, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling updateQueueResourceGroup.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateQueueResourceGroup.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateQueueResourceGroup.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/resource_groups/{id}`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * QueueResourcesServiceApi - functional programming interface
 * @export
 */
const QueueResourcesServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @summary CreateQueueResourceGroup
         * @param {string} queueId
         * @param {EngineCreateQueueResourceGroupRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createQueueResourceGroup(queueId, body, options) {
            const localVarAxiosArgs = await QueueResourcesServiceApiAxiosParamCreator(configuration).createQueueResourceGroup(queueId, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary DeleteQueueResourceGroup
         * @param {string} queueId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteQueueResourceGroup(queueId, id, domainId, options) {
            const localVarAxiosArgs = await QueueResourcesServiceApiAxiosParamCreator(configuration).deleteQueueResourceGroup(queueId, id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary ReadQueueResourceGroup
         * @param {string} queueId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readQueueResourceGroup(queueId, id, domainId, options) {
            const localVarAxiosArgs = await QueueResourcesServiceApiAxiosParamCreator(configuration).readQueueResourceGroup(queueId, id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary SearchQueueResourceGroup
         * @param {string} queueId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchQueueResourceGroup(queueId, page, size, q, sort, fields, id, options) {
            const localVarAxiosArgs = await QueueResourcesServiceApiAxiosParamCreator(configuration).searchQueueResourceGroup(queueId, page, size, q, sort, fields, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary UpdateQueueResourceGroup
         * @param {string} queueId
         * @param {string} id
         * @param {EngineUpdateQueueResourceGroupRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateQueueResourceGroup(queueId, id, body, options) {
            const localVarAxiosArgs = await QueueResourcesServiceApiAxiosParamCreator(configuration).updateQueueResourceGroup(queueId, id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * QueueResourcesServiceApi - factory interface
 * @export
 */
const QueueResourcesServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @summary CreateQueueResourceGroup
         * @param {string} queueId
         * @param {EngineCreateQueueResourceGroupRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createQueueResourceGroup(queueId, body, options) {
            return QueueResourcesServiceApiFp(configuration)
                .createQueueResourceGroup(queueId, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary DeleteQueueResourceGroup
         * @param {string} queueId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteQueueResourceGroup(queueId, id, domainId, options) {
            return QueueResourcesServiceApiFp(configuration)
                .deleteQueueResourceGroup(queueId, id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary ReadQueueResourceGroup
         * @param {string} queueId
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readQueueResourceGroup(queueId, id, domainId, options) {
            return QueueResourcesServiceApiFp(configuration)
                .readQueueResourceGroup(queueId, id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary SearchQueueResourceGroup
         * @param {string} queueId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchQueueResourceGroup(queueId, page, size, q, sort, fields, id, options) {
            return QueueResourcesServiceApiFp(configuration)
                .searchQueueResourceGroup(queueId, page, size, q, sort, fields, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary UpdateQueueResourceGroup
         * @param {string} queueId
         * @param {string} id
         * @param {EngineUpdateQueueResourceGroupRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateQueueResourceGroup(queueId, id, body, options) {
            return QueueResourcesServiceApiFp(configuration)
                .updateQueueResourceGroup(queueId, id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * QueueResourcesServiceApi - object-oriented interface
 * @export
 * @class QueueResourcesServiceApi
 * @extends {BaseAPI}
 */
class QueueResourcesServiceApi extends BaseAPI {
    /**
     *
     * @summary CreateQueueResourceGroup
     * @param {string} queueId
     * @param {EngineCreateQueueResourceGroupRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueResourcesServiceApi
     */
    createQueueResourceGroup(queueId, body, options) {
        return QueueResourcesServiceApiFp(this.configuration)
            .createQueueResourceGroup(queueId, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary DeleteQueueResourceGroup
     * @param {string} queueId
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueResourcesServiceApi
     */
    deleteQueueResourceGroup(queueId, id, domainId, options) {
        return QueueResourcesServiceApiFp(this.configuration)
            .deleteQueueResourceGroup(queueId, id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary ReadQueueResourceGroup
     * @param {string} queueId
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueResourcesServiceApi
     */
    readQueueResourceGroup(queueId, id, domainId, options) {
        return QueueResourcesServiceApiFp(this.configuration)
            .readQueueResourceGroup(queueId, id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary SearchQueueResourceGroup
     * @param {string} queueId
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueResourcesServiceApi
     */
    searchQueueResourceGroup(queueId, page, size, q, sort, fields, id, options) {
        return QueueResourcesServiceApiFp(this.configuration)
            .searchQueueResourceGroup(queueId, page, size, q, sort, fields, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary UpdateQueueResourceGroup
     * @param {string} queueId
     * @param {string} id
     * @param {EngineUpdateQueueResourceGroupRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueResourcesServiceApi
     */
    updateQueueResourceGroup(queueId, id, body, options) {
        return QueueResourcesServiceApiFp(this.configuration)
            .updateQueueResourceGroup(queueId, id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * QueueServiceApi - axios parameter creator
 * @export
 */
const QueueServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @summary Create Queue
         * @param {EngineCreateQueueRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createQueue: async (body, options = {}) => {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createQueue.');
            }
            const localVarPath = `/call_center/queues`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Remove Queue
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteQueue: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteQueue.');
            }
            const localVarPath = `/call_center/queues/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Patch Queue
         * @param {string} id
         * @param {EnginePatchQueueRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchQueue: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling patchQueue.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling patchQueue.');
            }
            const localVarPath = `/call_center/queues/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Queue item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readQueue: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readQueue.');
            }
            const localVarPath = `/call_center/queues/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List of Queue
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<string>} [id]
         * @param {Array<number>} [type]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchQueue: async (page, size, q, sort, fields, id, type, options = {}) => {
            const localVarPath = `/call_center/queues`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            if (type) {
                localVarQueryParameter['type'] = type;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [joinedAtFrom]
         * @param {string} [joinedAtTo]
         * @param {string} [domainId]
         * @param {Array<string>} [fields]
         * @param {string} [sort]
         * @param {string} [q]
         * @param {Array<number>} [queueId]
         * @param {Array<number>} [teamId]
         * @param {Array<number>} [type]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchQueueReportGeneral: async (page, size, joinedAtFrom, joinedAtTo, domainId, fields, sort, q, queueId, teamId, type, options = {}) => {
            const localVarPath = `/call_center/queues/reports/general`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (joinedAtFrom !== undefined) {
                localVarQueryParameter['joined_at.from'] = joinedAtFrom;
            }
            if (joinedAtTo !== undefined) {
                localVarQueryParameter['joined_at.to'] = joinedAtTo;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (queueId) {
                localVarQueryParameter['queue_id'] = queueId;
            }
            if (teamId) {
                localVarQueryParameter['team_id'] = teamId;
            }
            if (type) {
                localVarQueryParameter['type'] = type;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update Queue
         * @param {string} id
         * @param {EngineUpdateQueueRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateQueue: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateQueue.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateQueue.');
            }
            const localVarPath = `/call_center/queues/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * QueueServiceApi - functional programming interface
 * @export
 */
const QueueServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @summary Create Queue
         * @param {EngineCreateQueueRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createQueue(body, options) {
            const localVarAxiosArgs = await QueueServiceApiAxiosParamCreator(configuration).createQueue(body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Remove Queue
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteQueue(id, domainId, options) {
            const localVarAxiosArgs = await QueueServiceApiAxiosParamCreator(configuration).deleteQueue(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Patch Queue
         * @param {string} id
         * @param {EnginePatchQueueRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async patchQueue(id, body, options) {
            const localVarAxiosArgs = await QueueServiceApiAxiosParamCreator(configuration).patchQueue(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Queue item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readQueue(id, domainId, options) {
            const localVarAxiosArgs = await QueueServiceApiAxiosParamCreator(configuration).readQueue(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List of Queue
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<string>} [id]
         * @param {Array<number>} [type]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchQueue(page, size, q, sort, fields, id, type, options) {
            const localVarAxiosArgs = await QueueServiceApiAxiosParamCreator(configuration).searchQueue(page, size, q, sort, fields, id, type, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [joinedAtFrom]
         * @param {string} [joinedAtTo]
         * @param {string} [domainId]
         * @param {Array<string>} [fields]
         * @param {string} [sort]
         * @param {string} [q]
         * @param {Array<number>} [queueId]
         * @param {Array<number>} [teamId]
         * @param {Array<number>} [type]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchQueueReportGeneral(page, size, joinedAtFrom, joinedAtTo, domainId, fields, sort, q, queueId, teamId, type, options) {
            const localVarAxiosArgs = await QueueServiceApiAxiosParamCreator(configuration).searchQueueReportGeneral(page, size, joinedAtFrom, joinedAtTo, domainId, fields, sort, q, queueId, teamId, type, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update Queue
         * @param {string} id
         * @param {EngineUpdateQueueRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateQueue(id, body, options) {
            const localVarAxiosArgs = await QueueServiceApiAxiosParamCreator(configuration).updateQueue(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * QueueServiceApi - factory interface
 * @export
 */
const QueueServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @summary Create Queue
         * @param {EngineCreateQueueRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createQueue(body, options) {
            return QueueServiceApiFp(configuration)
                .createQueue(body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Remove Queue
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteQueue(id, domainId, options) {
            return QueueServiceApiFp(configuration)
                .deleteQueue(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Patch Queue
         * @param {string} id
         * @param {EnginePatchQueueRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchQueue(id, body, options) {
            return QueueServiceApiFp(configuration)
                .patchQueue(id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Queue item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readQueue(id, domainId, options) {
            return QueueServiceApiFp(configuration)
                .readQueue(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List of Queue
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<string>} [id]
         * @param {Array<number>} [type]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchQueue(page, size, q, sort, fields, id, type, options) {
            return QueueServiceApiFp(configuration)
                .searchQueue(page, size, q, sort, fields, id, type, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [joinedAtFrom]
         * @param {string} [joinedAtTo]
         * @param {string} [domainId]
         * @param {Array<string>} [fields]
         * @param {string} [sort]
         * @param {string} [q]
         * @param {Array<number>} [queueId]
         * @param {Array<number>} [teamId]
         * @param {Array<number>} [type]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchQueueReportGeneral(page, size, joinedAtFrom, joinedAtTo, domainId, fields, sort, q, queueId, teamId, type, options) {
            return QueueServiceApiFp(configuration)
                .searchQueueReportGeneral(page, size, joinedAtFrom, joinedAtTo, domainId, fields, sort, q, queueId, teamId, type, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update Queue
         * @param {string} id
         * @param {EngineUpdateQueueRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateQueue(id, body, options) {
            return QueueServiceApiFp(configuration)
                .updateQueue(id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * QueueServiceApi - object-oriented interface
 * @export
 * @class QueueServiceApi
 * @extends {BaseAPI}
 */
class QueueServiceApi extends BaseAPI {
    /**
     *
     * @summary Create Queue
     * @param {EngineCreateQueueRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueServiceApi
     */
    createQueue(body, options) {
        return QueueServiceApiFp(this.configuration)
            .createQueue(body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Remove Queue
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueServiceApi
     */
    deleteQueue(id, domainId, options) {
        return QueueServiceApiFp(this.configuration)
            .deleteQueue(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Patch Queue
     * @param {string} id
     * @param {EnginePatchQueueRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueServiceApi
     */
    patchQueue(id, body, options) {
        return QueueServiceApiFp(this.configuration)
            .patchQueue(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Queue item
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueServiceApi
     */
    readQueue(id, domainId, options) {
        return QueueServiceApiFp(this.configuration)
            .readQueue(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List of Queue
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<string>} [id]
     * @param {Array<number>} [type]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueServiceApi
     */
    searchQueue(page, size, q, sort, fields, id, type, options) {
        return QueueServiceApiFp(this.configuration)
            .searchQueue(page, size, q, sort, fields, id, type, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [joinedAtFrom]
     * @param {string} [joinedAtTo]
     * @param {string} [domainId]
     * @param {Array<string>} [fields]
     * @param {string} [sort]
     * @param {string} [q]
     * @param {Array<number>} [queueId]
     * @param {Array<number>} [teamId]
     * @param {Array<number>} [type]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueServiceApi
     */
    searchQueueReportGeneral(page, size, joinedAtFrom, joinedAtTo, domainId, fields, sort, q, queueId, teamId, type, options) {
        return QueueServiceApiFp(this.configuration)
            .searchQueueReportGeneral(page, size, joinedAtFrom, joinedAtTo, domainId, fields, sort, q, queueId, teamId, type, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update Queue
     * @param {string} id
     * @param {EngineUpdateQueueRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueServiceApi
     */
    updateQueue(id, body, options) {
        return QueueServiceApiFp(this.configuration)
            .updateQueue(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * QueueSkillServiceApi - axios parameter creator
 * @export
 */
const QueueSkillServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @param {number} queueId
         * @param {EngineCreateQueueSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createQueueSkill: async (queueId, body, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling createQueueSkill.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createQueueSkill.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/skills`.replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteQueueSkill: async (queueId, id, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling deleteQueueSkill.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteQueueSkill.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/skills/{id}`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {EnginePatchQueueSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchQueueSkill: async (queueId, id, body, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling patchQueueSkill.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling patchQueueSkill.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling patchQueueSkill.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/skills/{id}`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readQueueSkill: async (queueId, id, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling readQueueSkill.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readQueueSkill.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/skills/{id}`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {Array<number>} [skillId]
         * @param {Array<number>} [bucketId]
         * @param {Array<number>} [lvl]
         * @param {Array<number>} [minCapacity]
         * @param {Array<number>} [maxCapacity]
         * @param {boolean} [enabled]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchQueueSkill: async (queueId, page, size, q, sort, fields, id, skillId, bucketId, lvl, minCapacity, maxCapacity, enabled, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling searchQueueSkill.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/skills`.replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            if (skillId) {
                localVarQueryParameter['skill_id'] = skillId;
            }
            if (bucketId) {
                localVarQueryParameter['bucket_id'] = bucketId;
            }
            if (lvl) {
                localVarQueryParameter['lvl'] = lvl;
            }
            if (minCapacity) {
                localVarQueryParameter['min_capacity'] = minCapacity;
            }
            if (maxCapacity) {
                localVarQueryParameter['max_capacity'] = maxCapacity;
            }
            if (enabled !== undefined) {
                localVarQueryParameter['enabled'] = enabled;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {EngineUpdateQueueSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateQueueSkill: async (queueId, id, body, options = {}) => {
            // verify required parameter 'queueId' is not null or undefined
            if (queueId === null || queueId === undefined) {
                throw new RequiredError('queueId', 'Required parameter queueId was null or undefined when calling updateQueueSkill.');
            }
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateQueueSkill.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateQueueSkill.');
            }
            const localVarPath = `/call_center/queues/{queue_id}/skills/{id}`
                .replace(`{${'queue_id'}}`, encodeURIComponent(String(queueId)))
                .replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * QueueSkillServiceApi - functional programming interface
 * @export
 */
const QueueSkillServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @param {number} queueId
         * @param {EngineCreateQueueSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createQueueSkill(queueId, body, options) {
            const localVarAxiosArgs = await QueueSkillServiceApiAxiosParamCreator(configuration).createQueueSkill(queueId, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteQueueSkill(queueId, id, options) {
            const localVarAxiosArgs = await QueueSkillServiceApiAxiosParamCreator(configuration).deleteQueueSkill(queueId, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {EnginePatchQueueSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async patchQueueSkill(queueId, id, body, options) {
            const localVarAxiosArgs = await QueueSkillServiceApiAxiosParamCreator(configuration).patchQueueSkill(queueId, id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readQueueSkill(queueId, id, options) {
            const localVarAxiosArgs = await QueueSkillServiceApiAxiosParamCreator(configuration).readQueueSkill(queueId, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {Array<number>} [skillId]
         * @param {Array<number>} [bucketId]
         * @param {Array<number>} [lvl]
         * @param {Array<number>} [minCapacity]
         * @param {Array<number>} [maxCapacity]
         * @param {boolean} [enabled]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchQueueSkill(queueId, page, size, q, sort, fields, id, skillId, bucketId, lvl, minCapacity, maxCapacity, enabled, options) {
            const localVarAxiosArgs = await QueueSkillServiceApiAxiosParamCreator(configuration).searchQueueSkill(queueId, page, size, q, sort, fields, id, skillId, bucketId, lvl, minCapacity, maxCapacity, enabled, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {EngineUpdateQueueSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateQueueSkill(queueId, id, body, options) {
            const localVarAxiosArgs = await QueueSkillServiceApiAxiosParamCreator(configuration).updateQueueSkill(queueId, id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * QueueSkillServiceApi - factory interface
 * @export
 */
const QueueSkillServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @param {number} queueId
         * @param {EngineCreateQueueSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createQueueSkill(queueId, body, options) {
            return QueueSkillServiceApiFp(configuration)
                .createQueueSkill(queueId, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteQueueSkill(queueId, id, options) {
            return QueueSkillServiceApiFp(configuration)
                .deleteQueueSkill(queueId, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {EnginePatchQueueSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchQueueSkill(queueId, id, body, options) {
            return QueueSkillServiceApiFp(configuration)
                .patchQueueSkill(queueId, id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readQueueSkill(queueId, id, options) {
            return QueueSkillServiceApiFp(configuration)
                .readQueueSkill(queueId, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} queueId
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {Array<number>} [skillId]
         * @param {Array<number>} [bucketId]
         * @param {Array<number>} [lvl]
         * @param {Array<number>} [minCapacity]
         * @param {Array<number>} [maxCapacity]
         * @param {boolean} [enabled]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchQueueSkill(queueId, page, size, q, sort, fields, id, skillId, bucketId, lvl, minCapacity, maxCapacity, enabled, options) {
            return QueueSkillServiceApiFp(configuration)
                .searchQueueSkill(queueId, page, size, q, sort, fields, id, skillId, bucketId, lvl, minCapacity, maxCapacity, enabled, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} queueId
         * @param {number} id
         * @param {EngineUpdateQueueSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateQueueSkill(queueId, id, body, options) {
            return QueueSkillServiceApiFp(configuration)
                .updateQueueSkill(queueId, id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * QueueSkillServiceApi - object-oriented interface
 * @export
 * @class QueueSkillServiceApi
 * @extends {BaseAPI}
 */
class QueueSkillServiceApi extends BaseAPI {
    /**
     *
     * @param {number} queueId
     * @param {EngineCreateQueueSkillRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueSkillServiceApi
     */
    createQueueSkill(queueId, body, options) {
        return QueueSkillServiceApiFp(this.configuration)
            .createQueueSkill(queueId, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} queueId
     * @param {number} id
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueSkillServiceApi
     */
    deleteQueueSkill(queueId, id, options) {
        return QueueSkillServiceApiFp(this.configuration)
            .deleteQueueSkill(queueId, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} queueId
     * @param {number} id
     * @param {EnginePatchQueueSkillRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueSkillServiceApi
     */
    patchQueueSkill(queueId, id, body, options) {
        return QueueSkillServiceApiFp(this.configuration)
            .patchQueueSkill(queueId, id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} queueId
     * @param {number} id
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueSkillServiceApi
     */
    readQueueSkill(queueId, id, options) {
        return QueueSkillServiceApiFp(this.configuration)
            .readQueueSkill(queueId, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} queueId
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {Array<number>} [skillId]
     * @param {Array<number>} [bucketId]
     * @param {Array<number>} [lvl]
     * @param {Array<number>} [minCapacity]
     * @param {Array<number>} [maxCapacity]
     * @param {boolean} [enabled]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueSkillServiceApi
     */
    searchQueueSkill(queueId, page, size, q, sort, fields, id, skillId, bucketId, lvl, minCapacity, maxCapacity, enabled, options) {
        return QueueSkillServiceApiFp(this.configuration)
            .searchQueueSkill(queueId, page, size, q, sort, fields, id, skillId, bucketId, lvl, minCapacity, maxCapacity, enabled, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} queueId
     * @param {number} id
     * @param {EngineUpdateQueueSkillRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof QueueSkillServiceApi
     */
    updateQueueSkill(queueId, id, body, options) {
        return QueueSkillServiceApiFp(this.configuration)
            .updateQueueSkill(queueId, id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * RegionServiceApi - axios parameter creator
 * @export
 */
const RegionServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @param {EngineCreateRegionRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createRegion: async (body, options = {}) => {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createRegion.');
            }
            const localVarPath = `/regions`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteRegion: async (id, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteRegion.');
            }
            const localVarPath = `/regions/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} id
         * @param {EnginePatchRegionRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchRegion: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling patchRegion.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling patchRegion.');
            }
            const localVarPath = `/regions/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readRegion: async (id, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readRegion.');
            }
            const localVarPath = `/regions/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {string} [name]
         * @param {string} [description]
         * @param {Array<number>} [timezoneId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchRegion: async (page, size, q, sort, fields, id, name, description, timezoneId, options = {}) => {
            const localVarPath = `/regions`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            if (name !== undefined) {
                localVarQueryParameter['name'] = name;
            }
            if (description !== undefined) {
                localVarQueryParameter['description'] = description;
            }
            if (timezoneId) {
                localVarQueryParameter['timezone_id'] = timezoneId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @param {number} id
         * @param {EngineUpdateRegionRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateRegion: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateRegion.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateRegion.');
            }
            const localVarPath = `/regions/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * RegionServiceApi - functional programming interface
 * @export
 */
const RegionServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @param {EngineCreateRegionRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createRegion(body, options) {
            const localVarAxiosArgs = await RegionServiceApiAxiosParamCreator(configuration).createRegion(body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteRegion(id, options) {
            const localVarAxiosArgs = await RegionServiceApiAxiosParamCreator(configuration).deleteRegion(id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} id
         * @param {EnginePatchRegionRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async patchRegion(id, body, options) {
            const localVarAxiosArgs = await RegionServiceApiAxiosParamCreator(configuration).patchRegion(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readRegion(id, options) {
            const localVarAxiosArgs = await RegionServiceApiAxiosParamCreator(configuration).readRegion(id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {string} [name]
         * @param {string} [description]
         * @param {Array<number>} [timezoneId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchRegion(page, size, q, sort, fields, id, name, description, timezoneId, options) {
            const localVarAxiosArgs = await RegionServiceApiAxiosParamCreator(configuration).searchRegion(page, size, q, sort, fields, id, name, description, timezoneId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @param {number} id
         * @param {EngineUpdateRegionRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateRegion(id, body, options) {
            const localVarAxiosArgs = await RegionServiceApiAxiosParamCreator(configuration).updateRegion(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * RegionServiceApi - factory interface
 * @export
 */
const RegionServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @param {EngineCreateRegionRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createRegion(body, options) {
            return RegionServiceApiFp(configuration)
                .createRegion(body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteRegion(id, options) {
            return RegionServiceApiFp(configuration)
                .deleteRegion(id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} id
         * @param {EnginePatchRegionRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchRegion(id, body, options) {
            return RegionServiceApiFp(configuration)
                .patchRegion(id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} id
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readRegion(id, options) {
            return RegionServiceApiFp(configuration)
                .readRegion(id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {string} [name]
         * @param {string} [description]
         * @param {Array<number>} [timezoneId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchRegion(page, size, q, sort, fields, id, name, description, timezoneId, options) {
            return RegionServiceApiFp(configuration)
                .searchRegion(page, size, q, sort, fields, id, name, description, timezoneId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @param {number} id
         * @param {EngineUpdateRegionRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateRegion(id, body, options) {
            return RegionServiceApiFp(configuration)
                .updateRegion(id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * RegionServiceApi - object-oriented interface
 * @export
 * @class RegionServiceApi
 * @extends {BaseAPI}
 */
class RegionServiceApi extends BaseAPI {
    /**
     *
     * @param {EngineCreateRegionRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RegionServiceApi
     */
    createRegion(body, options) {
        return RegionServiceApiFp(this.configuration)
            .createRegion(body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} id
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RegionServiceApi
     */
    deleteRegion(id, options) {
        return RegionServiceApiFp(this.configuration)
            .deleteRegion(id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} id
     * @param {EnginePatchRegionRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RegionServiceApi
     */
    patchRegion(id, body, options) {
        return RegionServiceApiFp(this.configuration)
            .patchRegion(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} id
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RegionServiceApi
     */
    readRegion(id, options) {
        return RegionServiceApiFp(this.configuration)
            .readRegion(id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {string} [name]
     * @param {string} [description]
     * @param {Array<number>} [timezoneId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RegionServiceApi
     */
    searchRegion(page, size, q, sort, fields, id, name, description, timezoneId, options) {
        return RegionServiceApiFp(this.configuration)
            .searchRegion(page, size, q, sort, fields, id, name, description, timezoneId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @param {number} id
     * @param {EngineUpdateRegionRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RegionServiceApi
     */
    updateRegion(id, body, options) {
        return RegionServiceApiFp(this.configuration)
            .updateRegion(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * RoutingOutboundCallServiceApi - axios parameter creator
 * @export
 */
const RoutingOutboundCallServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @summary Create RoutingOutboundCall
         * @param {EngineCreateRoutingOutboundCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createRoutingOutboundCall: async (body, options = {}) => {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createRoutingOutboundCall.');
            }
            const localVarPath = `/routing/outbound/calls`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Remove RoutingOutboundCall
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteRoutingOutboundCall: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteRoutingOutboundCall.');
            }
            const localVarPath = `/routing/outbound/calls/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Move RoutingOutboundCall
         * @param {string} fromId
         * @param {string} toId
         * @param {EngineMovePositionRoutingOutboundCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        movePositionRoutingOutboundCall: async (fromId, toId, body, options = {}) => {
            // verify required parameter 'fromId' is not null or undefined
            if (fromId === null || fromId === undefined) {
                throw new RequiredError('fromId', 'Required parameter fromId was null or undefined when calling movePositionRoutingOutboundCall.');
            }
            // verify required parameter 'toId' is not null or undefined
            if (toId === null || toId === undefined) {
                throw new RequiredError('toId', 'Required parameter toId was null or undefined when calling movePositionRoutingOutboundCall.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling movePositionRoutingOutboundCall.');
            }
            const localVarPath = `/routing/outbound/calls/{from_id}/to/{to_id}`
                .replace(`{${'from_id'}}`, encodeURIComponent(String(fromId)))
                .replace(`{${'to_id'}}`, encodeURIComponent(String(toId)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Patch RoutingOutboundCall
         * @param {string} id
         * @param {EnginePatchRoutingOutboundCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchRoutingOutboundCall: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling patchRoutingOutboundCall.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling patchRoutingOutboundCall.');
            }
            const localVarPath = `/routing/outbound/calls/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary RoutingOutboundCall item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readRoutingOutboundCall: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readRoutingOutboundCall.');
            }
            const localVarPath = `/routing/outbound/calls/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List of RoutingOutboundCall
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {string} [name]
         * @param {Array<number>} [schemaId]
         * @param {string} [pattern]
         * @param {string} [description]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchRoutingOutboundCall: async (page, size, q, sort, fields, id, name, schemaId, pattern, description, options = {}) => {
            const localVarPath = `/routing/outbound/calls`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            if (name !== undefined) {
                localVarQueryParameter['name'] = name;
            }
            if (schemaId) {
                localVarQueryParameter['schema_id'] = schemaId;
            }
            if (pattern !== undefined) {
                localVarQueryParameter['pattern'] = pattern;
            }
            if (description !== undefined) {
                localVarQueryParameter['description'] = description;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update RoutingOutboundCall
         * @param {string} id
         * @param {EngineUpdateRoutingOutboundCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateRoutingOutboundCall: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateRoutingOutboundCall.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateRoutingOutboundCall.');
            }
            const localVarPath = `/routing/outbound/calls/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * RoutingOutboundCallServiceApi - functional programming interface
 * @export
 */
const RoutingOutboundCallServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @summary Create RoutingOutboundCall
         * @param {EngineCreateRoutingOutboundCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createRoutingOutboundCall(body, options) {
            const localVarAxiosArgs = await RoutingOutboundCallServiceApiAxiosParamCreator(configuration).createRoutingOutboundCall(body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Remove RoutingOutboundCall
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteRoutingOutboundCall(id, domainId, options) {
            const localVarAxiosArgs = await RoutingOutboundCallServiceApiAxiosParamCreator(configuration).deleteRoutingOutboundCall(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Move RoutingOutboundCall
         * @param {string} fromId
         * @param {string} toId
         * @param {EngineMovePositionRoutingOutboundCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async movePositionRoutingOutboundCall(fromId, toId, body, options) {
            const localVarAxiosArgs = await RoutingOutboundCallServiceApiAxiosParamCreator(configuration).movePositionRoutingOutboundCall(fromId, toId, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Patch RoutingOutboundCall
         * @param {string} id
         * @param {EnginePatchRoutingOutboundCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async patchRoutingOutboundCall(id, body, options) {
            const localVarAxiosArgs = await RoutingOutboundCallServiceApiAxiosParamCreator(configuration).patchRoutingOutboundCall(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary RoutingOutboundCall item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readRoutingOutboundCall(id, domainId, options) {
            const localVarAxiosArgs = await RoutingOutboundCallServiceApiAxiosParamCreator(configuration).readRoutingOutboundCall(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List of RoutingOutboundCall
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {string} [name]
         * @param {Array<number>} [schemaId]
         * @param {string} [pattern]
         * @param {string} [description]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchRoutingOutboundCall(page, size, q, sort, fields, id, name, schemaId, pattern, description, options) {
            const localVarAxiosArgs = await RoutingOutboundCallServiceApiAxiosParamCreator(configuration).searchRoutingOutboundCall(page, size, q, sort, fields, id, name, schemaId, pattern, description, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update RoutingOutboundCall
         * @param {string} id
         * @param {EngineUpdateRoutingOutboundCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateRoutingOutboundCall(id, body, options) {
            const localVarAxiosArgs = await RoutingOutboundCallServiceApiAxiosParamCreator(configuration).updateRoutingOutboundCall(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * RoutingOutboundCallServiceApi - factory interface
 * @export
 */
const RoutingOutboundCallServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @summary Create RoutingOutboundCall
         * @param {EngineCreateRoutingOutboundCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createRoutingOutboundCall(body, options) {
            return RoutingOutboundCallServiceApiFp(configuration)
                .createRoutingOutboundCall(body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Remove RoutingOutboundCall
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteRoutingOutboundCall(id, domainId, options) {
            return RoutingOutboundCallServiceApiFp(configuration)
                .deleteRoutingOutboundCall(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Move RoutingOutboundCall
         * @param {string} fromId
         * @param {string} toId
         * @param {EngineMovePositionRoutingOutboundCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        movePositionRoutingOutboundCall(fromId, toId, body, options) {
            return RoutingOutboundCallServiceApiFp(configuration)
                .movePositionRoutingOutboundCall(fromId, toId, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Patch RoutingOutboundCall
         * @param {string} id
         * @param {EnginePatchRoutingOutboundCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchRoutingOutboundCall(id, body, options) {
            return RoutingOutboundCallServiceApiFp(configuration)
                .patchRoutingOutboundCall(id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary RoutingOutboundCall item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readRoutingOutboundCall(id, domainId, options) {
            return RoutingOutboundCallServiceApiFp(configuration)
                .readRoutingOutboundCall(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List of RoutingOutboundCall
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {string} [name]
         * @param {Array<number>} [schemaId]
         * @param {string} [pattern]
         * @param {string} [description]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchRoutingOutboundCall(page, size, q, sort, fields, id, name, schemaId, pattern, description, options) {
            return RoutingOutboundCallServiceApiFp(configuration)
                .searchRoutingOutboundCall(page, size, q, sort, fields, id, name, schemaId, pattern, description, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update RoutingOutboundCall
         * @param {string} id
         * @param {EngineUpdateRoutingOutboundCallRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateRoutingOutboundCall(id, body, options) {
            return RoutingOutboundCallServiceApiFp(configuration)
                .updateRoutingOutboundCall(id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * RoutingOutboundCallServiceApi - object-oriented interface
 * @export
 * @class RoutingOutboundCallServiceApi
 * @extends {BaseAPI}
 */
class RoutingOutboundCallServiceApi extends BaseAPI {
    /**
     *
     * @summary Create RoutingOutboundCall
     * @param {EngineCreateRoutingOutboundCallRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RoutingOutboundCallServiceApi
     */
    createRoutingOutboundCall(body, options) {
        return RoutingOutboundCallServiceApiFp(this.configuration)
            .createRoutingOutboundCall(body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Remove RoutingOutboundCall
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RoutingOutboundCallServiceApi
     */
    deleteRoutingOutboundCall(id, domainId, options) {
        return RoutingOutboundCallServiceApiFp(this.configuration)
            .deleteRoutingOutboundCall(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Move RoutingOutboundCall
     * @param {string} fromId
     * @param {string} toId
     * @param {EngineMovePositionRoutingOutboundCallRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RoutingOutboundCallServiceApi
     */
    movePositionRoutingOutboundCall(fromId, toId, body, options) {
        return RoutingOutboundCallServiceApiFp(this.configuration)
            .movePositionRoutingOutboundCall(fromId, toId, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Patch RoutingOutboundCall
     * @param {string} id
     * @param {EnginePatchRoutingOutboundCallRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RoutingOutboundCallServiceApi
     */
    patchRoutingOutboundCall(id, body, options) {
        return RoutingOutboundCallServiceApiFp(this.configuration)
            .patchRoutingOutboundCall(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary RoutingOutboundCall item
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RoutingOutboundCallServiceApi
     */
    readRoutingOutboundCall(id, domainId, options) {
        return RoutingOutboundCallServiceApiFp(this.configuration)
            .readRoutingOutboundCall(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List of RoutingOutboundCall
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {string} [name]
     * @param {Array<number>} [schemaId]
     * @param {string} [pattern]
     * @param {string} [description]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RoutingOutboundCallServiceApi
     */
    searchRoutingOutboundCall(page, size, q, sort, fields, id, name, schemaId, pattern, description, options) {
        return RoutingOutboundCallServiceApiFp(this.configuration)
            .searchRoutingOutboundCall(page, size, q, sort, fields, id, name, schemaId, pattern, description, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update RoutingOutboundCall
     * @param {string} id
     * @param {EngineUpdateRoutingOutboundCallRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RoutingOutboundCallServiceApi
     */
    updateRoutingOutboundCall(id, body, options) {
        return RoutingOutboundCallServiceApiFp(this.configuration)
            .updateRoutingOutboundCall(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * RoutingSchemaServiceApi - axios parameter creator
 * @export
 */
const RoutingSchemaServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @summary Create RoutingSchema
         * @param {EngineCreateRoutingSchemaRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createRoutingSchema: async (body, options = {}) => {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createRoutingSchema.');
            }
            const localVarPath = `/routing/schema`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Remove RoutingSchema
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteRoutingSchema: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteRoutingSchema.');
            }
            const localVarPath = `/routing/schema/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Patch RoutingSchema
         * @param {string} id
         * @param {EnginePatchRoutingSchemaRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchRoutingSchema: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling patchRoutingSchema.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling patchRoutingSchema.');
            }
            const localVarPath = `/routing/schema/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PATCH',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary RoutingSchema item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readRoutingSchema: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readRoutingSchema.');
            }
            const localVarPath = `/routing/schema/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List RoutingSchema
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {string} [name]
         * @param {Array<string>} [type]
         * @param {boolean} [editor]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchRoutingSchema: async (page, size, q, sort, fields, id, name, type, editor, options = {}) => {
            const localVarPath = `/routing/schema`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            if (name !== undefined) {
                localVarQueryParameter['name'] = name;
            }
            if (type) {
                localVarQueryParameter['type'] = type;
            }
            if (editor !== undefined) {
                localVarQueryParameter['editor'] = editor;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update RoutingSchema
         * @param {string} id
         * @param {EngineUpdateRoutingSchemaRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateRoutingSchema: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateRoutingSchema.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateRoutingSchema.');
            }
            const localVarPath = `/routing/schema/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * RoutingSchemaServiceApi - functional programming interface
 * @export
 */
const RoutingSchemaServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @summary Create RoutingSchema
         * @param {EngineCreateRoutingSchemaRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createRoutingSchema(body, options) {
            const localVarAxiosArgs = await RoutingSchemaServiceApiAxiosParamCreator(configuration).createRoutingSchema(body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Remove RoutingSchema
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteRoutingSchema(id, domainId, options) {
            const localVarAxiosArgs = await RoutingSchemaServiceApiAxiosParamCreator(configuration).deleteRoutingSchema(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Patch RoutingSchema
         * @param {string} id
         * @param {EnginePatchRoutingSchemaRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async patchRoutingSchema(id, body, options) {
            const localVarAxiosArgs = await RoutingSchemaServiceApiAxiosParamCreator(configuration).patchRoutingSchema(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary RoutingSchema item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readRoutingSchema(id, domainId, options) {
            const localVarAxiosArgs = await RoutingSchemaServiceApiAxiosParamCreator(configuration).readRoutingSchema(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List RoutingSchema
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {string} [name]
         * @param {Array<string>} [type]
         * @param {boolean} [editor]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchRoutingSchema(page, size, q, sort, fields, id, name, type, editor, options) {
            const localVarAxiosArgs = await RoutingSchemaServiceApiAxiosParamCreator(configuration).searchRoutingSchema(page, size, q, sort, fields, id, name, type, editor, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update RoutingSchema
         * @param {string} id
         * @param {EngineUpdateRoutingSchemaRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateRoutingSchema(id, body, options) {
            const localVarAxiosArgs = await RoutingSchemaServiceApiAxiosParamCreator(configuration).updateRoutingSchema(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * RoutingSchemaServiceApi - factory interface
 * @export
 */
const RoutingSchemaServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @summary Create RoutingSchema
         * @param {EngineCreateRoutingSchemaRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createRoutingSchema(body, options) {
            return RoutingSchemaServiceApiFp(configuration)
                .createRoutingSchema(body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Remove RoutingSchema
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteRoutingSchema(id, domainId, options) {
            return RoutingSchemaServiceApiFp(configuration)
                .deleteRoutingSchema(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Patch RoutingSchema
         * @param {string} id
         * @param {EnginePatchRoutingSchemaRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        patchRoutingSchema(id, body, options) {
            return RoutingSchemaServiceApiFp(configuration)
                .patchRoutingSchema(id, body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary RoutingSchema item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readRoutingSchema(id, domainId, options) {
            return RoutingSchemaServiceApiFp(configuration)
                .readRoutingSchema(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List RoutingSchema
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {string} [name]
         * @param {Array<string>} [type]
         * @param {boolean} [editor]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchRoutingSchema(page, size, q, sort, fields, id, name, type, editor, options) {
            return RoutingSchemaServiceApiFp(configuration)
                .searchRoutingSchema(page, size, q, sort, fields, id, name, type, editor, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update RoutingSchema
         * @param {string} id
         * @param {EngineUpdateRoutingSchemaRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateRoutingSchema(id, body, options) {
            return RoutingSchemaServiceApiFp(configuration)
                .updateRoutingSchema(id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * RoutingSchemaServiceApi - object-oriented interface
 * @export
 * @class RoutingSchemaServiceApi
 * @extends {BaseAPI}
 */
class RoutingSchemaServiceApi extends BaseAPI {
    /**
     *
     * @summary Create RoutingSchema
     * @param {EngineCreateRoutingSchemaRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RoutingSchemaServiceApi
     */
    createRoutingSchema(body, options) {
        return RoutingSchemaServiceApiFp(this.configuration)
            .createRoutingSchema(body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Remove RoutingSchema
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RoutingSchemaServiceApi
     */
    deleteRoutingSchema(id, domainId, options) {
        return RoutingSchemaServiceApiFp(this.configuration)
            .deleteRoutingSchema(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Patch RoutingSchema
     * @param {string} id
     * @param {EnginePatchRoutingSchemaRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RoutingSchemaServiceApi
     */
    patchRoutingSchema(id, body, options) {
        return RoutingSchemaServiceApiFp(this.configuration)
            .patchRoutingSchema(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary RoutingSchema item
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RoutingSchemaServiceApi
     */
    readRoutingSchema(id, domainId, options) {
        return RoutingSchemaServiceApiFp(this.configuration)
            .readRoutingSchema(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List RoutingSchema
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {string} [name]
     * @param {Array<string>} [type]
     * @param {boolean} [editor]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RoutingSchemaServiceApi
     */
    searchRoutingSchema(page, size, q, sort, fields, id, name, type, editor, options) {
        return RoutingSchemaServiceApiFp(this.configuration)
            .searchRoutingSchema(page, size, q, sort, fields, id, name, type, editor, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update RoutingSchema
     * @param {string} id
     * @param {EngineUpdateRoutingSchemaRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RoutingSchemaServiceApi
     */
    updateRoutingSchema(id, body, options) {
        return RoutingSchemaServiceApiFp(this.configuration)
            .updateRoutingSchema(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * RoutingVariableServiceApi - axios parameter creator
 * @export
 */
const RoutingVariableServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @summary Create RoutingVariable
         * @param {EngineCreateRoutingVariableRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createRoutingVariable: async (body, options = {}) => {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createRoutingVariable.');
            }
            const localVarPath = `/routing/variables`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Remove RoutingVariable
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteRoutingVariable: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteRoutingVariable.');
            }
            const localVarPath = `/routing/variables/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary RoutingVariable item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readRoutingVariable: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readRoutingVariable.');
            }
            const localVarPath = `/routing/variables/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List of RoutingVariable
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchRoutingVariable: async (page, size, domainId, options = {}) => {
            const localVarPath = `/routing/variables`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update RoutingVariable
         * @param {string} id
         * @param {EngineUpdateRoutingVariableRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateRoutingVariable: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateRoutingVariable.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateRoutingVariable.');
            }
            const localVarPath = `/routing/variables/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * RoutingVariableServiceApi - functional programming interface
 * @export
 */
const RoutingVariableServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @summary Create RoutingVariable
         * @param {EngineCreateRoutingVariableRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createRoutingVariable(body, options) {
            const localVarAxiosArgs = await RoutingVariableServiceApiAxiosParamCreator(configuration).createRoutingVariable(body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Remove RoutingVariable
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteRoutingVariable(id, domainId, options) {
            const localVarAxiosArgs = await RoutingVariableServiceApiAxiosParamCreator(configuration).deleteRoutingVariable(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary RoutingVariable item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readRoutingVariable(id, domainId, options) {
            const localVarAxiosArgs = await RoutingVariableServiceApiAxiosParamCreator(configuration).readRoutingVariable(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List of RoutingVariable
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchRoutingVariable(page, size, domainId, options) {
            const localVarAxiosArgs = await RoutingVariableServiceApiAxiosParamCreator(configuration).searchRoutingVariable(page, size, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update RoutingVariable
         * @param {string} id
         * @param {EngineUpdateRoutingVariableRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateRoutingVariable(id, body, options) {
            const localVarAxiosArgs = await RoutingVariableServiceApiAxiosParamCreator(configuration).updateRoutingVariable(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * RoutingVariableServiceApi - factory interface
 * @export
 */
const RoutingVariableServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @summary Create RoutingVariable
         * @param {EngineCreateRoutingVariableRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createRoutingVariable(body, options) {
            return RoutingVariableServiceApiFp(configuration)
                .createRoutingVariable(body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Remove RoutingVariable
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteRoutingVariable(id, domainId, options) {
            return RoutingVariableServiceApiFp(configuration)
                .deleteRoutingVariable(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary RoutingVariable item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readRoutingVariable(id, domainId, options) {
            return RoutingVariableServiceApiFp(configuration)
                .readRoutingVariable(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List of RoutingVariable
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchRoutingVariable(page, size, domainId, options) {
            return RoutingVariableServiceApiFp(configuration)
                .searchRoutingVariable(page, size, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update RoutingVariable
         * @param {string} id
         * @param {EngineUpdateRoutingVariableRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateRoutingVariable(id, body, options) {
            return RoutingVariableServiceApiFp(configuration)
                .updateRoutingVariable(id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * RoutingVariableServiceApi - object-oriented interface
 * @export
 * @class RoutingVariableServiceApi
 * @extends {BaseAPI}
 */
class RoutingVariableServiceApi extends BaseAPI {
    /**
     *
     * @summary Create RoutingVariable
     * @param {EngineCreateRoutingVariableRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RoutingVariableServiceApi
     */
    createRoutingVariable(body, options) {
        return RoutingVariableServiceApiFp(this.configuration)
            .createRoutingVariable(body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Remove RoutingVariable
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RoutingVariableServiceApi
     */
    deleteRoutingVariable(id, domainId, options) {
        return RoutingVariableServiceApiFp(this.configuration)
            .deleteRoutingVariable(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary RoutingVariable item
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RoutingVariableServiceApi
     */
    readRoutingVariable(id, domainId, options) {
        return RoutingVariableServiceApiFp(this.configuration)
            .readRoutingVariable(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List of RoutingVariable
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RoutingVariableServiceApi
     */
    searchRoutingVariable(page, size, domainId, options) {
        return RoutingVariableServiceApiFp(this.configuration)
            .searchRoutingVariable(page, size, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update RoutingVariable
     * @param {string} id
     * @param {EngineUpdateRoutingVariableRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof RoutingVariableServiceApi
     */
    updateRoutingVariable(id, body, options) {
        return RoutingVariableServiceApiFp(this.configuration)
            .updateRoutingVariable(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * SkillServiceApi - axios parameter creator
 * @export
 */
const SkillServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @summary Create Skill
         * @param {EngineCreateSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createSkill: async (body, options = {}) => {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling createSkill.');
            }
            const localVarPath = `/call_center/skills`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'POST',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Remove Skill
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteSkill: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling deleteSkill.');
            }
            const localVarPath = `/call_center/skills/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'DELETE',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Skill item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readSkill: async (id, domainId, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling readSkill.');
            }
            const localVarPath = `/call_center/skills/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (domainId !== undefined) {
                localVarQueryParameter['domain_id'] = domainId;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary List of Skill
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchSkill: async (page, size, q, sort, fields, id, options = {}) => {
            const localVarPath = `/call_center/skills`;
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            if (page !== undefined) {
                localVarQueryParameter['page'] = page;
            }
            if (size !== undefined) {
                localVarQueryParameter['size'] = size;
            }
            if (q !== undefined) {
                localVarQueryParameter['q'] = q;
            }
            if (sort !== undefined) {
                localVarQueryParameter['sort'] = sort;
            }
            if (fields) {
                localVarQueryParameter['fields'] = fields;
            }
            if (id) {
                localVarQueryParameter['id'] = id;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Update Skill
         * @param {string} id
         * @param {EngineUpdateSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateSkill: async (id, body, options = {}) => {
            // verify required parameter 'id' is not null or undefined
            if (id === null || id === undefined) {
                throw new RequiredError('id', 'Required parameter id was null or undefined when calling updateSkill.');
            }
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body', 'Required parameter body was null or undefined when calling updateSkill.');
            }
            const localVarPath = `/call_center/skills/{id}`.replace(`{${'id'}}`, encodeURIComponent(String(id)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'PUT',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarHeaderParameter['Content-Type'] = 'application/json';
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            const needsSerialization = typeof body !== 'string' ||
                localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.data = needsSerialization
                ? JSON.stringify(body !== undefined ? body : {})
                : body || '';
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * SkillServiceApi - functional programming interface
 * @export
 */
const SkillServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @summary Create Skill
         * @param {EngineCreateSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async createSkill(body, options) {
            const localVarAxiosArgs = await SkillServiceApiAxiosParamCreator(configuration).createSkill(body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Remove Skill
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async deleteSkill(id, domainId, options) {
            const localVarAxiosArgs = await SkillServiceApiAxiosParamCreator(configuration).deleteSkill(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Skill item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async readSkill(id, domainId, options) {
            const localVarAxiosArgs = await SkillServiceApiAxiosParamCreator(configuration).readSkill(id, domainId, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary List of Skill
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async searchSkill(page, size, q, sort, fields, id, options) {
            const localVarAxiosArgs = await SkillServiceApiAxiosParamCreator(configuration).searchSkill(page, size, q, sort, fields, id, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
        /**
         *
         * @summary Update Skill
         * @param {string} id
         * @param {EngineUpdateSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async updateSkill(id, body, options) {
            const localVarAxiosArgs = await SkillServiceApiAxiosParamCreator(configuration).updateSkill(id, body, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * SkillServiceApi - factory interface
 * @export
 */
const SkillServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @summary Create Skill
         * @param {EngineCreateSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        createSkill(body, options) {
            return SkillServiceApiFp(configuration)
                .createSkill(body, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Remove Skill
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        deleteSkill(id, domainId, options) {
            return SkillServiceApiFp(configuration)
                .deleteSkill(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Skill item
         * @param {string} id
         * @param {string} [domainId]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        readSkill(id, domainId, options) {
            return SkillServiceApiFp(configuration)
                .readSkill(id, domainId, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary List of Skill
         * @param {number} [page]
         * @param {number} [size]
         * @param {string} [q]
         * @param {string} [sort]
         * @param {Array<string>} [fields]
         * @param {Array<number>} [id]
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        searchSkill(page, size, q, sort, fields, id, options) {
            return SkillServiceApiFp(configuration)
                .searchSkill(page, size, q, sort, fields, id, options)
                .then((request) => request(axios, basePath));
        },
        /**
         *
         * @summary Update Skill
         * @param {string} id
         * @param {EngineUpdateSkillRequest} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        updateSkill(id, body, options) {
            return SkillServiceApiFp(configuration)
                .updateSkill(id, body, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * SkillServiceApi - object-oriented interface
 * @export
 * @class SkillServiceApi
 * @extends {BaseAPI}
 */
class SkillServiceApi extends BaseAPI {
    /**
     *
     * @summary Create Skill
     * @param {EngineCreateSkillRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof SkillServiceApi
     */
    createSkill(body, options) {
        return SkillServiceApiFp(this.configuration)
            .createSkill(body, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Remove Skill
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof SkillServiceApi
     */
    deleteSkill(id, domainId, options) {
        return SkillServiceApiFp(this.configuration)
            .deleteSkill(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Skill item
     * @param {string} id
     * @param {string} [domainId]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof SkillServiceApi
     */
    readSkill(id, domainId, options) {
        return SkillServiceApiFp(this.configuration)
            .readSkill(id, domainId, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary List of Skill
     * @param {number} [page]
     * @param {number} [size]
     * @param {string} [q]
     * @param {string} [sort]
     * @param {Array<string>} [fields]
     * @param {Array<number>} [id]
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof SkillServiceApi
     */
    searchSkill(page, size, q, sort, fields, id, options) {
        return SkillServiceApiFp(this.configuration)
            .searchSkill(page, size, q, sort, fields, id, options)
            .then((request) => request(this.axios, this.basePath));
    }
    /**
     *
     * @summary Update Skill
     * @param {string} id
     * @param {EngineUpdateSkillRequest} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof SkillServiceApi
     */
    updateSkill(id, body, options) {
        return SkillServiceApiFp(this.configuration)
            .updateSkill(id, body, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable
/**
 * UserHelperServiceApi - axios parameter creator
 * @export
 */
const UserHelperServiceApiAxiosParamCreator = function (configuration) {
    return {
        /**
         *
         * @param {string} type
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        defaultDeviceConfig: async (type, options = {}) => {
            // verify required parameter 'type' is not null or undefined
            if (type === null || type === undefined) {
                throw new RequiredError('type', 'Required parameter type was null or undefined when calling defaultDeviceConfig.');
            }
            const localVarPath = `/user/device/config/{type}`.replace(`{${'type'}}`, encodeURIComponent(String(type)));
            const localVarUrlObj = parse(localVarPath, true);
            let baseOptions;
            if (configuration) {
                baseOptions = configuration.baseOptions;
            }
            const localVarRequestOptions = {
                method: 'GET',
                ...baseOptions,
                ...options,
            };
            const localVarHeaderParameter = {};
            const localVarQueryParameter = {};
            // authentication AccessToken required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
                    ? await configuration.apiKey('X-Webitel-Access')
                    : await configuration.apiKey;
                localVarHeaderParameter['X-Webitel-Access'] = localVarApiKeyValue;
            }
            localVarUrlObj.query = {
                ...localVarUrlObj.query,
                ...localVarQueryParameter,
                ...options.query,
            };
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            let headersFromBaseOptions = baseOptions && baseOptions.headers ? baseOptions.headers : {};
            localVarRequestOptions.headers = {
                ...localVarHeaderParameter,
                ...headersFromBaseOptions,
                ...options.headers,
            };
            return {
                url: format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    };
};
/**
 * UserHelperServiceApi - functional programming interface
 * @export
 */
const UserHelperServiceApiFp = function (configuration) {
    return {
        /**
         *
         * @param {string} type
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        async defaultDeviceConfig(type, options) {
            const localVarAxiosArgs = await UserHelperServiceApiAxiosParamCreator(configuration).defaultDeviceConfig(type, options);
            return (axios = globalAxios, basePath = BASE_PATH) => {
                const axiosRequestArgs = {
                    ...localVarAxiosArgs.options,
                    url: basePath + localVarAxiosArgs.url,
                };
                return axios.request(axiosRequestArgs);
            };
        },
    };
};
/**
 * UserHelperServiceApi - factory interface
 * @export
 */
const UserHelperServiceApiFactory = function (configuration, basePath, axios) {
    return {
        /**
         *
         * @param {string} type
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        defaultDeviceConfig(type, options) {
            return UserHelperServiceApiFp(configuration)
                .defaultDeviceConfig(type, options)
                .then((request) => request(axios, basePath));
        },
    };
};
/**
 * UserHelperServiceApi - object-oriented interface
 * @export
 * @class UserHelperServiceApi
 * @extends {BaseAPI}
 */
class UserHelperServiceApi extends BaseAPI {
    /**
     *
     * @param {string} type
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof UserHelperServiceApi
     */
    defaultDeviceConfig(type, options) {
        return UserHelperServiceApiFp(this.configuration)
            .defaultDeviceConfig(type, options)
            .then((request) => request(this.axios, this.basePath));
    }
}

// tslint:disable

// tslint:disable
/**
 * Webitel engine API
 * No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)
 *
 * The version of the OpenAPI document: 20.12.0
 * Contact: support@webitel.com
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */
class Configuration {
    constructor(param = {}) {
        this.apiKey = param.apiKey;
        this.username = param.username;
        this.password = param.password;
        this.accessToken = param.accessToken;
        this.basePath = param.basePath;
        this.baseOptions = param.baseOptions;
    }
}

class Log {
    debug(msg, ...supportingDetails) {
        this.emitLogMessage('debug', msg, supportingDetails);
    }
    info(msg, ...supportingDetails) {
        this.emitLogMessage('info', msg, supportingDetails);
    }
    warn(msg, ...supportingDetails) {
        this.emitLogMessage('warn', msg, supportingDetails);
    }
    error(msg, ...supportingDetails) {
        this.emitLogMessage('error', msg, supportingDetails);
    }
    emitLogMessage(msgType, msg, supportingDetails) {
        if (supportingDetails.length > 0) {
            console[msgType](msg, ...supportingDetails);
        }
        else {
            console[msgType](msg);
        }
    }
}

class Session {
    constructor(e) {
        this.session = e.session;
        this.callId = e.request.getHeader('X-Webitel-Uuid');
        this.incoming = e.session.direction === 'incoming';
        this.instanceId = e.request.getHeader('X-Webitel-Sock-Id');
    }
    get id() {
        return this.session.id;
    }
    getLocalMedia() {
        if (this.session.connection) {
            const local = this.session.connection.getLocalStreams();
            if (local.length) {
                return local;
            }
        }
        return [];
    }
    getPeerMedia() {
        if (this.session.connection) {
            const peer = this.session.connection.getRemoteStreams();
            if (peer.length) {
                return peer;
            }
        }
        return [];
    }
    answer(s) {
        return new Promise(() => {
            this.session.answer(s);
        });
    }
}

class SipPhone extends EventEmitter {
    constructor(instanceId, debug$$1) {
        super();
        this.instanceId = instanceId;
        this.type = 'webrtc';
        this.sessionCache = new Map();
        if (debug$$1) {
            debug.enable('*');
        }
        else {
            debug.disable();
        }
        this.log = new Log();
        this.on('unregistered', () => {
            this.sessionCache.clear();
        });
    }
    async callOption(req = {}) {
        const mediaStream = await this.getUserCallStream(req);
        return {
            sessionTimersExpires: 120,
            pcConfig: req.disableStun
                ? undefined
                : {
                    iceServers: [{ urls: ['stun:stun.l.google.com:19302'] }],
                },
            mediaStream,
        };
    }
    async answer(id, req) {
        if (this.sessionCache.has(id)) {
            const session = this.sessionCache.get(id);
            const params = await this.callOption(req);
            return session.answer(params);
        }
        return false;
    }
    async call(req) {
        const params = {};
        let display = null;
        if (req.params) {
            params.audio = req.params.audio;
            params.video = req.params.video || false;
            params.screen = req.params.screen || false;
            params.disableStun = req.params.disableStun || false;
        }
        if (typeof req.destination !== 'string') {
            throw new Error('bad destination type');
        }
        const invite = await this.callOption(params);
        if (req.params && req.params.display) {
            display = {
                extraHeaders: [`X-Webitel-Display: ${req.params.display}`],
            };
        }
        await this.ua.call(req.destination, {
            ...invite,
            ...display,
        });
    }
    async register(sipConf) {
        const socket = new WebSocketInterface(sipConf.server);
        const configuration = {
            realm: sipConf.realm,
            uri: sipConf.uri,
            authorization_user: sipConf.authorization_user,
            ha1: sipConf.ha1,
            user_agent: SipPhone.userAgent,
            sockets: [socket],
            session_timers: true,
            // use_preloaded_route: true,
            display_name: sipConf.display_name,
            register_expires: sipConf.register_expires || 90,
            connection_recovery_min_interval: 5,
            connection_recovery_max_interval: 60,
        };
        const ua = (this.ua = new UA(configuration));
        // ua.on('connected', (e: object) => {})
        ua.on('newRTCSession', (e) => {
            const session = e.session;
            const id = session.id;
            const callSession = new Session(e);
            this.storeSession(id, callSession);
            session.on('connecting', () => {
                this.emit('localStreams', callSession, session.connection.getLocalStreams());
            });
            if (session.connection) {
                session.connection.addEventListener('addstream', (evt) => {
                    this.emit('peerStreams', callSession, new Array(evt.stream));
                });
            }
            session.on('peerconnection', (peer) => {
                peer.peerconnection.addEventListener('addstream', async (evt) => {
                    // set remote audio stream
                    this.emit('peerStreams', callSession, new Array(evt.stream));
                });
            });
            session.on('ended', () => {
                // this handler will be called for incoming calls too
                this.removeSession(id, session.connection);
            });
            session.on('failed', () => {
                // this handler will be called for incoming calls too
                this.removeSession(id, session.connection);
            });
            session.on('accepted', () => {
                // the call has answered
                if (!callSession.incoming) {
                    this.emit('peerStreams', callSession, session.connection.getRemoteStreams());
                }
            });
            session.on('confirmed', () => {
                // this handler will be called for incoming calls too
            });
            this.emit('newSession', callSession);
        });
        ua.on('disconnected', (e) => {
            this.emit('unregistered');
        });
        // ua.on('registered', (e: object) => {})
        ua.on('unregistered', (e) => {
            this.emit('unregistered');
        });
        ua.on('registrationFailed', (e) => {
            this.log.error('registrationFailed', e);
        });
        ua.on('registered', (e) => {
            this.emit('registered');
        });
        ua.start();
    }
    async unregister() {
        if (this.ua) {
            this.ua.removeAllListeners();
            await this.ua.stop();
            delete this.ua;
        }
    }
    sipSessionByCallId(id) {
        for (const sess of this.allSession) {
            if (sess.callId === id) {
                return sess;
            }
        }
        return null;
    }
    sipSessionBySipId(id) {
        for (const sess of this.allSession) {
            if (sess.id.startsWith(id)) {
                return sess;
            }
        }
        return null;
    }
    removeSession(id, connection) {
        if (connection) {
            const localStreams = connection.getLocalStreams();
            const remoteStreams = connection.getRemoteStreams();
            if (localStreams) {
                localStreams.forEach((stream) => {
                    stream.getTracks().forEach((track) => {
                        track.stop();
                    });
                });
            }
            if (remoteStreams) {
                remoteStreams.forEach((stream) => {
                    stream.getTracks().forEach((track) => {
                        track.stop();
                    });
                });
            }
        }
        if (this.sessionCache.has(id)) {
            this.sessionCache.delete(id);
            return true;
        }
        return false;
    }
    storeSession(id, session) {
        if (this.sessionCache.has(id)) {
            throw new Error('Session already store');
        }
        this.sessionCache.set(id, session);
    }
    get allSession() {
        return Array.from(this.sessionCache.values());
    }
    getMediaConstraints(req) {
        if (req.screen) {
            return {
                video: false,
                audio: false,
                screen: true,
            };
        }
        return {
            video: req.video || false,
            audio: true,
        };
    }
    getSession(id) {
        if (this.sessionCache.has(id)) {
            return this.sessionCache.get(id);
        }
        return null;
    }
    hasSession(id) {
        return this.sessionCache.has(id);
    }
    async getUserScreen() {
        return new Promise(async (resolve, reject) => {
            try {
                const stream = (await navigator.mediaDevices.getDisplayMedia());
                resolve(stream);
            }
            catch (e) {
                reject(e);
            }
        });
    }
    async getUserCallStream(req) {
        let stream;
        if (req.screen === true) {
            stream = await this.getUserScreen();
        }
        if (req.audio !== false || req.video) {
            const camera = await this.getUserMedia(req.audio, req.video);
            if (stream) {
                camera.getTracks().forEach((track) => {
                    stream.addTrack(track);
                });
            }
            else {
                stream = camera;
            }
        }
        if (!stream) {
            throw new Error('bad stream request');
        }
        return stream;
    }
    async getUserMedia(audio, video) {
        return new Promise(async (resolve, reject) => {
            const mediaConstraints = {
                audio: !(audio === false),
                video: undefined,
            };
            if (video) {
                mediaConstraints.video = {
                    width: { min: 10, ideal: 1280, max: 1920 },
                    height: { min: 10, ideal: 720, max: 1080 },
                    frameRate: { ideal: 10, max: 15 },
                };
            }
            return getMediaStream(mediaConstraints)
                .then(resolve)
                .catch(reject);
        });
    }
}
SipPhone.userAgent = 'Webitel-Phone/0.0.1';
async function getMediaStream(constraints) {
    return new Promise((resolve, reject) => {
        if (navigator.mediaDevices && navigator.mediaDevices.getUserMedia) {
            navigator.mediaDevices
                .getUserMedia(constraints)
                .then((stream) => resolve(stream))
                .catch((err) => reject(err));
        }
        else {
            /// <reference types="webrtc" />
            const getUserMedia = navigator.getUserMedia ||
                navigator.webkitGetUserMedia ||
                navigator.mozGetUserMedia ||
                navigator.msGetUserMedia;
            getUserMedia(constraints, (stream) => resolve(stream), (err) => reject(err));
        }
    });
}

var ChannelName;
(function (ChannelName) {
    ChannelName["Call"] = "call";
    ChannelName["Chat"] = "chat";
    ChannelName["Task"] = "task";
})(ChannelName || (ChannelName = {}));
class Task {
    constructor(client, e, distribute) {
        this.client = client;
        this.distribute = distribute;
        this.id = e.attempt_id;
        this.state = e.status;
        this.lastStatusChange = e.timestamp;
        this.createdAt = e.timestamp;
        this._processing = null;
        this.offeringAt = 0;
        this.answeredAt = 0;
        this.bridgedAt = 0;
        this.startProcessingAt = 0;
        this.stopAt = 0;
        this.closedAt = 0;
        this.communication = distribute.communication;
        this.history = [distribute];
    }
    get queue() {
        return {
            id: this.distribute.queue_id,
            name: this.distribute.queue_name,
        };
    }
    get duration() {
        return Math.round((Date.now() - this.lastStatusChange) / 1000);
    }
    get channel() {
        return this.distribute.channel;
    }
    get allowAccept() {
        return (this.channel === ChannelName.Task &&
            (this.bridgedAt === 0 && this.closedAt === 0));
    }
    get allowDecline() {
        return this.allowAccept;
    }
    get allowClose() {
        return (this.channel === ChannelName.Task &&
            this.closedAt === 0 &&
            this.bridgedAt > 0);
    }
    get memberId() {
        return this.distribute.member_id;
    }
    get queueId() {
        return this.distribute.queue_id;
    }
    get hasReporting() {
        return this.distribute.has_reporting;
    }
    get allowReporting() {
        return this.hasReporting && this.bridgedAt > 0;
    }
    get agentChannelId() {
        return this.distribute.agent_channel_id;
    }
    setAnswered(t) {
        this.answeredAt = t;
        this.lastStatusChange = Date.now();
    }
    setBridged(t) {
        this.bridgedAt = t;
        this.lastStatusChange = Date.now();
    }
    setProcessing(p) {
        if (!this.startProcessingAt) {
            this.startProcessingAt = Date.now();
        }
        if (p.sec) {
            p.timeout = Date.now() + p.sec * 1000; // bug
        }
        this._processing = p;
    }
    setTransferred(d) {
        this.distribute = d;
        this.history.push(d);
    }
    get processingTimeoutAt() {
        if (!this._processing || !this._processing.timeout) {
            return null;
        }
        return this._processing.timeout;
    }
    get processingSec() {
        if (!this._processing || !this._processing.sec) {
            return null;
        }
        return this._processing.sec;
    }
    get renewalSec() {
        if (!this._processing || !this._processing.renewal_sec) {
            return null;
        }
        return this._processing.renewal_sec;
    }
    /*
      control
     */
    async accept() {
        return this.client.request(`cc_agent_task_accept`, {
            agent_id: this.distribute.agent_id,
            attempt_id: this.id,
            app_id: this.distribute.app_id,
        });
    }
    async close() {
        return this.client.request(`cc_agent_task_close`, {
            agent_id: this.distribute.agent_id,
            attempt_id: this.id,
            app_id: this.distribute.app_id,
        });
    }
    // todo
    async decline() {
        return this.client.request(`cc_agent_task_close`, {
            agent_id: this.distribute.agent_id,
            attempt_id: this.id,
            app_id: this.distribute.app_id,
        });
    }
    async reporting(reporting) {
        return this.client.request('cc_reporting', {
            attempt_id: this.id,
            ...reporting,
        });
    }
    async renew(sec) {
        return this.client.request('cc_renewal', {
            attempt_id: this.id,
            renewal_sec: sec ? sec : this.processingSec,
        });
    }
}

var AgentStatus;
(function (AgentStatus) {
    AgentStatus["Online"] = "online";
    AgentStatus["Offline"] = "offline";
    AgentStatus["Pause"] = "pause";
    AgentStatus["BreakOut"] = "break_out";
})(AgentStatus || (AgentStatus = {}));
var ChannelState;
(function (ChannelState) {
    ChannelState["Waiting"] = "waiting";
    ChannelState["Distribute"] = "distribute";
    ChannelState["Offering"] = "offering";
    ChannelState["Answered"] = "answered";
    ChannelState["Active"] = "active";
    ChannelState["Bridged"] = "bridged";
    ChannelState["Hold"] = "hold";
    ChannelState["Missed"] = "missed";
    ChannelState["WrapTime"] = "wrap_time";
    ChannelState["Processing"] = "processing";
    ChannelState["Transfer"] = "transfer";
})(ChannelState || (ChannelState = {}));
var ChannelType;
(function (ChannelType) {
    ChannelType["Call"] = "call";
    ChannelType["Email"] = "email";
    ChannelType["Chat"] = "chat";
})(ChannelType || (ChannelType = {}));
class Agent {
    constructor(client, info) {
        this.client = client;
        this.info = info;
        this.task = new Map();
        this._channel = info.channels[0]; // todo
        this.lastStatusChange = Date.now() - this.info.status_duration * 1000;
    }
    get agentId() {
        return this.info.agent_id;
    }
    get onDemand() {
        return this.info.on_demand;
    }
    get status() {
        return this.info.status;
    }
    get statusPayload() {
        return this.info.status_payload;
    }
    get isAdmin() {
        return this.info.is_admin;
    }
    get isSupervisor() {
        return this.info.is_supervisor;
    }
    get team() {
        return this.info.team;
    }
    get supervisor() {
        return this.info.supervisor;
    }
    get auditor() {
        return this.info.auditor;
    }
    get channels() {
        return [this._channel];
    }
    get channel() {
        return this._channel;
    }
    get stateDuration() {
        return Math.round((Date.now() - this.lastStatusChange) / 1000);
    }
    // todo need refactor
    onChannelEvent(e) {
        let task;
        switch (e.status) {
            case ChannelState.Distribute:
                const distributeEvent = e;
                if (!distributeEvent) {
                    throw new Error('bad event');
                }
                task = new Task(this.client, e, distributeEvent.distribute);
                this.task.set(task.id, task);
                if (task.agentChannelId) {
                    switch (task.channel) {
                        case ChannelType.Call:
                            const call = this.client.callById(task.agentChannelId);
                            if (call && !call.task) {
                                call.task = task;
                            }
                            break;
                        case ChannelType.Chat:
                            const chat = this.client.conversationById(task.agentChannelId);
                            if (chat && !chat.task) {
                                chat.task = task;
                            }
                            break;
                        default:
                    }
                }
                break;
            case ChannelState.Offering:
                task = this.task.get(e.attempt_id);
                if (task) {
                    task.offeringAt = e.timestamp;
                }
                break;
            case ChannelState.Bridged:
                task = this.task.get(e.attempt_id);
                if (task) {
                    task.bridgedAt = e.timestamp;
                }
                break;
            case ChannelState.Transfer:
                const transfer = e;
                task = this.task.get(transfer.to_attempt_id);
                if (task) {
                    this.task.delete(task.id);
                    task.id = transfer.attempt_id;
                    task.setTransferred(transfer.distribute);
                    this.task.set(task.id, task);
                }
                break;
            case ChannelState.Answered:
                task = this.task.get(e.attempt_id);
                if (task) {
                    task.answeredAt = e.timestamp;
                }
                break;
            case ChannelState.Missed:
                if (e.attempt_id) {
                    task = this.task.get(e.attempt_id);
                    if (task) {
                        task.stopAt = e.timestamp;
                        const missedEvent = e;
                        if (!missedEvent) {
                            throw new Error('bad event');
                        }
                        this.setChannelStateTimeout(e.channel, e, missedEvent.missed.timeout);
                        this.task.delete(e.attempt_id);
                        this.client.reportingChannelTask(task);
                        return task;
                    }
                }
                break;
            case ChannelState.WrapTime:
                if (e.attempt_id) {
                    const wrapTimeEvent = e;
                    if (!wrapTimeEvent) {
                        throw new Error('bad event');
                    }
                    task = this.task.get(e.attempt_id);
                    this.task.delete(e.attempt_id);
                    this.client.reportingChannelTask(task);
                }
                break;
            case ChannelState.Processing:
                if (e.attempt_id) {
                    const processingEvent = e;
                    if (!processingEvent) {
                        throw new Error('bad event');
                    }
                    task = this.task.get(e.attempt_id);
                    if (task) {
                        task.state = e.status;
                        task.closedAt = e.timestamp; // todo
                        task.setProcessing(processingEvent.processing);
                        this.setChannelStateTimeout(e.channel, e, processingEvent.processing.timeout);
                        return task;
                    }
                }
                break;
            // TODO
            case ChannelState.Waiting:
                if (e.attempt_id) {
                    task = this.task.get(e.attempt_id);
                    if (task) {
                        task.stopAt = e.timestamp;
                        this.task.delete(e.attempt_id);
                        this.client.reportingChannelTask(task);
                    }
                }
                break;
            default:
            // throw new Error("not found task")
        }
        this.setChannelState(e.channel, e);
        if (task) {
            task.state = e.status;
            return task;
        }
        else {
            return undefined;
        }
    }
    async online(channels, onDemand) {
        return this.client.agentSetOnline(this.agentId, channels, onDemand);
    }
    async waiting(channel) {
        return this.client.agentSetWaiting(this.agentId, channel);
    }
    async pause(payload) {
        return this.client.agentSetPause(this.agentId, payload);
    }
    async offline() {
        return this.client.agentSetOffline(this.agentId);
    }
    setStatus(e) {
        if (e.status === AgentStatus.Online) {
            this.info.on_demand = e.on_demand || false;
            this._channel = e.channels[0];
        }
        this.info.status = e.status;
        this.info.status_payload = e.status_payload ? e.status_payload : null;
        this.lastStatusChange = Date.now();
    }
    async directMember(memberId, communicationId) {
        return this.client.request(`cc_member_direct`, {
            agent_id: this.agentId,
            member_id: memberId,
            communication_id: communicationId,
        });
    }
    async offlineMembers(q, page, perPage) {
        return this.client.request(`cc_fetch_offline_members`, {
            agent_id: this.agentId,
            page,
            q,
            per_page: perPage,
        });
    }
    hasTask(task) {
        return this.task.has(task.id);
    }
    setChannelState(name, e) {
        this._channel.channel = name ? name : null;
        this._channel.state = e.status;
        this._channel.joined_at = e.timestamp;
        this._channel.timeout = undefined;
    }
    setChannelStateTimeout(name, e, timeout) {
        this._channel.channel = name ? name : null;
        this._channel.state = e.status; // FIXME
        this._channel.timeout = timeout;
        this._channel.joined_at = e.timestamp;
    }
}

var CallReportingStatus;
(function (CallReportingStatus) {
    CallReportingStatus["Abandoned"] = "abandoned";
    CallReportingStatus["Cancel"] = "cancel";
    CallReportingStatus["Success"] = "success";
})(CallReportingStatus || (CallReportingStatus = {}));
var CallActions;
(function (CallActions) {
    CallActions["Ringing"] = "ringing";
    CallActions["Active"] = "active";
    CallActions["Bridge"] = "bridge";
    CallActions["Hold"] = "hold";
    CallActions["DTMF"] = "dtmf";
    CallActions["Voice"] = "voice";
    CallActions["Silence"] = "silence";
    CallActions["Execute"] = "execute";
    CallActions["Update"] = "update";
    CallActions["Hangup"] = "hangup";
    CallActions["Reporting"] = "reporting";
    CallActions["PeerStream"] = "peerStream";
    CallActions["LocalStream"] = "localStream";
    CallActions["Destroy"] = "destroy";
})(CallActions || (CallActions = {}));
var CallDirection;
(function (CallDirection) {
    CallDirection["Inbound"] = "inbound";
    CallDirection["Outbound"] = "outbound";
})(CallDirection || (CallDirection = {}));
class Call {
    constructor(client, e) {
        this.client = client;
        // FIXME check _muted from channel
        const callInfo = e.data;
        this._muted = false;
        this.voice = true;
        this.createdAt = +e.timestamp;
        this.task = null;
        this.data = null;
        this.answeredAt = 0;
        this.hangupAt = 0;
        this.bridgedAt = 0;
        this.reportingAt = 0;
        this.autoAnswered = false;
        this.peerStreams = [];
        this.localStreams = [];
        // fixme
        if (client.phone) {
            if (callInfo.sip_id) {
                this.setSip(client.phone.sipSessionBySipId(callInfo.sip_id));
            }
            else {
                this.setSip(client.phone.sipSessionByCallId(e.id));
            }
        }
        this.params = {};
        this.id = e.id;
        this.digits = [];
        this.applications = [];
        this.appId = e.app_id;
        this.setState(e);
        this.setInfo(callInfo);
        if (this.queue && this.client.agent) {
            this.task = this.client.agent.task.get(+this.queue.attempt_id) || null;
        }
    }
    // set private
    setState(s) {
        this.state = s.event;
    }
    setSip(sip) {
        if (sip && !this.sip) {
            this.sip = sip;
            this.localStreams = sip.getLocalMedia();
            this.peerStreams = sip.getPeerMedia();
        }
    }
    get duration() {
        if (this.hangupAt === 0) {
            return Math.round((Date.now() - this.createdAt) / 1000); // tslint:disable-line
        }
        else {
            return Math.round((this.hangupAt - this.createdAt) / 1000); // tslint:disable-line
        }
    }
    get allowDtmf() {
        return this.answeredAt > 0 && this.allowHangup;
    }
    get allowHangup() {
        return this.hangupAt === 0;
    }
    get allowHold() {
        return (this.hangupAt === 0 &&
            (this.state === 'active' || this.state === 'bridge'));
    }
    get allowAnswer() {
        return (this.client.phone &&
            this.hangupAt === 0 &&
            this.answeredAt === 0 &&
            (this.queue ||
                this.direction === CallDirection.Inbound ||
                (this.params && !this.params.autoAnswer)));
    }
    get allowUnHold() {
        return this.hangupAt === 0 && this.state === 'hold';
    }
    get memberCommunication() {
        if (!this.task) {
            return null;
        }
        else {
            return this.task.communication;
        }
    }
    // todo
    async getMember(req) {
        if (!this.isMember) {
            throw new Error(`call is not from outbound queue`);
        }
        return this.client.request(`cc_member_page`, {
            queue_id: this.task.queueId,
            member_id: this.task.memberId,
        });
    }
    get isMember() {
        return !!(this.task && this.task.memberId);
    }
    setActive(e) {
        if (!this.answeredAt) {
            if (this.direction === CallDirection.Inbound) {
                this.bridgedAt = +e.timestamp;
                if (this.parentId) {
                    this.bridgedId = this.parentId;
                }
            }
            this.answeredAt = +e.timestamp;
        }
        this.setState(e);
    }
    setBridged(s) {
        const bridged = s.data;
        if (!this.bridgedAt) {
            this.bridgedAt = +s.timestamp;
        }
        this.bridgedId = bridged.bridged_id;
        if (bridged.to) {
            if (this.direction === CallDirection.Inbound) {
                this.from = bridged.to;
            }
            else {
                this.to = bridged.to;
            }
        }
        this.bridgedId = bridged.bridged_id;
        if (bridged.payload) {
            this.variables = bridged.payload;
        }
        if (bridged.queue) {
            this.queue = bridged.queue;
        }
    }
    setHold(e) {
        this.setState(e);
    }
    setInfo(s) {
        this.parentId = s.parent_id;
        this.destination = s.destination;
        this.direction = s.direction;
        this.from = s.from;
        this.to = s.to;
        this.variables = s.payload;
        this.queue = s.queue || null;
        this.sipId = s.sip_id || null;
        this.params = s.params;
    }
    setPeerStreams(streams) {
        if (!streams) {
            this.peerStreams = [];
        }
        else {
            this.peerStreams = streams;
        }
    }
    setLocalStreams(streams) {
        if (!streams) {
            this.localStreams = [];
        }
        else {
            this.localStreams = streams;
        }
    }
    setVoice() {
        this.voice = true;
    }
    setSilence() {
        this.voice = false;
    }
    setExecute(s) {
        this.applications.push(s.application);
    }
    addDigit(s) {
        this.digits.push(s.digit);
    }
    setHangup(s) {
        const hangup = s.data;
        this.hangupAt = +s.timestamp;
        this.hangupCause = hangup.cause;
        this.hangupSipCode = hangup.sip;
        this.voice = false;
        this.peerStreams = [];
        if (+hangup.reporting_at) {
            this.reportingAt = +hangup.reporting_at; // FIXME type number
        }
        this.setState(s);
    }
    get allowReporting() {
        return this.answeredAt > 0 && this.hasReporting;
    }
    get hasReporting() {
        if (this.task) {
            return this.task.hasReporting;
        }
        return false;
    }
    get display() {
        return `${this.displayNumber} (${this.displayName})`;
    }
    get muted() {
        return this._muted;
    }
    get isHold() {
        return this.state === CallActions.Hold;
    }
    get allowInboundVideo() {
        if (this.params) {
            return this.params.video || false;
        }
        return false;
    }
    get allowOutboundVideo() {
        if (this.params) {
            return this.params.video || false;
        }
        return false;
    }
    get active() {
        return this.hangupAt === 0;
    }
    get displayNumber() {
        if (this.direction === 'inbound') {
            return this.from.number;
        }
        else {
            if (this.to) {
                return this.to.number;
            }
            return this.destination;
        }
    }
    get displayName() {
        if (this.direction === 'inbound') {
            return this.from.name;
        }
        else {
            if (this.to) {
                return this.to.name;
            }
            return this.destination;
        }
    }
    get autoAnswer() {
        return ((this.queue && this.queue.queue_type === 'offline') ||
            (this.params && this.params.autoAnswer === true));
    }
    /* Call control */
    async answer(req) {
        if (this.sip && this.client.phone) {
            const params = await this.client.phone.callOption(req);
            await this.sip.answer(params);
            return true;
        }
        return false;
    }
    answerDelay(req, delay) {
        if (this.autoAnswered) {
            return;
        }
        this.autoAnswered = true;
        setTimeout(async () => {
            await this.answer(req);
        }, delay);
    }
    async hangup(cause) {
        let _cause = cause;
        if (this.answeredAt === 0 && !cause) {
            _cause =
                this.direction === CallDirection.Inbound
                    ? 'USER_BUSY'
                    : 'ORIGINATOR_CANCEL';
        }
        return this.client.request('call_hangup', {
            id: this.id,
            app_id: this.appId,
            cause: _cause,
        });
    }
    async eavesdrop(req) {
        req.id = this.id;
        return this.client.request('call_eavesdrop', req);
    }
    async toggleHold() {
        if (this.state === CallActions.Hold) {
            return this.unHold();
        }
        else {
            return this.hold();
        }
    }
    async hold() {
        if (this.state === CallActions.Hold) {
            throw new Error('Call is hold');
        }
        return this.client.request('call_hold', {
            id: this.id,
            app_id: this.appId,
        });
    }
    async unHold() {
        if (this.state !== CallActions.Hold) {
            throw new Error('Call is active');
        }
        return this.client.request('call_unhold', {
            id: this.id,
            app_id: this.appId,
        });
    }
    async sendDTMF(dtmf) {
        return this.client.request('call_dtmf', {
            id: this.id,
            app_id: this.appId,
            dtmf,
        });
    }
    async blindTransfer(destination) {
        return this.client.request('call_blind_transfer', {
            id: this.id,
            app_id: this.appId,
            destination,
        });
    }
    async mute(mute = false) {
        const res = await this.client.request('call_mute', {
            id: this.id,
            app_id: this.appId,
            mute,
        });
        this._muted = mute;
        return res;
    }
    async bridgeTo(call) {
        return this.client.request('call_bridge', {
            from_id: this.id,
            from_app_id: this.appId,
            to_id: call.id,
            to_app_id: call.appId,
        });
    }
    async reporting(reporting) {
        if (!this.task) {
            throw new Error(`this call not in queue`);
        }
        return this.task.reporting(reporting);
    }
    async renew(sec) {
        if (!this.task) {
            throw new Error(`this conversation not in queue`);
        }
        return this.task.renew(sec);
    }
    async callToUser(req) {
        req.nodeId = this.appId;
        req.parentCallId = this.id || null;
        req.sendToCallId = this.parentId || null;
        return this.client.inviteToUser(req);
    }
}

function formatWebSocketUri(host) {
    return host.replace(/^http/, 'ws');
}
function formatBaseUri(host) {
    let res = host.replace(/^ws/, 'http');
    if (res.endsWith('/')) {
        res = res.slice(0, -1);
    }
    if (res.endsWith('/ws')) {
        res = res.slice(0, -3);
    }
    return res;
}
function chunkString(str, len) {
    const size = Math.ceil(str.length / len);
    const r = new Array(size);
    let offset = 0;
    for (let i = 0; i < size; i++) {
        r[i] = str.substr(offset, len);
        offset += len;
    }
    return r;
}

const maxSizeMessage = 4096;
var ChatActions;
(function (ChatActions) {
    ChatActions["Message"] = "message";
    ChatActions["UserInvite"] = "user_invite";
    ChatActions["Invite"] = "invite_conversation";
    ChatActions["Joined"] = "join_conversation";
    ChatActions["Close"] = "close_conversation";
    ChatActions["Leave"] = "leave_conversation";
    ChatActions["Decline"] = "decline_invite";
    ChatActions["Update"] = "update_channel";
    ChatActions["Destroy"] = "destroy";
})(ChatActions || (ChatActions = {}));
var ConversationState;
(function (ConversationState) {
    ConversationState["Init"] = "init";
    ConversationState["Invite"] = "invite";
    ConversationState["Active"] = "active";
    ConversationState["Pending"] = "pending";
    ConversationState["Closed"] = "closed";
})(ConversationState || (ConversationState = {}));
class Conversation {
    constructor(client, conversationId, title, members, messages, variables) {
        this.client = client;
        this.conversationId = conversationId;
        this.title = title;
        this.channelId = null;
        this.createdAt = Date.now();
        this.answeredAt = 0;
        this.updatedAt = 0;
        this.invitedAt = 0;
        this.closedAt = 0;
        this.task = null;
        this.data = null;
        this.members = (members || []).map((i) => wrapChannelMember(i));
        this._messages = messages || [];
        this.state = ConversationState.Invite;
        this.variables = variables;
        if (variables &&
            variables.hasOwnProperty('cc_attempt_id') &&
            this.client.agent) {
            this.task = this.client.agent.task.get(+variables.cc_attempt_id) || null;
        }
    }
    setInvite(inviteId, timestamp) {
        this.inviteId = inviteId;
        this.invitedAt = timestamp;
    }
    setAnswered(channelId, timestamp, member) {
        this.state = ConversationState.Active;
        this.answeredAt = timestamp;
        this.channelId = channelId;
        this.member = wrapChannelMember(member);
        this.inviteId = null; // deleted in DB
    }
    setClosed(timestamp) {
        this.state = ConversationState.Closed;
        this.closedAt = timestamp;
    }
    get id() {
        return this.conversationId;
    }
    get messages() {
        return this.getMessages();
    }
    getMessages() {
        return this._messages.map((i) => {
            const msg = {
                id: i.id,
                type: i.type,
                member: this.messageMember(i),
                channelId: i.channel_id,
                createdAt: i.created_at,
                updatedAt: i.updated_at,
            };
            if (i.hasOwnProperty('file')) {
                i.file.url = this.client.fileUrlDownload(i.file.id);
                msg.file = i.file;
            }
            if (i.hasOwnProperty('text')) {
                msg.text = i.text;
            }
            return msg;
        });
    }
    async next() {
        throw new Error('TODO');
    }
    async pagination(page, perPage) {
        throw new Error('TODO');
    }
    newMessage(e) {
        this._messages.push(e);
    }
    get allowDecline() {
        return !!this.inviteId;
    }
    get allowJoin() {
        return !!this.inviteId;
    }
    get allowLeave() {
        return !!this.channelId && this.closedAt === 0;
    }
    get allowReporting() {
        return this.answeredAt > 0 && this.hasReporting;
    }
    get hasReporting() {
        return this.variables && this.variables.cc_reporting === 'true';
    }
    /*
    Actions
     */
    async decline() {
        if (!this.inviteId)
            throw new Error('This conversation is joined');
        return this.client.request(`decline_chat`, {
            invite_id: this.inviteId,
        });
    }
    async join() {
        if (!this.inviteId)
            throw new Error('This conversation is joined');
        return this.client.request(`join_chat`, {
            invite_id: this.inviteId,
        });
    }
    async close(cause) {
        if (!this.channelId)
            throw new Error('This conversation not active');
        return this.client.request(`close_chat`, {
            channel_id: this.channelId,
            conversation_id: this.id,
            cause,
        });
    }
    async leave(cause) {
        if (!this.channelId)
            throw new Error('This conversation not active');
        return this.client.request(`leave_chat`, {
            channel_id: this.channelId,
            conversation_id: this.id,
            cause,
        });
    }
    async sendText(text) {
        if (!this.channelId) {
            throw new Error('conversation not active');
        }
        if (!text.length) {
            throw new Error('empty message');
        }
        const chunk = chunkString(text, maxSizeMessage);
        for (let i = 0; i < chunk.length; i++) {
            if (i + 1 >= chunk.length) {
                return this.sendMessageTextChunk(chunk[i]);
            }
            await this.sendMessageTextChunk(chunk[i]);
        }
    }
    async sendFile(file, cb) {
        const storedFiles = await this.client.storeFile(this.id, [file], cb);
        const f = storedFiles[0];
        // todo bug if chat response error
        return this.client.request(`send_file_chat`, {
            channel_id: this.channelId,
            conversation_id: this.id,
            id: f.id,
            name: file.name,
            mime: f.mime,
            url: f.shared,
            size: f.size,
        });
    }
    async send(data, cb) {
        if (typeof data === 'string') {
            return this.sendText(data);
        }
        else if (data instanceof File) {
            return this.sendFile(data, cb);
        }
        else {
            throw new Error('unknown send data');
        }
    }
    async addToChat(userId, title) {
        return this.client.request(`add_to_chat`, {
            channel_id: this.channelId,
            conversation_id: this.id,
            user_id: userId,
            title,
        });
    }
    async startChat(userId) {
        return this.client.request(`start_chat`, {
            user_id: userId,
        });
    }
    async updateChannel() {
        return this.client.request(`update_channel_chat`, {
            channel_id: this.channelId,
        });
    }
    async reporting(reporting) {
        if (!this.task) {
            throw new Error(`this conversation not in queue`);
        }
        return this.task.reporting(reporting);
    }
    async renew(sec) {
        if (!this.task) {
            throw new Error(`this conversation not in queue`);
        }
        return this.task.renew(sec);
    }
    sendMessageTextChunk(text) {
        return this.client.request(`send_text_chat`, {
            channel_id: this.channelId,
            conversation_id: this.id,
            text,
        });
    }
    messageMember(msg) {
        if (this.member && msg.channel_id === this.channelId) {
            return {
                self: true,
            };
        }
        for (const m of this.members) {
            if (m.id === msg.channel_id) {
                return m;
            }
        }
        return null; // TODO
    }
}
function wrapChannelMember(m) {
    // todo
    m.messenger = m.type;
    return m;
}

const SOCKET_URL_SUFFIX = 'websocket';
const spamData = `\u0000\u0000\u0000\u0000`;
class Socket extends EventEmitter {
    constructor(host) {
        super();
        this.host = host;
        // this.socket = null
    }
    connect(token) {
        this.socket = new WebSocket(`${formatWebSocketUri(this.host)}/${SOCKET_URL_SUFFIX}`);
        this.socket.onclose = (e) => this.onClose(e.code);
        this.socket.onmessage = (e) => this.onMessage(e.data);
        this.socket.onopen = () => this.onOpen();
    }
    send(request) {
        this.socket.send(JSON.stringify(request));
        return null;
    }
    close(code) {
        this.socket.close(code || 1000);
        delete this.socket;
    }
    onOpen() {
        this.emit('open');
    }
    onClose(code) {
        this.emit('close', code);
        this.off('*');
    }
    onMessage(data) {
        if (data !== spamData) {
            const message = JSON.parse(data);
            this.emit('message', message);
        }
    }
}

// tslint:disable-next-line: match-default-export-name
const API_HEADER_TOKEN = 'X-Webitel-Access';
const WEBSOCKET_AUTHENTICATION_CHALLENGE = 'authentication_challenge';
const WEBSOCKET_DEFAULT_DEVICE_CONFIG = 'user_default_device';
const WEBSOCKET_AGENT_SESSION = 'cc_agent_session';
const WEBSOCKET_PING = 'ping';
// const WEBSOCKET_CALL_BY_USER = 'call_by_user'
const WEBSOCKET_MAKE_OUTBOUND_CALL = 'call_invite';
const WEBSOCKET_MAKE_USER_CALL = 'call_user';
const WEBSOCKET_EVENT_HELLO = 'hello';
const WEBSOCKET_EVENT_CALL = 'call';
const WEBSOCKET_EVENT_CHAT = 'chat';
const WEBSOCKET_EVENT_USER_STATE = 'user_state';
const WEBSOCKET_EVENT_AGENT_STATUS = 'agent_status';
const WEBSOCKET_EVENT_CHANNEL_STATUS = 'channel';
const TASK_EVENT = 'task';
const WEBSOCKET_EVENT_SIP = 'sip';
var Response;
(function (Response) {
    Response["STATUS_FAIL"] = "FAIL";
    Response["STATUS_OK"] = "OK";
})(Response || (Response = {}));
class Client extends EventEmitter {
    constructor(_config) {
        super();
        this._config = _config;
        this.reqSeq = 0;
        this.queueRequest = new Map();
        this.log = new Log();
        this.eventHandler = new EventEmitter();
        this.callStore = new Map();
        this.conversationStore = new Map();
        this.pingTimer = null;
        this.basePath = `${formatBaseUri(_config.storageEndpoint || _config.endpoint)}`;
    }
    async connect() {
        await this.connectToSocket();
    }
    async disconnect() {
        this.socket.close();
    }
    async subscribeCall(handler, data) {
        const calls = (await this.request(`subscribe_call`, data));
        this.eventHandler.on(WEBSOCKET_EVENT_CALL, handler);
        if (calls.items && calls.items.length) {
            for (const c of calls.items) {
                if (c.hangup_at && c.hangup_at > 0 && !c.task) {
                    continue;
                }
                const e = {
                    cc_app_id: '',
                    app_id: c.app_id,
                    event: c.state,
                    id: c.id,
                    timestamp: Date.now(),
                    data: {
                        app_id: c.app_id,
                        event: c.state,
                        id: c.id,
                        timestamp: Date.now(),
                        cc_app_id: '',
                        sip_id: '',
                        parent_id: c.parent_id,
                        direction: c.direction,
                        destination: c.destination,
                        queue: c.queue,
                        from: c.from,
                        to: c.to,
                        payload: c.variables,
                    },
                };
                const call = new Call(this, e);
                call.createdAt = c.created_at;
                call.answeredAt = c.answered_at || 0;
                call.bridgedAt = c.bridged_at || 0;
                call.hangupAt = c.hangup_at || 0;
                call.reportingAt = c.reporting_at || 0;
                if (c.task) {
                    call.task = new Task(this, {
                        attempt_id: c.task.attempt_id,
                        channel: c.task.channel,
                        status: c.task.status,
                        timestamp: Date.now(),
                    }, c.task // todo add app_id
                    );
                    call.queue.reporting = 'true';
                }
                this.callStore.set(call.id, call);
            }
        }
        return calls;
    }
    async subscribeChat(handler, data) {
        const res = (await this.request(`subscribe_chat`, data));
        this.eventHandler.on(WEBSOCKET_EVENT_CHAT, handler);
        if (res && res.items) {
            for (const conv of res.items) {
                // tslint:disable-next-line: no-assign-mutated-array
                const messages = conv.messages.reverse();
                const c = new Conversation(this, conv.id, conv.title, conv.members, messages, conv.variables);
                c.createdAt = conv.created_at;
                if (conv.invite_id) {
                    c.setInvite(conv.invite_id, conv.created_at);
                }
                else if (conv.channel_id) {
                    c.setAnswered(conv.channel_id, conv.joined_at, {
                        id: conv.channel_id,
                        name: 'TODO',
                        type: 'webitel',
                    });
                }
                else {
                    this.log.error(`conversation ${conv.id} not valid`);
                    continue;
                }
                this.conversationStore.set(c.id, c);
            }
        }
        return res;
    }
    async subscribeUsersStatus(handler, data) {
        const res = await this.request(`subscribe_users_status`, data);
        this.eventHandler.on(WEBSOCKET_EVENT_USER_STATE, handler);
        return res;
    }
    async subscribeAgentsStatus(handler, data) {
        const res = await this.request(`cc_agent_subscribe_status`, data);
        this.eventHandler.on(WEBSOCKET_EVENT_AGENT_STATUS, handler);
        return res;
    }
    subscribeTask(handler) {
        this.eventHandler.on(TASK_EVENT, handler);
    }
    async unSubscribe(action, handler, data) {
        const res = await this.request(`un_subscribe_${action}`, data);
        // this.eventHandler.listeners(action)
        // this.eventHandler.removeListener(action, handler)
        // this.eventHandler.off(action, handler)
        return res;
    }
    async destroy() {
        await this.disconnect();
        if (this.phone) {
            await this.phone.unregister();
            this.phone.off('*');
            delete this.phone;
        }
        this.eventHandler.off('*');
    }
    allCall() {
        return Array.from(this.callStore.values());
    }
    allConversations() {
        return Array.from(this.conversationStore.values());
    }
    allTask() {
        if (!this.agent) {
            return [];
        }
        return Array.from(this.agent.task.values());
    }
    callById(id) {
        if (this.callStore.has(id)) {
            return this.callStore.get(id);
        }
    }
    conversationById(id) {
        if (this.conversationStore.has(id)) {
            return this.conversationStore.get(id);
        }
    }
    callBySipId(id) {
        for (const call of this.allCall()) {
            if (call.sipId && id.startsWith(call.sipId)) {
                // FIXME
                return call;
            }
        }
    }
    async reportingTask(attemptId, reporting) {
        return this.request('cc_reporting', {
            attempt_id: attemptId,
            ...reporting,
        });
    }
    async agentActiveTasks(agentId) {
        return this.request(`cc_agent_tasks`, {
            agent_id: agentId,
        });
    }
    async auth() {
        return this.request(WEBSOCKET_AUTHENTICATION_CHALLENGE, {
            token: this._config.token,
        });
    }
    async ping() {
        return this.request(WEBSOCKET_PING, {
            ping: 1,
        });
    }
    sessionInfo() {
        return this.connectionInfo.session;
    }
    get version() {
        return this.connectionInfo.server_version;
    }
    get instanceId() {
        return this.connectionInfo.sock_id;
    }
    async agentSession() {
        const info = await this.request(WEBSOCKET_AGENT_SESSION);
        this.agent = new Agent(this, info);
        const taskList = (await this.agentActiveTasks(this.agent.agentId));
        // todo
        for (const t of taskList.items) {
            const task = new Task(this, {
                status: t.status,
                timestamp: Date.now() - t.duration * 1000,
                channel: t.channel,
                attempt_id: t.attempt_id,
            }, t);
            task.postProcessData = {};
            task.state = t.state;
            if (t.bridged_at) {
                task.bridgedAt = t.bridged_at;
                task.answeredAt = t.bridged_at;
            }
            if (t.leaving_at) {
                task.closedAt = t.leaving_at;
            }
            this.agent.task.set(task.id, task);
        }
        for (const call of this.allCall()) {
            if (call.task) {
                this.agent.task.set(call.task.id, call.task);
            }
        }
        return this.agent;
    }
    async invite(req) {
        return this.request(WEBSOCKET_MAKE_OUTBOUND_CALL, req);
    }
    async call(req) {
        if (this.phone) {
            await this.phone.call(req);
        }
        else {
            await this.invite(req);
        }
    }
    async callRecordId(id) {
        return this.request('call_recordings', { id });
    }
    async eavesdrop(req) {
        return this.request('call_eavesdrop', req);
    }
    async agentSetOnline(agentId, channels, onDemand) {
        return this.request('cc_agent_online', {
            agent_id: agentId,
            channels,
            on_demand: onDemand,
        });
    }
    async agentSetWaiting(agentId, channel) {
        return this.request('cc_agent_waiting', {
            agent_id: agentId,
            channel,
        });
    }
    async agentSetPause(agentId, payload) {
        return this.request('cc_agent_pause', {
            agent_id: agentId,
            payload,
        });
    }
    async agentSetOffline(agentId) {
        return this.request('cc_agent_offline', { agent_id: agentId });
    }
    inviteToUser(req) {
        return this.request(WEBSOCKET_MAKE_USER_CALL, req);
    }
    async answer(id, req) {
        if (this.phone) {
            return this.phone.answer(id, req);
        }
    }
    request(action, data) {
        return new Promise((resolve, reject) => {
            this.queueRequest.set(++this.reqSeq, { resolve, reject });
            this.socket.send({
                seq: this.reqSeq,
                action,
                data,
            });
        });
    }
    useWebPhone() {
        return this._config.registerWebDevice || false;
    }
    callDestroyed(call) {
        return call.hangupAt > 0 && !this.hasAgentTask(call.task);
    }
    conversationDestroyed(conv) {
        return conv.closedAt > 0 && !this.hasAgentTask(conv.task);
    }
    reportingChannelTask(task) {
        switch (task.channel) {
            case ChannelName.Call:
                for (const call of this.allCall()) {
                    if (this.callDestroyed(call)) {
                        this.destroyCall(call);
                        return;
                    }
                }
                break;
            case ChannelName.Chat:
                if (task.agentChannelId) {
                    const conv = this.conversationById(task.agentChannelId);
                    if (conv && this.conversationDestroyed(conv)) {
                        this.destroyConversation(conv);
                        return;
                    }
                }
                break;
            default:
        }
    }
    async registerCallClient(phone) {
        this.phone = phone;
        this.phone.on('peerStreams', (session, streams) => {
            const call = this.callBySession(session);
            if (call && !call.peerStreams.length) {
                call.setPeerStreams(streams);
                this.eventHandler.emit(WEBSOCKET_EVENT_CALL, CallActions.PeerStream, call);
            }
        });
        this.phone.on('localStreams', (session, streams) => {
            const call = this.callBySession(session);
            if (call && !call.localStreams.length) {
                call.setLocalStreams(streams);
                this.eventHandler.emit(WEBSOCKET_EVENT_CALL, CallActions.LocalStream, call);
            }
        });
        this.phone.on('newSession', this.onNewCallSession.bind(this));
        try {
            const conf = await this.deviceConfig(this.phone.type);
            await this.phone.register(conf);
        }
        catch (e) {
            // FIXME add handle error
            this.log.error(e);
        }
    }
    async deviceConfig(name) {
        return this.request(WEBSOCKET_DEFAULT_DEVICE_CONFIG, { name });
    }
    async storeFile(id, files, cb) {
        if (!files || files.length < 1) {
            throw new Error('no files');
        }
        const formData = new FormData();
        for (const file of files) {
            formData.append(file.name, file); // todo name
        }
        const result = await globalAxios.post(`${this.basePath}/api/storage/file/${id}/upload`, formData, {
            headers: {
                [API_HEADER_TOKEN]: this._config.token,
            },
            onUploadProgress: cb,
        });
        const data = result.data;
        for (const f of data) {
            f.shared = this.basePath + f.shared;
        }
        return data;
    }
    fileUrlDownload(fileId) {
        return `${this.basePath}/api/storage/file/${fileId}/download?access_token=${this._config.token}`;
    }
    fileUrlStream(fileId) {
        return `${this.basePath}/api/storage/file/${fileId}/stream?access_token=${this._config.token}`;
    }
    async onMessage(message) {
        this.log.debug('receive message: ', message);
        if (message.seq_reply > 0) {
            if (this.queueRequest.has(message.seq_reply)) {
                const promise = this.queueRequest.get(message.seq_reply);
                this.queueRequest.delete(message.seq_reply);
                if (message.status === Response.STATUS_OK) {
                    promise.resolve(message.data);
                }
                else {
                    promise.reject(message.error);
                }
            }
        }
        else {
            switch (message.event) {
                case WEBSOCKET_EVENT_HELLO:
                    await this.connected(message.data);
                    this.log.debug(`opened session ${this.connectionInfo.sock_id} for userId=${this.connectionInfo.session.user_id}`);
                    break;
                case WEBSOCKET_EVENT_CALL:
                    await this.handleCallEvents(message.data.call);
                    break;
                case WEBSOCKET_EVENT_CHAT:
                    await this.handleChatEvents(message.data);
                    break;
                case WEBSOCKET_EVENT_USER_STATE:
                    this.handleUserStateEvent(message.data.state);
                    break;
                case WEBSOCKET_EVENT_SIP:
                    this.eventHandler.emit(WEBSOCKET_EVENT_SIP, message.data);
                    break;
                case WEBSOCKET_EVENT_AGENT_STATUS:
                    this.handleAgentStatus(message.data);
                    break;
                case WEBSOCKET_EVENT_CHANNEL_STATUS:
                    this.handleChannelEvents(message.data);
                    break;
                default:
                    this.log.error(`event ${message.event} not handler`);
            }
        }
    }
    handleAgentStatus(e) {
        if (this.agent && e.agent_id === this.agent.agentId) {
            this.agent.setStatus(e);
        }
        this.eventHandler.emit(WEBSOCKET_EVENT_AGENT_STATUS, e, this.agent);
    }
    handleChannelEvents(e) {
        if (this.agent) {
            e.timestamp = Date.now(); // bug
            const task = this.agent.onChannelEvent(e) || undefined;
            this.eventHandler.emit(TASK_EVENT, e.status, task);
        }
    }
    async pingServer() {
        if (this.pingTimer) {
            await this.ping();
        }
        if (this.connectionInfo.ping_interval) {
            // @ts-ignore
            this.pingTimer = setTimeout(this.pingServer.bind(this), +this.connectionInfo.ping_interval);
        }
    }
    async connected(info) {
        this.connectionInfo = info;
        if (info.ping_interval) {
            await this.pingServer();
        }
        if (!this.useWebPhone()) {
            return;
        }
        return this.registerCallClient(new SipPhone(this.instanceId, this._config.debug));
    }
    callBySession(session) {
        for (const call of this.allCall()) {
            if (call.sip === session) {
                return call;
            }
        }
    }
    async onNewCallSession(session) {
        let call;
        if (session.callId) {
            call = this.callById(session.callId);
        }
        else {
            call = this.callBySipId(session.id);
        }
        if (call) {
            call.setSip(session);
            this.checkAutoAnswer(call);
        }
    }
    checkAutoAnswer(call) {
        if ( /*!document.hidden && */call.autoAnswer) {
            call.answerDelay({
                video: call.params.video,
                screen: call.params.screen,
                disableStun: call.params.disableStun,
            }, 300);
        }
    }
    connectToSocket() {
        return new Promise((resolve, reject) => {
            try {
                this.socket = new Socket(this._config.endpoint);
                this.socket.connect(this._config.token);
            }
            catch (e) {
                reject(e);
                return;
            }
            this.socket.on('message', this.onMessage.bind(this));
            this.socket.on('close', (code) => {
                if (this.pingTimer) {
                    clearTimeout(this.pingTimer);
                }
                this.log.error('socket close code: ', code);
                this.eventHandler.off('*');
                if (code !== 1000) {
                    this.emit('disconnected', code);
                    reject(new Error(`close socket code: ${code}`));
                }
            });
            this.socket.on('open', () => {
                resolve(null);
                this.emit('connected');
            });
        });
    }
    handleUserStateEvent(event) {
        this.eventHandler.emit(WEBSOCKET_EVENT_USER_STATE, event);
    }
    async handleCallEvents(event) {
        let call;
        event.timestamp = Date.now(); // todo bug
        switch (event.event) {
            case CallActions.Ringing:
                call = new Call(this, event);
                this.callStore.set(call.id, call);
                this.checkAutoAnswer(call);
                break;
            case CallActions.Active:
                call = this.callById(event.id);
                if (call) {
                    call.setActive(event);
                }
                break;
            case CallActions.Bridge:
                call = this.callById(event.id);
                if (call) {
                    call.setBridged(event);
                }
                break;
            case CallActions.Execute:
                call = this.callById(event.id);
                if (call) {
                    call.setExecute(event.data);
                }
                break;
            case CallActions.DTMF:
                call = this.callById(event.id);
                if (call) {
                    call.addDigit(event.data);
                }
                break;
            case CallActions.Voice:
                call = this.callById(event.id);
                if (call) {
                    call.setVoice();
                }
                break;
            case CallActions.Silence:
                call = this.callById(event.id);
                if (call) {
                    call.setSilence();
                }
                break;
            case CallActions.Hold:
                call = this.callById(event.id);
                if (call) {
                    call.setHold(event);
                }
                break;
            case CallActions.Hangup:
                call = this.callById(event.id);
                if (call) {
                    call.setHangup(event);
                }
                break;
            default:
                throw new Error('Unhandled action');
        }
        if (call) {
            this.eventHandler.emit(WEBSOCKET_EVENT_CALL, event.event, call);
            if (this.callDestroyed(call)) {
                this.destroyCall(call);
            }
        }
    }
    async handleChatEvents(event) {
        let conversation;
        const timestamp = Date.now(); // todo bug
        switch (event.action) {
            case ChatActions.UserInvite:
                const inv = event.data;
                conversation = new Conversation(this, inv.conversation_id, inv.title, inv.members, inv.messages, inv.variables);
                conversation.setInvite(inv.invite_id, timestamp);
                this.conversationStore.set(conversation.id, conversation);
                break;
            case ChatActions.Joined:
                const joined = event.data;
                conversation = this.conversationById(joined.conversation_id);
                if (conversation) {
                    conversation.setAnswered(joined.member.id, timestamp, joined.member);
                }
                break;
            case ChatActions.Message:
                const message = event.data;
                message.timestamp = timestamp;
                conversation = this.conversationById(message.conversation_id);
                if (conversation) {
                    conversation.newMessage(message);
                }
                break;
            case ChatActions.Close:
            case ChatActions.Leave:
            case ChatActions.Decline:
                // TODO leave & CLOSE:
                const e = event.data;
                conversation = this.conversationById(e.conversation_id);
                if (conversation) {
                    conversation.setClosed(timestamp);
                }
                break;
            default:
        }
        if (conversation) {
            this.eventHandler.emit(WEBSOCKET_EVENT_CHAT, event.action, conversation);
            if (this.conversationDestroyed(conversation)) {
                this.destroyConversation(conversation);
            }
        }
    }
    hasAgentTask(task) {
        if (!this.agent || !task) {
            return false;
        }
        return this.agent.hasTask(task);
    }
    destroyCall(call) {
        // FIXME sync channel & call event
        this.callStore.delete(call.id);
        this.eventHandler.emit(WEBSOCKET_EVENT_CALL, CallActions.Destroy, call);
    }
    destroyConversation(conv) {
        // FIXME sync channel & call event
        this.conversationStore.delete(conv.id);
        this.eventHandler.emit(WEBSOCKET_EVENT_CHAT, ChatActions.Destroy, conv);
    }
}

class SipClient extends EventEmitter {
}

var commonjsGlobal = typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

function unwrapExports (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x.default : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var ee = createCommonjsModule(function (module, exports) {
var __generator = (commonjsGlobal && commonjsGlobal.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.$listeners = Symbol('EventEmitter.listeners');
exports.$addListener = Symbol('EventEmitter.addListener');
/** Statically typed event emitter */
var EventEmitter$$1 = /** @class */ (function () {
    function EventEmitter$$1() {
        this[exports.$listeners] = {};
    }
    /** Count the number of listeners for an event */
    EventEmitter$$1.count = function (ee, key) {
        var count = 0;
        var list = ee[exports.$listeners][key];
        if (list) {
            var cb = list.first;
            while (++count) {
                if (cb.next) {
                    cb = cb.next;
                }
                else
                    break;
            }
        }
        return count;
    };
    /** Check if an event has listeners */
    EventEmitter$$1.has = function (ee, key) {
        if (key == '*') {
            for (key in ee[exports.$listeners])
                return true;
            return false;
        }
        return ee[exports.$listeners][key] !== undefined;
    };
    /** Get an array of event keys that have listeners */
    EventEmitter$$1.keys = function (ee) {
        return Object.keys(ee[exports.$listeners]);
    };
    /** Call the given listener when no other listeners exist */
    EventEmitter$$1.unhandle = function (ee, key, impl, disposables) {
        var listener = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            if (!ee[exports.$listeners][key].first.next)
                return impl.apply(void 0, args);
        };
        return ee.on(key, listener, disposables);
    };
    /** Implementation */
    EventEmitter$$1.prototype.on = function (arg, fn, disposables) {
        if (typeof fn == 'function') {
            return this[exports.$addListener](arg, fn, disposables);
        }
        return this[exports.$addListener](arg, undefined, fn);
    };
    /** Implementation */
    EventEmitter$$1.prototype.one = function (arg, fn, disposables) {
        if (typeof fn == 'function') {
            return this[exports.$addListener](arg, fn, disposables, true);
        }
        return this[exports.$addListener](arg, undefined, fn, true);
    };
    /** Implementation */
    EventEmitter$$1.prototype.off = function (arg, fn) {
        if (arg == '*') {
            var cache = this[exports.$listeners];
            this[exports.$listeners] = {};
            if (this._onEventUnhandled) {
                for (var key in cache) {
                    this._onEventUnhandled(key);
                }
            }
            return this;
        }
        if (typeof fn == 'function') {
            var list = this[exports.$listeners][arg];
            if (list && unlink(list, function (l) { return l.fn == fn; })) {
                return this;
            }
        }
        delete this[exports.$listeners][arg];
        if (this._onEventUnhandled) {
            this._onEventUnhandled(arg);
        }
        return this;
    };
    /** Implementation */
    EventEmitter$$1.prototype.emit = function (key) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var result;
        var gen = this.listeners(key);
        while (true) {
            var _a = gen.next(), listener = _a.value, done = _a.done;
            if (done) {
                return result;
            }
            else {
                var generated = listener.apply(void 0, args);
                if (generated !== undefined) {
                    result = generated;
                }
            }
        }
    };
    /** Iterate over the listeners of an event */
    EventEmitter$$1.prototype.listeners = function (key) {
        var list, prev, curr;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    list = this[exports.$listeners][key];
                    if (!list)
                        return [2 /*return*/];
                    prev = null;
                    curr = list.first;
                    _a.label = 1;
                case 1:
                    return [4 /*yield*/, curr.fn
                        // One-time listener
                    ];
                case 2:
                    _a.sent();
                    // One-time listener
                    if (curr.once) {
                        // Splice it.
                        if (prev) {
                            prev.next = curr.next;
                        }
                        // Shift it.
                        else if (curr.next) {
                            list.first = curr = curr.next;
                            return [3 /*break*/, 1];
                        }
                        // Delete it.
                        else {
                            delete this[exports.$listeners][key];
                            if (this._onEventUnhandled) {
                                this._onEventUnhandled(key);
                            }
                            return [2 /*return*/];
                        }
                    }
                    // Recurring listener
                    else {
                        prev = curr;
                    }
                    // Continue to the next listener.
                    if (curr.next) {
                        curr = curr.next;
                        return [3 /*break*/, 1];
                    }
                    // Update the last listener.
                    list.last = curr;
                    // All done.
                    return [2 /*return*/];
                case 3: return [2 /*return*/];
            }
        });
    };
    /** Implementation of the `on` and `one` methods */
    EventEmitter$$1.prototype[exports.$addListener] = function (arg, fn, disposables, once) {
        var _this = this;
        if (once === void 0) { once = false; }
        if (typeof arg == 'object') {
            var key_1;
            var _loop_1 = function () {
                if (typeof arg[key_1] == 'function') {
                    var fn_1 = arg[key_1];
                    var list = addListener(this_1[exports.$listeners], key_1, {
                        fn: fn_1,
                        once: once,
                        next: null,
                    });
                    if (disposables) {
                        disposables.push({
                            dispose: function () { return _this.off(key_1, fn_1); },
                        });
                    }
                    if (fn_1 == list.first.fn && this_1._onEventHandled) {
                        this_1._onEventHandled(key_1);
                    }
                }
            };
            var this_1 = this;
            for (key_1 in arg) {
                _loop_1();
            }
            return this;
        }
        if (typeof fn == 'function') {
            var key_2 = arg;
            var list = addListener(this[exports.$listeners], key_2, {
                fn: fn,
                once: once,
                next: null,
            });
            if (disposables) {
                disposables.push({
                    dispose: function () { return _this.off(key_2, fn); },
                });
            }
            if (fn == list.first.fn && this._onEventHandled) {
                this._onEventHandled(arg);
            }
        }
        return fn;
    };
    /** Unique symbol for accessing the internal listener cache */
    EventEmitter$$1.ev = exports.$listeners;
    return EventEmitter$$1;
}());
exports.EventEmitter = EventEmitter$$1;
function addListener(cache, key, cb) {
    var list = cache[key];
    if (list) {
        list.last.next = cb;
        list.last = cb;
    }
    else {
        cache[key] = list = { first: cb, last: cb };
    }
    return list;
}
/** Remove listeners that match the filter function */
function unlink(list, filter) {
    var prev = null;
    var curr = list.first;
    while (true) {
        // Return true to unlink the listener.
        if (filter(curr)) {
            // Splice it.
            if (prev) {
                prev.next = curr.next;
                if (curr.next) {
                    curr = curr.next;
                }
                else
                    break;
            }
            // Shift it.
            else if (curr.next) {
                list.first = curr = curr.next;
            }
            // No listeners remain.
            else {
                return null;
            }
        }
        // Keep this listener.
        else {
            prev = curr;
            if (curr.next) {
                curr = curr.next;
            }
            else
                break;
        }
    }
    // At least one listener remains.
    list.last = prev;
    return list;
}
});

unwrapExports(ee);
var ee_1 = ee.$listeners;
var ee_2 = ee.$addListener;
var ee_3 = ee.EventEmitter;

class Session$1 {
    constructor(client, id) {
        this.client = client;
        this.callId = id;
        this.incoming = false;
        this.instanceId = 'fixme';
    }
    get id() {
        return this.callId;
    }
    getLocalMedia() {
        return [];
    }
    getPeerMedia() {
        return [];
    }
    answer(s) {
        return new Promise((resolve) => {
            this.client.answer(this.id);
            resolve();
        });
    }
}

class ExternalClient extends ee_3 {
    constructor(config) {
        super();
        this.config = config;
        this.type = 'external';
        this.schema = 'wtel://';
    }
    async callOption(req) {
        return req;
    }
    async register(sipConf) {
        return;
    }
    async unregister() {
        return;
    }
    async call(req) {
        if (!req.destination) {
            return;
        }
        this.exec(req.destination);
    }
    answer(id) {
        this.exec(id, 'answer');
    }
    sipSessionByCallId(id) {
        return new Session$1(this, id);
    }
    sipSessionBySipId(id) {
        return null;
    }
    open(token) {
        this.exec(encodeURIComponent(JSON.stringify({
            token,
            ...this.config,
        })), 'open');
    }
    exec(args, action) {
        // TODO escape
        let url = `${this.schema}${args}`;
        if (action) {
            url += ` ${action}`;
        }
        const link = document.createElement('a');
        link.download = 'File name';
        link.href = url;
        link.click();
        return;
    }
}

// tslint:disable

export { AgentPauseCauseServiceApiAxiosParamCreator, AgentPauseCauseServiceApiFp, AgentPauseCauseServiceApiFactory, AgentPauseCauseServiceApi, AgentServiceApiAxiosParamCreator, AgentServiceApiFp, AgentServiceApiFactory, AgentServiceApi, AgentSkillServiceApiAxiosParamCreator, AgentSkillServiceApiFp, AgentSkillServiceApiFactory, AgentSkillServiceApi, AgentTeamServiceApiAxiosParamCreator, AgentTeamServiceApiFp, AgentTeamServiceApiFactory, AgentTeamServiceApi, BackendProfileServiceApiAxiosParamCreator, BackendProfileServiceApiFp, BackendProfileServiceApiFactory, BackendProfileServiceApi, BucketServiceApiAxiosParamCreator, BucketServiceApiFp, BucketServiceApiFactory, BucketServiceApi, CalendarServiceApiAxiosParamCreator, CalendarServiceApiFp, CalendarServiceApiFactory, CalendarServiceApi, CallServiceApiAxiosParamCreator, CallServiceApiFp, CallServiceApiFactory, CallServiceApi, CommunicationTypeServiceApiAxiosParamCreator, CommunicationTypeServiceApiFp, CommunicationTypeServiceApiFactory, CommunicationTypeServiceApi, EmailProfileServiceApiAxiosParamCreator, EmailProfileServiceApiFp, EmailProfileServiceApiFactory, EmailProfileServiceApi, ListServiceApiAxiosParamCreator, ListServiceApiFp, ListServiceApiFactory, ListServiceApi, MediaFileServiceApiAxiosParamCreator, MediaFileServiceApiFp, MediaFileServiceApiFactory, MediaFileServiceApi, MemberServiceApiAxiosParamCreator, MemberServiceApiFp, MemberServiceApiFactory, MemberServiceApi, OutboundResourceGroupServiceApiAxiosParamCreator, OutboundResourceGroupServiceApiFp, OutboundResourceGroupServiceApiFactory, OutboundResourceGroupServiceApi, OutboundResourceServiceApiAxiosParamCreator, OutboundResourceServiceApiFp, OutboundResourceServiceApiFactory, OutboundResourceServiceApi, QueueBucketServiceApiAxiosParamCreator, QueueBucketServiceApiFp, QueueBucketServiceApiFactory, QueueBucketServiceApi, QueueHookServiceApiAxiosParamCreator, QueueHookServiceApiFp, QueueHookServiceApiFactory, QueueHookServiceApi, QueueResourcesServiceApiAxiosParamCreator, QueueResourcesServiceApiFp, QueueResourcesServiceApiFactory, QueueResourcesServiceApi, QueueServiceApiAxiosParamCreator, QueueServiceApiFp, QueueServiceApiFactory, QueueServiceApi, QueueSkillServiceApiAxiosParamCreator, QueueSkillServiceApiFp, QueueSkillServiceApiFactory, QueueSkillServiceApi, RegionServiceApiAxiosParamCreator, RegionServiceApiFp, RegionServiceApiFactory, RegionServiceApi, RoutingOutboundCallServiceApiAxiosParamCreator, RoutingOutboundCallServiceApiFp, RoutingOutboundCallServiceApiFactory, RoutingOutboundCallServiceApi, RoutingSchemaServiceApiAxiosParamCreator, RoutingSchemaServiceApiFp, RoutingSchemaServiceApiFactory, RoutingSchemaServiceApi, RoutingVariableServiceApiAxiosParamCreator, RoutingVariableServiceApiFp, RoutingVariableServiceApiFactory, RoutingVariableServiceApi, SkillServiceApiAxiosParamCreator, SkillServiceApiFp, SkillServiceApiFactory, SkillServiceApi, UserHelperServiceApiAxiosParamCreator, UserHelperServiceApiFp, UserHelperServiceApiFactory, UserHelperServiceApi, Configuration, Response, Client, CallReportingStatus, CallActions, CallDirection, Call, AgentStatus, ChannelState, ChannelType, Agent, ChatActions, ConversationState, Conversation, SipClient, ExternalClient, SipPhone, Log };
//# sourceMappingURL=index.esm.js.map
